$bg_color: #0c0c0d;
$fg_color: #ffffff;
$bd_color: $bg_color;
$hover_bg_color:#3790FA;
$hover_fg_color:#fff;
$hover_bd_color:$hover_bg_color;
$active_bg_color: #3790FA;
$active_fg_color: #fff;
$active_bd_color: $active_bg_color;
$selected_bg_color: #3790FA;
$selected_fg_color: #fff;
$selected_borders_color: $selected_bg_color;
$disable_bg_color:#1C1C1E;
$disable_fg_color: rgba($fg_color,0.3);
$disable_bd_color: $disable_bg_color;
$backdrop_bg_color:#0c0c0d;
$backdrop_fg_color:#ffffff;
$backdrop_bd_color:$backdrop_bg_color;

/* All button color  */
$button_bg_color:#393A3E;
$button_fg_color:#fff;
$button_bd_color:$button_bg_color;
$bt_hover_bg_color:#3790FA;
$bt_hover_fg_color:#fff;
$bt_hover_bd_color:$bt_hover_bg_color;
$bt_active_bg_color:#3790FA;
$bt_active_fg_color:#fff;
$bt_active_bd_color:$bt_active_bg_color;
$bt_selected_bg_color: #3790FA;
$bt_selected_fg_color:#fff;
$bt_selected_borders_color: $selected_bg_color;
$bt_disable_bg_color: #1C1C1E ;
$bt_disable_fg_color:#474747;
$bt_disable_bd_color:$bt_disable_bg_color;
$bt_backdrop_bg_color:#1C1C1E;
$bt_backdrop_fg_color:#ffffff;
$bt_backdrop_bd_color:$bt_backdrop_bg_color;

/*For future colors can change*/
$change_bg_color:#2D2E32 ;
/*sp_bd_color*/
$sp_bd_color:#696969;
/* Text Entries */
$text_bg_color: #242424  ;

/**  The above are all the basic colors, it is recommended to adjust only these colors  **/

/*check and radio button color*/
$crb_bg_color:$bg_color;
$crb_fg_color: #ffffff;
$crb_bd_color:#48484D;
$crb_hover_bg_color: #2D418D  ;
$crb_hover_fg_color:#2D418D ;
$crb_hover_bd_color:  #6B8EEB;
$crb_active_bg_color: #3790FA ;
$crb_active_fg_color:#ffffff;
$crb_active_bd_color: #415FC4 ;
$crb_disable_bg_color: #1C1C1E  ;
$crb_disable_fg_color: #474747  ;
$crb_disable_bd_color: #1C1C1E ;
$crb_backdrop_bg_color:#1C1C1E;
$crb_backdrop_fg_color: #343438;
$crb_backdrop_bd_color: #1C1C1E;


/** Link **/
$link_fg_color: $fg_color;
$link_active_fg_color: $active_fg_color;
$link_visited_fg_color: $fg_color;
$link_selected_fg_color: $selected_fg_color;
$dark_fill: mix($bg_color, $bg_color, 50%);
$alt_borders_color: darken($bg_color, 35%);
$borders_edge: transparentize(black, 0.2);/*shadow*/
$top_hilight: $borders_edge;

/**popover**/
$popover_bg_color: $bg_color;
$warning_color: #d42320;
$error_color: #0C0C0D;
$success_color: $active_bg_color;
$destructive_color: #d42320;

/**osd**/
$osd_fg_color: $fg_color;
$osd_text_color: $fg_color;
$osd_bg_color: transparentize(#1c1f20, 0.3);
$osd_borders_color: transparentize(black, 0.3);

$shadow_color: transparentize(black, 0.9);
$drop_target_color: $bg_color;

//colors for the backdrop state, derived from the main colors.
$backdrop_base_color: $backdrop_bg_color;
$backdrop_text_color: $backdrop_fg_color;
$backdrop_selected_fg_color: $backdrop_base_color;
$backdrop_borders_color: 	$backdrop_bd_color;
$backdrop_dark_fill: $disable_bg_color;
$backdrop_sidebar_bg_color: mix($backdrop_bg_color, $backdrop_base_color, 50%);
$backdrop_scrollbar_bg_color: darken($backdrop_bg_color, 3%);
$backdrop_scrollbar_slider_color: mix($backdrop_fg_color, $backdrop_bg_color, 40%);
$backdrop_menu_color: $backdrop_base_color;
/*
 * Base color
 */
/* background colors in different states */
$base_normal_bg_color: $bg_color;
$base_active_bg_color: $active_bg_color;
$base_prelight_bg_color: $active_bg_color;
$base_selected_bg_color:  $active_bg_color;
$base_focused_bg_color: $bg_color;
$base_backdrop_bg_color: $backdrop_bg_color;
$base_checked_bg_color: $base_active_bg_color;

/* foreground colors in different states */
$base_normal_fg_color: $fg_color;
$base_active_fg_color: $active_fg_color;
$base_prelight_fg_color: #3790FA;
$base_selected_fg_color: $active_fg_color;
$base_focused_fg_color: $fg_color;
$base_backdrop_fg_color: $backdrop_fg_color;
$base_checked_fg_color: $fg_color;

/* border color in different states */
$base_normal_border_color: $backdrop_borders_color ;
$base_active_border_color: $active_bd_color;
$base_prelight_border_color: $active_bd_color;
$base_selected_border_color: $backdrop_borders_color;
$base_focused_border_color: $backdrop_borders_color;
$base_backdrop_border_color: $backdrop_bd_color;
$base_checked_border_color: $backdrop_borders_color;

/*
 * background image in different states.
 * The color value of bg_image is the same as the bg_color
 */
$base_normal_bg_image: image($bg_color);
$base_active_bg_image: image($base_active_bg_color);
$base_prelight_bg_image: image($base_prelight_bg_color);
$base_insensitive_bg_image: image($bg_color);
$base_focused_bg_image: image($base_focused_bg_color);
$base_backdrop_bg_image: image($base_backdrop_bg_color);
$base_backdrop_sensitive_bg_image: image($bg_color);
$base_backdrop_insensitive_bg_image: image($bg_color);
$base_checked_bg_image: image($base_checked_bg_color);

/*
 * ComboBox
 */
$combobox_button_normal_bg_image: image(#2C2C2E);
/* ComboBox dropdown colors */
$combobox_dropdown_menuitem_normal_bg_color: #1C1C1E;

/*
 * Notebooks
 */
$notebook_tab_normal_bg_color: $change_bg_color;
$notebook_tab_checked_bg_color: $bg_color;
$notebook_tab_closebutton_normal_bg_image: image($change_bg_color);
$notebook_tab_closebutton_active_bg_image: image(#cccccc);
$notebook_tab_closebutton_checked_bg_image: image($notebook_tab_checked_bg_color);

/*
 * Scrollbar
 */
$scrollbar_bg_color: mix($bg_color, $fg_color, 80%);
$scrollbar_normal_bg_color: $bg_color;
$scrollbar_prelight_bg_color: #303033;
$scrollbar_free_space_color: #343438;
$scrollbar_slider_normal_color: #3D3D41;
$scrollbar_slider_active_color: #85858C;
$scrollbar_slider_prelight_color: #616166;
$scrollbar_slider_insensitive_color: $disable_bg_color;

/*
 * Scale & Progressbar
 */
$trough_free_space_normal_color: #f8f8f8;
$trough_free_space_insensitive_color: $disable_bg_color;
$trough_filled_space_normal_color: $active_bg_color;
$trough_filled_space_insensitive_color: $disable_bg_color;
$trough_border_normal_color: $bd_color;


/*
 * Frame
 */
$frame_label_normal_fg_color: $base_normal_fg_color;
$frame_border_normal_color: $base_normal_border_color;


/*
 * Treeview
 */
$treeview_view_selected_bg_color: #18181A;

/*
*ToolBar
*/
$toolbar_button_bg_color:$change_bg_color;

/*
 * Headerbar
 */
$headerbar_color: lighten($bg_color, 5%);
$headerbar_normal_fg_color: $fg_color;
$headerbar_backdrop_fg_color: $fg_color;
$headerbar_normal_bg_image: image($bg_color);
$headerbar_backdrop_bg_image: image($bg_color);
$headerbar_button_normal_fg_color: $fg_color;
$headerbar_button_backdrop_fg_color: $fg_color;
$headerbar_button_prelight_fg_color: $active_fg_color;
$headerbar_button_active_fg_color: $active_fg_color;
$headerbar_button_normal_bg_image: image($button_bg_color);
$headerbar_button_backdrop_bg_image: image($bg_color);
$headerbar_button_prelight_bg_image: image(#545d64);
$headerbar_button_active_bg_image: image(#434d55);
$headerbar_closebutton_prelight_bg_image: image(#d42320);
$headerbar_closebutton_active_bg_image: image(#ac4844);
$headerbar_normal_border_color: #0C0C0D;
$headerbar_backdrop_border_color: #0C0C0D;


/*
 * menu
 */
$menu_color:#3D3E42;
$menuitem_bg_color: $menu_color;
$menuitem_insensitive_bg_color:$menu_color;

