//apps rely on some named colors to be exported
/* GTK NAMED COLORS
   ----------------
   use responsibly! */

// Sass thinks we're using the colors in the variables as strings and may shoot
// warning, it's innocuous and can be defeated by using "" + $var


/*mate-terminal background color*/
@define-color text_view_bg #{"" + $base_normal_bg_color};

/*
widget text/foreground color */
@define-color theme_fg_color #{"" + $base_normal_fg_color};

/*
text color for entries, views and content in general */
@define-color theme_text_color #{"" + $base_normal_fg_color};

/*
widget base background color */
@define-color theme_bg_color #{"" + $base_normal_bg_color};

/*
text widgets and the like base background color */
@define-color theme_base_color #{"" + $base_normal_bg_color};

/*
base background color of selections */
@define-color theme_selected_bg_color #{"" + $base_selected_bg_color};

/*
text/foreground color of selections */
@define-color theme_selected_fg_color #{"" + $base_selected_fg_color};

/*
base background color of insensitive widgets */
@define-color insensitive_bg_color #{"" + $disable_bg_color};

/*
text foreground color of insensitive widgets */
@define-color insensitive_fg_color #{"" + $disable_fg_color};

/*
insensitive text widgets and the like base background color */
@define-color insensitive_base_color #{"" + $disable_bg_color};

/*
widget text/foreground color on backdrop windows */
@define-color theme_unfocused_fg_color #{"" + $base_backdrop_fg_color};

/*
text color for entries, views and content in general on backdrop windows */
@define-color theme_unfocused_text_color #{"" + $base_normal_fg_color};

/*
widget base background color on backdrop windows */
@define-color theme_unfocused_bg_color #{"" + $base_backdrop_bg_color};

/*
text widgets and the like base background color on backdrop windows */
@define-color theme_unfocused_base_color #{"" + $base_backdrop_bg_color};

/*
base background color of selections on backdrop windows */
@define-color theme_unfocused_selected_bg_color #{"" + $base_selected_bg_color};

/*
text/foreground color of selections on backdrop windows */
@define-color theme_unfocused_selected_fg_color #{"" + $base_selected_fg_color};

/*
insensitive color on backdrop windows*/
@define-color unfocused_insensitive_color #{"" + $base_backdrop_bg_color};

/*
widgets main borders color */
@define-color borders #{"" + $sp_bd_color};

/*
widgets main borders color on backdrop windows */
@define-color unfocused_borders #{"" + $base_backdrop_border_color};

/*
these are pretty self explicative */
@define-color warning_color #{"" + $warning_color};

@define-color error_color #{"" + $error_color};

@define-color success_color #{"" + $success_color};

@define-color destructive_color #{$destructive_color};

//WM

$_wm_highlight: $top_hilight;

// Sass gets mad if this is done directly in the color definition

/*
these colors are exported for the window manager and shouldn't be used in applications,
read if you used those and something break with a version upgrade you're on your own... */
@define-color wm_title #{"" + $fg_color};

//@define-color wm_unfocused_title #{$backdrop_fg_color};

@define-color wm_highlight #{"" + $_wm_highlight};

@define-color wm_borders_edge #{"" + $borders_edge};

@define-color wm_bg_a #{"" + $bg_color};

@define-color wm_bg_b #{"" + $bg_color};

//@define-color wm_shadow alpha(black, 0.35);

@define-color wm_border alpha(black, 0.18);

@define-color wm_button_hover_color_a shade(#{$active_bg_color}, 1.3);

@define-color wm_button_hover_color_b #{$active_bg_color};

@define-color wm_button_active_color_a shade(#{$active_bg_color}, 0.85);

@define-color wm_button_active_color_b shade(#{$active_bg_color}, 0.89);

@define-color wm_button_active_color_c shade(#{$active_bg_color}, 0.9);

//FIXME this is really an API

@define-color content_view_bg #{"" + $bg_color};
