// I think these rules can override the default rules in GTK+'s Adwaita theme
$variant: 'light';

@import 'ukui-colors';
@import 'ukui-drawing';
@import 'colors-public';
@import 'common';
@import 'ukui-common';


