/* All button color  */
/*For future colors can change*/
/*sp_bd_color*/
/* Text Entries */
/**  The above are all the basic colors, it is recommended to adjust only these colors  **/
/*check and radio button color*/
/** Link **/
/*shadow*/
/**popover**/
/**osd**/
/*
 * Base color
 */
/* background colors in different states */
/* foreground colors in different states */
/* border color in different states */
/*
 * background image in different states.
 * The color value of bg_image is the same as the bg_color
 */
/*
 * ComboBox
 */
/* ComboBox dropdown colors */
/*
 * Notebooks
 */
/*
 * Scrollbar
 */
/*
 * Scale & Progressbar
 */
/*
 * Frame
 */
/*
 * Treeview
 */
/*
*ToolBar
*/
/*
 * Headerbar
 */
/*
 * menu
 */
/* GTK NAMED COLORS
   ----------------
   use responsibly! */
/*mate-terminal background color*/
@define-color text_view_bg #0c0c0d;
/*
widget text/foreground color */
@define-color theme_fg_color #ffffff;
/*
text color for entries, views and content in general */
@define-color theme_text_color #ffffff;
/*
widget base background color */
@define-color theme_bg_color #0c0c0d;
/*
text widgets and the like base background color */
@define-color theme_base_color #0c0c0d;
/*
base background color of selections */
@define-color theme_selected_bg_color #3790FA;
/*
text/foreground color of selections */
@define-color theme_selected_fg_color #fff;
/*
base background color of insensitive widgets */
@define-color insensitive_bg_color #1C1C1E;
/*
text foreground color of insensitive widgets */
@define-color insensitive_fg_color rgba(255, 255, 255, 0.3);
/*
insensitive text widgets and the like base background color */
@define-color insensitive_base_color #1C1C1E;
/*
widget text/foreground color on backdrop windows */
@define-color theme_unfocused_fg_color #ffffff;
/*
text color for entries, views and content in general on backdrop windows */
@define-color theme_unfocused_text_color #ffffff;
/*
widget base background color on backdrop windows */
@define-color theme_unfocused_bg_color #0c0c0d;
/*
text widgets and the like base background color on backdrop windows */
@define-color theme_unfocused_base_color #0c0c0d;
/*
base background color of selections on backdrop windows */
@define-color theme_unfocused_selected_bg_color #3790FA;
/*
text/foreground color of selections on backdrop windows */
@define-color theme_unfocused_selected_fg_color #fff;
/*
insensitive color on backdrop windows*/
@define-color unfocused_insensitive_color #0c0c0d;
/*
widgets main borders color */
@define-color borders #696969;
/*
widgets main borders color on backdrop windows */
@define-color unfocused_borders #0c0c0d;
/*
these are pretty self explicative */
@define-color warning_color #d42320;
@define-color error_color #0C0C0D;
@define-color success_color #3790FA;
@define-color destructive_color #d42320;
/*
these colors are exported for the window manager and shouldn't be used in applications,
read if you used those and something break with a version upgrade you're on your own... */
@define-color wm_title #ffffff;
@define-color wm_highlight rgba(0, 0, 0, 0.8);
@define-color wm_borders_edge rgba(0, 0, 0, 0.8);
@define-color wm_bg_a #0c0c0d;
@define-color wm_bg_b #0c0c0d;
@define-color wm_border alpha(black, 0.18);
@define-color wm_button_hover_color_a shade(#3790FA, 1.3);
@define-color wm_button_hover_color_b #3790FA;
@define-color wm_button_active_color_a shade(#3790FA, 0.85);
@define-color wm_button_active_color_b shade(#3790FA, 0.89);
@define-color wm_button_active_color_c shade(#3790FA, 0.9);
@define-color content_view_bg #0c0c0d;
*, .selection-mode.titlebar:not(headerbar) .subtitle:link,
.selection-mode.titlebar:not(headerbar) button.subtitle:visited,
headerbar.selection-mode .subtitle:link,
headerbar.selection-mode button.subtitle:visited, button:link,
button:visited {
  padding: 0;
  -GtkToolButton-icon-spacing: 4;
  -GtkTextView-error-underline-color: #0C0C0D;
  -GtkScrolledWindow-scrollbar-spacing: 0;
  -GtkToolItemGroup-expander-size: 0;
  -GtkWidget-text-handle-width: 20;
  -GtkWidget-text-handle-height: 20;
  -GtkDialog-button-spacing: 5;
  -GtkDialog-action-area-border: 0;
  outline-color: alpha(currentColor,0.3);
  outline-style: dashed;
  outline-offset: -3px;
  outline-width: 0px;
  -gtk-outline-radius: 2px;
  -gtk-secondary-caret-color: #3790FA; }

/******************
 * Base States *
 ******************/
.background {
  background-color: #0c0c0d;
  color: #ffffff; }
  .background:backdrop, label:backdrop selection.background, progressbar:backdrop trough.background, progressbar:backdrop progress.background {
    color: #ffffff;
    -gtk-icon-shadow: none; }

.background.rubberband {
  background-color: transparent; }

*:disabled, label:disabled selection, label:disabled .selection-mode.titlebar:not(headerbar) selection.subtitle:link, .selection-mode.titlebar:not(headerbar) label:disabled selection.subtitle:link, .selection-mode.titlebar:not(headerbar) .subtitle:disabled:link,
.selection-mode.titlebar:not(headerbar) button.subtitle:disabled:visited,
headerbar.selection-mode .subtitle:disabled:link,
headerbar.selection-mode label:disabled selection.subtitle:link,
label:disabled headerbar.selection-mode selection.subtitle:link,
headerbar.selection-mode button.subtitle:disabled:visited, button:disabled:link,
button:disabled:visited {
  -gtk-icon-effect: dim; }

.gtkstyle-fallback {
  color: #ffffff;
  background-color: #0c0c0d; }
  .gtkstyle-fallback:hover {
    color: #fff;
    background-color: #3790FA; }
  .gtkstyle-fallback:active {
    color: #fff;
    background-color: #3790FA; }
  .gtkstyle-fallback:disabled, label:disabled selection.gtkstyle-fallback {
    color: rgba(255, 255, 255, 0.3);
    background-color: #1C1C1E; }
  .gtkstyle-fallback:selected, .selection-mode.titlebar:not(headerbar) .gtkstyle-fallback.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.gtkstyle-fallback.subtitle:visited,
  headerbar.selection-mode .gtkstyle-fallback.subtitle:link,
  headerbar.selection-mode button.gtkstyle-fallback.subtitle:visited {
    color: #fff;
    background-color: #3790FA; }

.view, iconview,
.view text,
iconview text,
textview text {
  color: #ffffff;
  background-color: #0c0c0d; }
  .view:backdrop, iconview:backdrop, label:backdrop selection.view, progressbar:backdrop trough.view, progressbar:backdrop progress.view,
  .view text:backdrop,
  iconview text:backdrop,
  textview text:backdrop {
    color: #ffffff;
    background-color: #0c0c0d; }
    .view:backdrop:disabled, iconview:backdrop:disabled, label:disabled selection.view:backdrop, label:disabled label:backdrop selection.view, label:backdrop label:disabled selection.view, label:backdrop selection.view:disabled, progressbar:backdrop trough.view:disabled, progressbar:backdrop progress.view:disabled,
    .view text:backdrop:disabled,
    iconview text:backdrop:disabled,
    textview text:backdrop:disabled {
      color: rgba(255, 255, 255, 0.3); }
  .view:disabled, iconview:disabled, label:disabled selection.view,
  .view text:disabled,
  iconview text:disabled,
  textview text:disabled {
    color: rgba(255, 255, 255, 0.3);
    background-color: #1C1C1E; }

textview border {
  background-color: #0c0c0d; }

.rubberband,
rubberband,
flowbox rubberband,
.content-view rubberband,
.content-view .rubberband,
treeview.view rubberband {
  background-color: rgba(6, 116, 248, 0.2);
  background-image: linear-gradient(to bottom, rgba(55, 144, 250, 0.35), rgba(55, 144, 250, 0.35)); }

flowbox flowboxchild {
  padding: 3px; }
  flowbox flowboxchild:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:link,
  flowbox headerbar.selection-mode flowboxchild.subtitle:link,
  headerbar.selection-mode flowbox flowboxchild.subtitle:link {
    outline-offset: -2px; }

.content-view .tile {
  margin: 2px;
  background-color: transparent;
  padding: 0; }
  .content-view .tile:backdrop, .content-view label:backdrop selection.tile, label:backdrop .content-view selection.tile, .content-view progressbar:backdrop trough.tile, progressbar:backdrop .content-view trough.tile, .content-view progressbar:backdrop progress.tile, progressbar:backdrop .content-view progress.tile {
    background-color: transparent; }
  .content-view .tile:active, .content-view .tile:selected, .content-view .selection-mode.titlebar:not(headerbar) .tile.subtitle:link, .selection-mode.titlebar:not(headerbar) .content-view .tile.subtitle:link,
  .content-view .selection-mode.titlebar:not(headerbar) button.tile.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) .content-view button.tile.subtitle:visited,
  .content-view headerbar.selection-mode .tile.subtitle:link,
  headerbar.selection-mode .content-view .tile.subtitle:link,
  .content-view headerbar.selection-mode button.tile.subtitle:visited,
  headerbar.selection-mode .content-view button.tile.subtitle:visited {
    background-color: transparent; }
  .content-view .tile:disabled, .content-view label:disabled selection.tile, label:disabled .content-view selection.tile {
    background-color: #1C1C1E; }

label {
  caret-color: currentColor; }
  label.separator {
    color: #ffffff; }
    label.separator:backdrop {
      color: #ffffff; }
  label selection {
    background-color: #3790FA;
    color: #fff; }
  label:disabled {
    color: #C0C0C0; }
    label:disabled:backdrop {
      color: rgba(255, 255, 255, 0.3); }
  label:backdrop {
    color: #ffffff; }

.dim-label, label.separator, .titlebar:not(headerbar) .subtitle,
headerbar .subtitle {
  opacity: 0.55;
  text-shadow: none; }

assistant .sidebar {
  background-color: #0c0c0d; }
  assistant .sidebar:backdrop, assistant label:backdrop selection.sidebar, label:backdrop assistant selection.sidebar, assistant progressbar:backdrop trough.sidebar, progressbar:backdrop assistant trough.sidebar, assistant progressbar:backdrop progress.sidebar, progressbar:backdrop assistant progress.sidebar {
    background-color: #0c0c0d; }
assistant.csd .sidebar {
  border-top-style: none; }
assistant .sidebar label {
  padding: 6px 12px; }
assistant .sidebar label.highlight {
  background-color: #3d3d3d; }

.csd popover.background.touch-selection, .csd popover.background.magnifier, popover.background.touch-selection, popover.background.magnifier, .csd popover.background.osd, popover.background.osd, .app-notification,
.app-notification.frame, .osd .scale-popup,
.osd {
  color: #ffffff;
  border: none;
  border-radius: 4px;
  background-color: rgba(28, 31, 32, 0.7);
  background-clip: padding-box;
  text-shadow: none;
  -gtk-icon-shadow: none; }
  label:backdrop selection.app-notification, label:backdrop .osd selection.scale-popup, .osd label:backdrop selection.scale-popup, popover.background.touch-selection:backdrop, popover.background.magnifier:backdrop, popover.background.osd:backdrop, progressbar:backdrop trough.app-notification, progressbar:backdrop .osd trough.scale-popup, .osd progressbar:backdrop trough.scale-popup, progressbar:backdrop progress.app-notification, progressbar:backdrop .osd progress.scale-popup, .osd progressbar:backdrop progress.scale-popup, .app-notification:backdrop, .osd .scale-popup:backdrop,
  .osd:backdrop,
  label:backdrop selection.osd,
  progressbar:backdrop trough.osd,
  progressbar:backdrop progress.osd {
    text-shadow: none;
    -gtk-icon-shadow: none; }

/****************************
 * Spinner Animation*
 ****************************/
@keyframes spin {
  to {
    -gtk-icon-transform: rotate(1turn); } }
spinner {
  background: none;
  opacity: 0;
  -gtk-icon-source: -gtk-icontheme("process-working-symbolic"); }
  spinner:checked {
    opacity: 1;
    animation: spin 1s linear infinite; }
    spinner:checked:disabled {
      opacity: 0.5; }

/******************
 * Text Entries*
 ******************/
spinbutton:not(.vertical), entry {
  min-height: 32px;
  padding-left: 8px;
  padding-right: 8px;
  border: 1px;
  border-radius: 4px;
  border-color: #0c0c0d;
  transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  background-color: #242424; }
  spinbutton:not(.vertical) image.left,
  entry image.left {
    margin-right: 6px; }
  spinbutton:not(.vertical) image.right,
  entry image.right {
    margin-left: 6px; }
  spinbutton:not(.vertical) undershoot.left,
  entry undershoot.left {
    background-color: transparent;
    background-image: linear-gradient(to top, rgba(255, 255, 255, 0.2) 50%, rgba(255, 255, 255, 0.2) 50%);
    padding-left: 1px;
    background-size: 1px 10px;
    background-repeat: repeat-y;
    background-origin: content-box;
    background-position: left center;
    border: none;
    box-shadow: none; }
  spinbutton:not(.vertical) undershoot.right,
  entry undershoot.right {
    background-color: transparent;
    background-image: linear-gradient(to top, rgba(255, 255, 255, 0.2) 50%, rgba(255, 255, 255, 0.2) 50%);
    padding-right: 1px;
    background-size: 1px 10px;
    background-repeat: repeat-y;
    background-origin: content-box;
    background-position: right center;
    border: none;
    box-shadow: none; }
  spinbutton.flat:focus:not(.vertical), spinbutton.flat:backdrop:not(.vertical), spinbutton.flat:disabled:not(.vertical), spinbutton.flat:backdrop:disabled:not(.vertical), spinbutton.flat:not(.vertical),
  entry.flat:focus,
  entry.flat:backdrop,
  entry.flat:disabled,
  entry.flat:backdrop:disabled,
  entry.flat {
    min-height: 0;
    padding: 2px;
    background-color: #1C1C1E;
    border-color: transparent;
    border-radius: 0; }
  spinbutton:hover:not(.vertical),
  entry:hover {
    border-color: #3790FA; }
  spinbutton:focus:not(.vertical),
  entry:focus {
    border-color: #0c0c0d; }
  spinbutton:disabled:not(.vertical),
  entry:disabled {
    color: rgba(255, 255, 255, 0.3);
    border-color: #1C1C1E;
    background-color: #1C1C1E;
    box-shadow: none; }
  spinbutton:backdrop:not(.vertical),
  entry:backdrop {
    color: #ffffff;
    border-color: #0c0c0d;
    background-color: #0c0c0d;
    box-shadow: none;
    background-color: #242424;
    transition: 200ms ease-out; }
  spinbutton:backdrop:disabled:not(.vertical),
  entry:backdrop:disabled {
    color: rgba(255, 255, 255, 0.3);
    border-color: #1C1C1E;
    background-color: #1C1C1E;
    box-shadow: none; }
  spinbutton.error:not(.vertical),
  entry.error {
    color: #0C0C0D;
    border-color: #0C0C0D; }
    spinbutton.error:focus:not(.vertical),
    entry.error:focus {
      border-color: #0c0c0d; }
    spinbutton.error:not(.vertical) selection,
    entry.error selection {
      background-color: #0C0C0D; }
  spinbutton.warning:not(.vertical),
  entry.warning {
    color: #d42320;
    border-color: #d42320; }
    spinbutton.warning:focus:not(.vertical),
    entry.warning:focus {
      border-color: #0c0c0d; }
    spinbutton.warning:not(.vertical) selection,
    entry.warning selection {
      background-color: #d42320; }
  spinbutton:not(.vertical) image,
  entry image {
    color: #cececf; }
    spinbutton:not(.vertical) image:hover,
    entry image:hover {
      color: #fff; }
    spinbutton:not(.vertical) image:active,
    entry image:active {
      color: #3790FA; }
    spinbutton:not(.vertical) image:backdrop,
    entry image:backdrop {
      color: #cececf; }
  spinbutton:drop(active):focus:not(.vertical), spinbutton:drop(active):not(.vertical),
  entry:drop(active):focus,
  entry:drop(active) {
    border-color: #0c0c0d;
    box-shadow: inset 0 0 0 1px #0c0c0d; }
  .osd spinbutton:not(.vertical),
  .osd entry {
    color: #ffffff;
    border-color: rgba(0, 0, 0, 0.7);
    background-color: rgba(0, 0, 0, 0.5);
    background-clip: padding-box;
    box-shadow: none;
    text-shadow: 0 1px white;
    -gtk-icon-shadow: 0 1px white; }
    .osd spinbutton:focus:not(.vertical),
    .osd entry:focus {
      color: #ffffff;
      border-color: #3790FA;
      background-color: rgba(0, 0, 0, 0.5);
      background-clip: padding-box;
      box-shadow: inset 0 0 0 1px #3790FA;
      text-shadow: 0 1px white;
      -gtk-icon-shadow: 0 1px white; }
    .osd spinbutton:backdrop:not(.vertical),
    .osd entry:backdrop {
      color: #ffffff;
      border-color: rgba(0, 0, 0, 0.7);
      background-color: rgba(0, 0, 0, 0.5);
      background-clip: padding-box;
      box-shadow: none;
      text-shadow: none;
      -gtk-icon-shadow: none; }
    .osd spinbutton:disabled:not(.vertical),
    .osd entry:disabled {
      color: rgba(255, 255, 255, 0.3);
      border-color: rgba(0, 0, 0, 0.7);
      background-color: #1C1C1E;
      background-clip: padding-box;
      box-shadow: none;
      text-shadow: none;
      -gtk-icon-shadow: none; }
spinbutton:not(.vertical) progress,
entry progress {
  margin: 2px -6px;
  background-color: transparent;
  background-image: none;
  border-radius: 0;
  border-width: 0 0 2px;
  border-color: #3790FA;
  border-style: solid;
  box-shadow: none; }
  spinbutton:not(.vertical) progress:backdrop, spinbutton:not(.vertical) progressbar:backdrop progress, progressbar:backdrop spinbutton:not(.vertical) progress,
  entry progress:backdrop,
  entry progressbar:backdrop progress,
  progressbar:backdrop entry progress {
    background-color: transparent; }
.linked:not(.vertical) > spinbutton:focus:not(.vertical) + spinbutton:not(.vertical), .linked:not(.vertical) > spinbutton:focus:not(.vertical) + button, .linked:not(.vertical) > spinbutton:focus:not(.vertical) + combobox > box > button.combo, .linked:not(.vertical) >
entry:focus + spinbutton:not(.vertical), .linked:not(.vertical) >
entry:focus + button, .linked:not(.vertical) >
entry:focus + combobox > box > button.combo, .linked:not(.vertical) > spinbutton:focus:not(.vertical) +
entry, .linked:not(.vertical) >
entry:focus +
entry {
  border-left-color: #3790FA; }
.linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + spinbutton:not(.vertical), .linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + button, .linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + combobox > box > button.combo, .linked:not(.vertical) >
entry:drop(active) + spinbutton:not(.vertical), .linked:not(.vertical) >
entry:drop(active) + button, .linked:not(.vertical) >
entry:drop(active) + combobox > box > button.combo, .linked:not(.vertical) > spinbutton:drop(active):not(.vertical) +
entry, .linked:not(.vertical) >
entry:drop(active) +
entry {
  border-left-color: #0c0c0d; }
.linked.vertical > spinbutton:not(:disabled):not(.vertical) + entry:not(:disabled), .linked.vertical > spinbutton:not(:disabled):not(.vertical) + spinbutton:not(:disabled):not(.vertical), .linked.vertical >
entry:not(:disabled) + entry:not(:disabled), .linked.vertical >
entry:not(:disabled) + spinbutton:not(:disabled):not(.vertical) {
  border-top-color: #0c0c0d; }
  .linked.vertical > spinbutton:not(:disabled):not(.vertical) + entry:not(:disabled):backdrop, .linked.vertical > spinbutton:not(:disabled):not(.vertical) + spinbutton:not(:disabled):backdrop:not(.vertical), .linked.vertical >
  entry:not(:disabled) + entry:not(:disabled):backdrop, .linked.vertical >
  entry:not(:disabled) + spinbutton:not(:disabled):backdrop:not(.vertical) {
    border-top-color: #0c0c0d; }
.linked.vertical > spinbutton:disabled:not(.vertical) + spinbutton:disabled:not(.vertical), .linked.vertical > spinbutton:disabled:not(.vertical) + entry:disabled, .linked.vertical >
entry:disabled + spinbutton:disabled:not(.vertical), .linked.vertical >
entry:disabled + entry:disabled {
  border-top-color: #0c0c0d; }
.linked.vertical > spinbutton:not(.vertical) + spinbutton:focus:not(:only-child):not(.vertical),
.linked.vertical > spinbutton:not(.vertical) + entry:focus:not(:only-child), .linked.vertical >
entry + spinbutton:focus:not(:only-child):not(.vertical),
.linked.vertical >
entry + entry:focus:not(:only-child) {
  border-top-color: #3790FA; }
.linked.vertical > spinbutton:not(.vertical) + spinbutton:drop(active):not(:only-child):not(.vertical),
.linked.vertical > spinbutton:not(.vertical) + entry:drop(active):not(:only-child), .linked.vertical >
entry + spinbutton:drop(active):not(:only-child):not(.vertical),
.linked.vertical >
entry + entry:drop(active):not(:only-child) {
  border-top-color: #0c0c0d; }
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + spinbutton:not(.vertical),
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + entry,
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + button,
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + combobox > box > button.combo, .linked.vertical >
entry:focus:not(:only-child) + spinbutton:not(.vertical),
.linked.vertical >
entry:focus:not(:only-child) + entry,
.linked.vertical >
entry:focus:not(:only-child) + button,
.linked.vertical >
entry:focus:not(:only-child) + combobox > box > button.combo {
  border-top-color: #3790FA; }
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + spinbutton:not(.vertical),
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + entry,
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + button,
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + combobox > box > button.combo, .linked.vertical >
entry:drop(active):not(:only-child) + spinbutton:not(.vertical),
.linked.vertical >
entry:drop(active):not(:only-child) + entry,
.linked.vertical >
entry:drop(active):not(:only-child) + button,
.linked.vertical >
entry:drop(active):not(:only-child) + combobox > box > button.combo {
  border-top-color: #0c0c0d; }

treeview entry:focus:dir(rtl), treeview entry:focus:dir(ltr) {
  background-color: #0c0c0d;
  transition-property: color, background; }
treeview entry.flat, treeview entry {
  border-radius: 0;
  background-image: none;
  background-color: #0c0c0d; }
  treeview entry.flat:focus, treeview entry:focus {
    border-color: #3790FA; }

.entry-tag {
  padding: 5px;
  margin-top: 2px;
  margin-bottom: 2px;
  border-style: none;
  color: #fff;
  background-color: #3790FA; }
  :dir(ltr) .entry-tag {
    margin-left: 8px;
    margin-right: -5px; }
  :dir(rtl) .entry-tag {
    margin-left: -5px;
    margin-right: 8px; }
  .entry-tag:hover {
    background-color: #69acfb; }
  :backdrop .entry-tag {
    color: #0c0c0d;
    background-color: #3790FA; }
  .entry-tag.button {
    background-color: transparent;
    color: rgba(255, 255, 255, 0.7); }
  :not(:backdrop) .entry-tag.button:hover {
    border: 1px solid #3790FA;
    color: #fff; }
  :not(:backdrop) .entry-tag.button:active {
    background-color: #3790FA;
    color: rgba(255, 255, 255, 0.7); }

/*************
 * Buttons*
 *************/
@keyframes needs_attention {
  from {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.01, to(#3790FA), to(transparent)); }
  to {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(#3790FA), to(transparent)); } }
combobox:drop(active) button.combo, notebook > header > tabs > arrow, button.sidebar-button, button.titlebutton, .selection-mode button.titlebutton,
button {
  min-height: 24px;
  min-width: 16px;
  padding: 4px 8px;
  border-radius: 4px;
  transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  color: #ffffff;
  outline-color: rgba(255, 255, 255, 0.3);
  border-color: #0c0c0d;
  background-image: image(#0c0c0d); }
  combobox:drop(active) button.flat.combo, combobox:drop(active) button.combo.sidebar-button, combobox:drop(active) button.combo.titlebutton, notebook > header > tabs > arrow, button.sidebar-button, button.titlebutton, .selection-mode button.titlebutton, .selection-mode button.titlebutton,
  button.flat,
  button.sidebar-button,
  button.titlebutton,
  .selection-mode button.titlebutton {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none;
    transition: none; }
    combobox:drop(active) button.flat.combo:hover, combobox:drop(active) button.combo.sidebar-button:hover, combobox:drop(active) button.combo.titlebutton:hover, notebook > header > tabs > arrow:hover, button.sidebar-button:hover, button.titlebutton:hover, .selection-mode button.titlebutton:hover,
    button.flat:hover,
    button.sidebar-button:hover,
    button.titlebutton:hover {
      transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
      transition-duration: 500ms; }
      combobox:drop(active) button.flat.combo:hover:active, notebook > header > tabs > arrow:hover:active, button.sidebar-button:hover:active, button.titlebutton:hover:active, .selection-mode button.titlebutton:hover:active,
      button.flat:hover:active,
      button.sidebar-button:hover:active,
      button.titlebutton:hover:active {
        transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94); }
  combobox:drop(active) button.combo:hover, notebook > header > tabs > arrow:hover, button.sidebar-button:hover, button.titlebutton:hover, .selection-mode button.titlebutton:hover,
  button:hover {
    color: #ffffff;
    outline-color: rgba(255, 255, 255, 0.3);
    border-color: #0c0c0d;
    background-image: image(#0c0c0d);
    -gtk-icon-effect: highlight; }
  combobox:drop(active) button.combo:active, notebook > header > tabs > arrow:active, button.sidebar-button:active, button.titlebutton:active, combobox:drop(active) button.combo:checked, notebook > header > tabs > arrow:checked, button.sidebar-button:checked, button.titlebutton:checked, .selection-mode button.titlebutton:active, .selection-mode button.titlebutton:checked,
  button:active,
  button:checked {
    color: #ffffff;
    outline-color: rgba(255, 255, 255, 0.3);
    border-color: #0c0c0d;
    background-image: image(#0c0c0d);
    text-shadow: none;
    -gtk-icon-shadow: none;
    transition-duration: 50ms; }
  combobox:drop(active) button.combo:backdrop.flat, combobox:drop(active) button.combo.sidebar-button:backdrop, combobox:drop(active) button.combo.titlebutton:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:backdrop, button.titlebutton:backdrop, combobox:drop(active) button.combo:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:backdrop, button.titlebutton:backdrop, .selection-mode button.titlebutton:backdrop, .selection-mode button.titlebutton:backdrop,
  button:backdrop.flat,
  button.sidebar-button:backdrop,
  button.titlebutton:backdrop,
  button:backdrop {
    border-color: #0c0c0d;
    background-image: image(#2D2E32);
    text-shadow: none;
    -gtk-icon-shadow: none;
    transition: 200ms ease-out;
    -gtk-icon-effect: none; }
    combobox:drop(active) button.combo:backdrop.flat label, notebook > header > tabs > arrow:backdrop label, button.sidebar-button:backdrop label, button.titlebutton:backdrop label, combobox:drop(active) button.combo:backdrop.flat, combobox:drop(active) button.combo.sidebar-button:backdrop, combobox:drop(active) button.combo.titlebutton:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:backdrop, button.titlebutton:backdrop, combobox:drop(active) button.combo:backdrop label, notebook > header > tabs > arrow:backdrop label, button.sidebar-button:backdrop label, button.titlebutton:backdrop label, combobox:drop(active) button.combo:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:backdrop, button.titlebutton:backdrop, .selection-mode button.titlebutton:backdrop label, .selection-mode button.titlebutton:backdrop, .selection-mode button.titlebutton:backdrop label, .selection-mode button.titlebutton:backdrop,
    button:backdrop.flat label,
    button.sidebar-button:backdrop label,
    button.titlebutton:backdrop label,
    button:backdrop.flat,
    button.sidebar-button:backdrop,
    button.titlebutton:backdrop,
    button:backdrop label,
    button:backdrop {
      color: #ffffff; }
    combobox:drop(active) button.combo:backdrop.flat:active, notebook > header > tabs > arrow:backdrop:active, button.sidebar-button:backdrop:active, button.titlebutton:backdrop:active, combobox:drop(active) button.combo:backdrop.flat:checked, notebook > header > tabs > arrow:backdrop:checked, button.sidebar-button:backdrop:checked, button.titlebutton:backdrop:checked, combobox:drop(active) button.combo:backdrop:active, notebook > header > tabs > arrow:backdrop:active, button.sidebar-button:backdrop:active, button.titlebutton:backdrop:active, combobox:drop(active) button.combo:backdrop:checked, notebook > header > tabs > arrow:backdrop:checked, button.sidebar-button:backdrop:checked, button.titlebutton:backdrop:checked, .selection-mode button.titlebutton:backdrop:active, .selection-mode button.titlebutton:backdrop:checked, .selection-mode button.titlebutton:backdrop:active, .selection-mode button.titlebutton:backdrop:checked,
    button:backdrop.flat:active,
    button.sidebar-button:backdrop:active,
    button.titlebutton:backdrop:active,
    button:backdrop.flat:checked,
    button.sidebar-button:backdrop:checked,
    button.titlebutton:backdrop:checked,
    button:backdrop:active,
    button:backdrop:checked {
      border-color: #0c0c0d;
      background-image: image(black); }
      combobox:drop(active) button.combo:backdrop.flat:active label, notebook > header > tabs > arrow:backdrop:active label, button.sidebar-button:backdrop:active label, button.titlebutton:backdrop:active label, combobox:drop(active) button.combo:backdrop.flat:active, notebook > header > tabs > arrow:backdrop:active, button.sidebar-button:backdrop:active, button.titlebutton:backdrop:active, combobox:drop(active) button.combo:backdrop.flat:checked label, notebook > header > tabs > arrow:backdrop:checked label, button.sidebar-button:backdrop:checked label, button.titlebutton:backdrop:checked label, combobox:drop(active) button.combo:backdrop.flat:checked, notebook > header > tabs > arrow:backdrop:checked, button.sidebar-button:backdrop:checked, button.titlebutton:backdrop:checked, combobox:drop(active) button.combo:backdrop:active label, notebook > header > tabs > arrow:backdrop:active label, button.sidebar-button:backdrop:active label, button.titlebutton:backdrop:active label, combobox:drop(active) button.combo:backdrop:active, notebook > header > tabs > arrow:backdrop:active, button.sidebar-button:backdrop:active, button.titlebutton:backdrop:active, combobox:drop(active) button.combo:backdrop:checked label, notebook > header > tabs > arrow:backdrop:checked label, button.sidebar-button:backdrop:checked label, button.titlebutton:backdrop:checked label, combobox:drop(active) button.combo:backdrop:checked, notebook > header > tabs > arrow:backdrop:checked, button.sidebar-button:backdrop:checked, button.titlebutton:backdrop:checked, .selection-mode button.titlebutton:backdrop:active label, .selection-mode button.titlebutton:backdrop:active, .selection-mode button.titlebutton:backdrop:checked label, .selection-mode button.titlebutton:backdrop:checked, .selection-mode button.titlebutton:backdrop:active label, .selection-mode button.titlebutton:backdrop:active, .selection-mode button.titlebutton:backdrop:checked label, .selection-mode button.titlebutton:backdrop:checked,
      button:backdrop.flat:active label,
      button.sidebar-button:backdrop:active label,
      button.titlebutton:backdrop:active label,
      button:backdrop.flat:active,
      button.sidebar-button:backdrop:active,
      button.titlebutton:backdrop:active,
      button:backdrop.flat:checked label,
      button.sidebar-button:backdrop:checked label,
      button.titlebutton:backdrop:checked label,
      button:backdrop.flat:checked,
      button.sidebar-button:backdrop:checked,
      button.titlebutton:backdrop:checked,
      button:backdrop:active label,
      button:backdrop:active,
      button:backdrop:checked label,
      button:backdrop:checked {
        color: #ffffff; }
    combobox:drop(active) button.combo:backdrop.flat:disabled, notebook > header > tabs > arrow:backdrop:disabled, button.sidebar-button:backdrop:disabled, button.titlebutton:backdrop:disabled, combobox:drop(active) button.combo:backdrop:disabled, notebook > header > tabs > arrow:backdrop:disabled, button.sidebar-button:backdrop:disabled, button.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop:disabled,
    button:backdrop.flat:disabled,
    button.sidebar-button:backdrop:disabled,
    button.titlebutton:backdrop:disabled,
    button:backdrop:disabled {
      border-color: #0c0c0d;
      background-image: image(#1C1C1E);
      text-shadow: none;
      -gtk-icon-shadow: none; }
      combobox:drop(active) button.combo:backdrop.flat:disabled label, notebook > header > tabs > arrow:backdrop:disabled label, button.sidebar-button:backdrop:disabled label, button.titlebutton:backdrop:disabled label, combobox:drop(active) button.combo:backdrop.flat:disabled, notebook > header > tabs > arrow:backdrop:disabled, button.sidebar-button:backdrop:disabled, button.titlebutton:backdrop:disabled, combobox:drop(active) button.combo:backdrop:disabled label, notebook > header > tabs > arrow:backdrop:disabled label, button.sidebar-button:backdrop:disabled label, button.titlebutton:backdrop:disabled label, combobox:drop(active) button.combo:backdrop:disabled, notebook > header > tabs > arrow:backdrop:disabled, button.sidebar-button:backdrop:disabled, button.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop:disabled label, .selection-mode button.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop:disabled label, .selection-mode button.titlebutton:backdrop:disabled,
      button:backdrop.flat:disabled label,
      button.sidebar-button:backdrop:disabled label,
      button.titlebutton:backdrop:disabled label,
      button:backdrop.flat:disabled,
      button.sidebar-button:backdrop:disabled,
      button.titlebutton:backdrop:disabled,
      button:backdrop:disabled label,
      button:backdrop:disabled {
        color: rgba(255, 255, 255, 0.3); }
      combobox:drop(active) button.combo:backdrop.flat:disabled:active, notebook > header > tabs > arrow:backdrop:disabled:active, button.sidebar-button:backdrop:disabled:active, button.titlebutton:backdrop:disabled:active, combobox:drop(active) button.combo:backdrop.flat:disabled:checked, notebook > header > tabs > arrow:backdrop:disabled:checked, button.sidebar-button:backdrop:disabled:checked, button.titlebutton:backdrop:disabled:checked, combobox:drop(active) button.combo:backdrop:disabled:active, notebook > header > tabs > arrow:backdrop:disabled:active, button.sidebar-button:backdrop:disabled:active, button.titlebutton:backdrop:disabled:active, combobox:drop(active) button.combo:backdrop:disabled:checked, notebook > header > tabs > arrow:backdrop:disabled:checked, button.sidebar-button:backdrop:disabled:checked, button.titlebutton:backdrop:disabled:checked, .selection-mode button.titlebutton:backdrop:disabled:active, .selection-mode button.titlebutton:backdrop:disabled:checked, .selection-mode button.titlebutton:backdrop:disabled:active, .selection-mode button.titlebutton:backdrop:disabled:checked,
      button:backdrop.flat:disabled:active,
      button.sidebar-button:backdrop:disabled:active,
      button.titlebutton:backdrop:disabled:active,
      button:backdrop.flat:disabled:checked,
      button.sidebar-button:backdrop:disabled:checked,
      button.titlebutton:backdrop:disabled:checked,
      button:backdrop:disabled:active,
      button:backdrop:disabled:checked {
        border-color: #0c0c0d;
        background-image: image(black); }
        combobox:drop(active) button.combo:backdrop.flat:disabled:active label, notebook > header > tabs > arrow:backdrop:disabled:active label, button.sidebar-button:backdrop:disabled:active label, button.titlebutton:backdrop:disabled:active label, combobox:drop(active) button.combo:backdrop.flat:disabled:checked label, notebook > header > tabs > arrow:backdrop:disabled:checked label, button.sidebar-button:backdrop:disabled:checked label, button.titlebutton:backdrop:disabled:checked label, combobox:drop(active) button.combo:backdrop:disabled:active label, notebook > header > tabs > arrow:backdrop:disabled:active label, button.sidebar-button:backdrop:disabled:active label, button.titlebutton:backdrop:disabled:active label, combobox:drop(active) button.combo:backdrop:disabled:checked label, notebook > header > tabs > arrow:backdrop:disabled:checked label, button.sidebar-button:backdrop:disabled:checked label, button.titlebutton:backdrop:disabled:checked label, .selection-mode button.titlebutton:backdrop:disabled:active label, .selection-mode button.titlebutton:backdrop:disabled:checked label, .selection-mode button.titlebutton:backdrop:disabled:active label, .selection-mode button.titlebutton:backdrop:disabled:checked label,
        button:backdrop.flat:disabled:active label,
        button.sidebar-button:backdrop:disabled:active label,
        button.titlebutton:backdrop:disabled:active label,
        button:backdrop.flat:disabled:checked label,
        button.sidebar-button:backdrop:disabled:checked label,
        button.titlebutton:backdrop:disabled:checked label,
        button:backdrop:disabled:active label,
        button:backdrop:disabled:checked label {
          color: rgba(255, 255, 255, 0.3); }
  combobox:drop(active) button.flat.combo:backdrop, combobox:drop(active) button.combo.sidebar-button:backdrop, combobox:drop(active) button.combo.titlebutton:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:backdrop, button.titlebutton:backdrop, combobox:drop(active) button.flat.combo:disabled, combobox:drop(active) button.combo.sidebar-button:disabled, combobox:drop(active) button.combo.titlebutton:disabled, notebook > header > tabs > arrow:disabled, button.sidebar-button:disabled, button.titlebutton:disabled, combobox:drop(active) button.flat.combo:backdrop:disabled, notebook > header > tabs > arrow:backdrop:disabled, button.sidebar-button:backdrop:disabled, button.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop, .selection-mode button.titlebutton:disabled, .selection-mode button.titlebutton:backdrop:disabled,
  button.flat:backdrop,
  button.sidebar-button:backdrop,
  button.titlebutton:backdrop,
  button.flat:disabled,
  button.sidebar-button:disabled,
  button.titlebutton:disabled,
  button.flat:backdrop:disabled,
  button.sidebar-button:backdrop:disabled,
  button.titlebutton:backdrop:disabled {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none; }
  combobox:drop(active) button.combo:disabled, notebook > header > tabs > arrow:disabled, button.sidebar-button:disabled, button.titlebutton:disabled, .selection-mode button.titlebutton:disabled,
  button:disabled {
    border-color: #1C1C1E;
    background-image: image(#1C1C1E);
    text-shadow: none;
    -gtk-icon-shadow: none; }
    combobox:drop(active) button.combo:disabled label, notebook > header > tabs > arrow:disabled label, button.sidebar-button:disabled label, button.titlebutton:disabled label, combobox:drop(active) button.combo:disabled, notebook > header > tabs > arrow:disabled, button.sidebar-button:disabled, button.titlebutton:disabled, .selection-mode button.titlebutton:disabled label, .selection-mode button.titlebutton:disabled,
    button:disabled label,
    button:disabled {
      color: rgba(255, 255, 255, 0.3); }
    combobox:drop(active) button.combo:disabled:active, notebook > header > tabs > arrow:disabled:active, button.sidebar-button:disabled:active, button.titlebutton:disabled:active, combobox:drop(active) button.combo:disabled:checked, notebook > header > tabs > arrow:disabled:checked, button.sidebar-button:disabled:checked, button.titlebutton:disabled:checked, .selection-mode button.titlebutton:disabled:active, .selection-mode button.titlebutton:disabled:checked,
    button:disabled:active,
    button:disabled:checked {
      border-color: #1C1C1E;
      background-image: image(black); }
      combobox:drop(active) button.combo:disabled:active label, notebook > header > tabs > arrow:disabled:active label, button.sidebar-button:disabled:active label, button.titlebutton:disabled:active label, combobox:drop(active) button.combo:disabled:active, notebook > header > tabs > arrow:disabled:active, button.sidebar-button:disabled:active, button.titlebutton:disabled:active, combobox:drop(active) button.combo:disabled:checked label, notebook > header > tabs > arrow:disabled:checked label, button.sidebar-button:disabled:checked label, button.titlebutton:disabled:checked label, combobox:drop(active) button.combo:disabled:checked, notebook > header > tabs > arrow:disabled:checked, button.sidebar-button:disabled:checked, button.titlebutton:disabled:checked, .selection-mode button.titlebutton:disabled:active label, .selection-mode button.titlebutton:disabled:active, .selection-mode button.titlebutton:disabled:checked label, .selection-mode button.titlebutton:disabled:checked,
      button:disabled:active label,
      button:disabled:active,
      button:disabled:checked label,
      button:disabled:checked {
        color: rgba(255, 255, 255, 0.3); }
  combobox:drop(active) button.image-button.combo, notebook > header > tabs > arrow.image-button, button.image-button.sidebar-button, button.image-button.titlebutton, .selection-mode button.image-button.titlebutton,
  button.image-button {
    min-width: 24px;
    padding-left: 4px;
    padding-right: 4px; }
  combobox:drop(active) button.text-button.combo, notebook > header > tabs > arrow.text-button, button.text-button.sidebar-button, button.text-button.titlebutton, .selection-mode button.text-button.titlebutton,
  button.text-button {
    padding-left: 16px;
    padding-right: 16px; }
  combobox:drop(active) button.text-button.image-button.combo, notebook > header > tabs > arrow.text-button.image-button, button.text-button.image-button.sidebar-button, button.text-button.image-button.titlebutton, .selection-mode button.text-button.image-button.titlebutton,
  button.text-button.image-button {
    padding-left: 8px;
    padding-right: 8px; }
    combobox:drop(active) button.text-button.image-button.combo label, notebook > header > tabs > arrow.text-button.image-button label, button.text-button.image-button.sidebar-button label, button.text-button.image-button.titlebutton label, .selection-mode button.text-button.image-button.titlebutton label,
    button.text-button.image-button label {
      padding-left: 8px;
      padding-right: 8px; }
  combobox:drop(active) button.combo, notebook > header > tabs > arrow:drop(active), button.sidebar-button:drop(active), button.titlebutton:drop(active), combobox:drop(active) .selection-mode button.combo.titlebutton, .selection-mode combobox:drop(active) button.combo.titlebutton, .selection-mode button.titlebutton:drop(active),
  button:drop(active),
  combobox:drop(active) button.combo {
    color: #0c0c0d;
    border-color: #0c0c0d;
    box-shadow: inset 0 0 0 1px #0c0c0d; }
row:selected button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled), .selection-mode.titlebar:not(headerbar) row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled),
headerbar.selection-mode row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled), row:selected .selection-mode button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled), .selection-mode row:selected button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled), .selection-mode.titlebar:not(headerbar) row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled),
headerbar.selection-mode row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled),
row:selected button.flat:not(:active):not(:checked):not(:hover):not(disabled),
.selection-mode.titlebar:not(headerbar) row.subtitle:link button.flat:not(:active):not(:checked):not(:hover):not(disabled),
headerbar.selection-mode row.subtitle:link button.flat:not(:active):not(:checked):not(:hover):not(disabled),
row:selected button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled),
.selection-mode.titlebar:not(headerbar) row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled),
headerbar.selection-mode row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled),
row:selected button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled),
.selection-mode.titlebar:not(headerbar) row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled),
headerbar.selection-mode row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled) {
  color: rgba(255, 255, 255, 0.3); }
  row:selected button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop, .selection-mode.titlebar:not(headerbar) row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  headerbar.selection-mode row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop, row:selected .selection-mode button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop, .selection-mode row:selected button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop, .selection-mode.titlebar:not(headerbar) row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  headerbar.selection-mode row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  row:selected button.flat:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  .selection-mode.titlebar:not(headerbar) row.subtitle:link button.flat:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  headerbar.selection-mode row.subtitle:link button.flat:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  row:selected button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  .selection-mode.titlebar:not(headerbar) row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  headerbar.selection-mode row.subtitle:link button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  row:selected button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  .selection-mode.titlebar:not(headerbar) row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop,
  headerbar.selection-mode row.subtitle:link button.titlebutton:not(:active):not(:checked):not(:hover):not(disabled):backdrop {
    color: #0c0c0d; }
.selection-mode button.osd.titlebutton,
button.osd {
  min-width: 24px;
  min-height: 32px;
  color: #ffffff;
  border-radius: 5px;
  border: none;
  box-shadow: none; }
  .selection-mode button.osd.image-button.titlebutton,
  button.osd.image-button {
    min-width: 32px; }
  .selection-mode button.osd.titlebutton:hover,
  button.osd:hover {
    border: none;
    box-shadow: none; }
  .selection-mode button.osd.titlebutton:active, .selection-mode button.osd.titlebutton:checked,
  button.osd:active,
  button.osd:checked {
    border: none;
    box-shadow: none; }
  .selection-mode button.osd.titlebutton:disabled:backdrop, .selection-mode button.osd.titlebutton:disabled,
  button.osd:disabled:backdrop,
  button.osd:disabled {
    border: none; }
  .selection-mode button.osd.titlebutton:backdrop,
  button.osd:backdrop {
    border: none; }
popover.background.touch-selection button.flat, popover.background.touch-selection button.sidebar-button, popover.background.touch-selection button.titlebutton, popover.background.magnifier button.flat, popover.background.magnifier button.sidebar-button, popover.background.magnifier button.titlebutton, .app-notification button.flat, .app-notification button.sidebar-button, .app-notification button.titlebutton, .osd .selection-mode button.titlebutton, .selection-mode .osd button.titlebutton,
.osd button.flat,
.osd button.sidebar-button,
.osd button.titlebutton {
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none; }
  popover.background.touch-selection button.flat:disabled, popover.background.touch-selection button.sidebar-button:disabled, popover.background.touch-selection button.titlebutton:disabled, popover.background.magnifier button.flat:disabled, popover.background.magnifier button.sidebar-button:disabled, popover.background.magnifier button.titlebutton:disabled, .app-notification button.flat:disabled, .app-notification button.sidebar-button:disabled, .app-notification button.titlebutton:disabled, .osd .selection-mode button.titlebutton:disabled, .selection-mode .osd button.titlebutton:disabled,
  .osd button.flat:disabled,
  .osd button.sidebar-button:disabled,
  .osd button.titlebutton:disabled {
    background-image: none;
    border-color: transparent;
    box-shadow: none; }
.selection-mode button.titlebutton, button.suggested-action.flat, button.suggested-action.sidebar-button, button.suggested-action.titlebutton, .selection-mode button.titlebutton {
  color: #3790FA; }
.selection-mode button.titlebutton:backdrop, .selection-mode button.titlebutton:disabled, .selection-mode button.titlebutton:backdrop:disabled, button.suggested-action.flat:backdrop, button.suggested-action.sidebar-button:backdrop, button.suggested-action.titlebutton:backdrop, .selection-mode button.titlebutton:backdrop, button.suggested-action.flat:disabled, button.suggested-action.sidebar-button:disabled, button.suggested-action.titlebutton:disabled, .selection-mode button.titlebutton:disabled, button.suggested-action.flat:backdrop:disabled, button.suggested-action.sidebar-button:backdrop:disabled, button.suggested-action.titlebutton:backdrop:disabled, .selection-mode button.titlebutton:backdrop:disabled {
  color: rgba(55, 144, 250, 0.8); }
.selection-mode button.destructive-action.titlebutton, button.destructive-action.flat, button.destructive-action.sidebar-button, button.destructive-action.titlebutton {
  color: #d42320; }
.selection-mode button.destructive-action.titlebutton:backdrop, .selection-mode button.destructive-action.titlebutton:disabled, .selection-mode button.destructive-action.titlebutton:backdrop:disabled, button.destructive-action.flat:backdrop, button.destructive-action.sidebar-button:backdrop, button.destructive-action.titlebutton:backdrop, button.destructive-action.flat:disabled, button.destructive-action.sidebar-button:disabled, button.destructive-action.titlebutton:disabled, button.destructive-action.flat:backdrop:disabled, button.destructive-action.sidebar-button:backdrop:disabled, button.destructive-action.titlebutton:backdrop:disabled {
  color: rgba(212, 35, 32, 0.8); }
.selection-mode .stack-switcher > button.titlebutton, .stack-switcher > button {
  outline-offset: -3px; }
  .selection-mode .stack-switcher > button.titlebutton > label, .stack-switcher > button > label {
    padding-left: 6px;
    padding-right: 6px; }
  .selection-mode .stack-switcher > button.titlebutton > image, .stack-switcher > button > image {
    padding-left: 6px;
    padding-right: 6px;
    padding-top: 3px;
    padding-bottom: 3px; }
  .selection-mode .stack-switcher > button.text-button.titlebutton, .stack-switcher > button.text-button {
    padding-left: 10px;
    padding-right: 10px; }
  .selection-mode .stack-switcher > button.image-button.titlebutton, .stack-switcher > button.image-button {
    padding-left: 2px;
    padding-right: 2px; }
  .selection-mode .stack-switcher > button.needs-attention.titlebutton:active > label,
  .selection-mode .stack-switcher > button.needs-attention.titlebutton:active > image, .selection-mode .stack-switcher > button.needs-attention.titlebutton:checked > label,
  .selection-mode .stack-switcher > button.needs-attention.titlebutton:checked > image, .stack-switcher > button.needs-attention:active > label,
  .stack-switcher > button.needs-attention:active > image, .stack-switcher > button.needs-attention:checked > label,
  .stack-switcher > button.needs-attention:checked > image {
    animation: none;
    background-image: none; }
.selection-mode button.font.titlebutton separator, .selection-mode button.file.titlebutton separator, button.font separator, button.file separator {
  background-color: transparent; }
.selection-mode button.font.titlebutton > box > box > label, button.font > box > box > label {
  font-weight: bold; }
.inline-toolbar .selection-mode button.titlebutton, .selection-mode .inline-toolbar button.titlebutton, .inline-toolbar .selection-mode button.titlebutton:backdrop, .selection-mode .inline-toolbar button.titlebutton:backdrop, .inline-toolbar button, .inline-toolbar button:backdrop {
  border-radius: 2px;
  border-width: 1px; }
.primary-toolbar .selection-mode button.titlebutton, .selection-mode .primary-toolbar button.titlebutton, .primary-toolbar button {
  -gtk-icon-shadow: none; }
.selection-mode button.circular.titlebutton, button.circular {
  border-radius: 9999px;
  -gtk-outline-radius: 9999px;
  background-origin: padding-box, border-box;
  background-clip: padding-box, border-box; }
  .selection-mode button.circular.titlebutton label, button.circular label {
    padding: 0; }
  .selection-mode button.circular.titlebutton:not(:checked):not(:active):not(:disabled):not(:backdrop), button.circular:not(:checked):not(:active):not(:disabled):not(:backdrop) {
    border-color: transparent; }
  .selection-mode button.circular.titlebutton:hover:not(:checked):not(:active):not(:disabled):not(:backdrop), button.circular:hover:not(:checked):not(:active):not(:disabled):not(:backdrop) {
    border-color: transparent; }

.selection-mode .stack-switcher > button.needs-attention.titlebutton > label,
.selection-mode .stack-switcher > button.needs-attention.titlebutton > image, .stack-switcher > button.needs-attention > label,
.stack-switcher > button.needs-attention > image, stacksidebar row.needs-attention > label {
  animation: needs_attention 150ms ease-in;
  background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(#3790FA), to(transparent)), -gtk-gradient(radial, center center, 0, center center, 0.5, to(rgba(255, 255, 255, 0.9607843137)), to(transparent));
  background-size: 6px 6px, 6px 6px;
  background-repeat: no-repeat;
  background-position: right 3px, right 4px; }
  .stack-switcher > button.needs-attention > label:backdrop,
  .stack-switcher > button.needs-attention > image:backdrop, stacksidebar row.needs-attention > label:backdrop {
    background-size: 6px 6px, 0 0; }
  .stack-switcher > button.needs-attention > label:dir(rtl),
  .stack-switcher > button.needs-attention > image:dir(rtl), stacksidebar row.needs-attention > label:dir(rtl) {
    background-position: left 3px, left 4px; }

.inline-toolbar toolbutton > button {
  box-shadow: none;
  border-radius: 4px;
  color: #fff;
  background-image: image(#2D2E32); }
  .inline-toolbar toolbutton > button:hover {
    color: #fff;
    background-image: image(#3790FA); }
  .inline-toolbar toolbutton > button:disabled {
    color: #474747;
    background-image: image(#2D2E32); }
  .inline-toolbar toolbutton > button:backdrop {
    color: #ffffff;
    background-image: image(#2D2E32); }
  .inline-toolbar toolbutton > button:backdrop:disabled {
    color: #474747;
    background-image: image(#2D2E32); }

toolbar.inline-toolbar toolbutton > button.flat, toolbar.inline-toolbar toolbutton > button.sidebar-button, toolbar.inline-toolbar toolbutton > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop > button.flat,
toolbar.inline-toolbar toolbutton:backdrop > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop > button.titlebutton, .linked:not(.vertical) > spinbutton:not(.vertical), .linked:not(.vertical) >
entry, .inline-toolbar .selection-mode button.titlebutton, .selection-mode .inline-toolbar button.titlebutton, .inline-toolbar button, .inline-toolbar button:backdrop, .selection-mode .linked > button.titlebutton, .linked > button, .linked > button:hover, .linked > button:active, .linked > button:checked, .linked > button:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.flat, toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:first-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton, toolbar.inline-toolbar toolbutton:last-child > button.flat, toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:last-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton, toolbar.inline-toolbar toolbutton:only-child > button.flat, toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:only-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton, combobox.linked button:nth-child(2):dir(ltr), combobox.linked button:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo, .linked:not(.vertical) > combobox:last-child > box > button.combo, .linked:not(.vertical) > combobox:only-child > box > button.combo, .linked > combobox > box > button.combo:dir(ltr), .linked > combobox > box > button.combo:dir(rtl) {
  border-radius: 0;
  border-right-style: none; }

.linked:not(.vertical) > spinbutton:first-child:not(.vertical), .linked:not(.vertical) >
entry:first-child, toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton:first-child > button.titlebutton, .inline-toolbar combobox.linked .selection-mode button.titlebutton:nth-child(2):dir(rtl), combobox.linked .selection-mode .inline-toolbar button.titlebutton:nth-child(2):dir(rtl), .inline-toolbar .selection-mode combobox.linked button.titlebutton:nth-child(2):dir(rtl), .selection-mode combobox.linked .inline-toolbar button.titlebutton:nth-child(2):dir(rtl), .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:first-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:first-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode .linked.vertical > combobox:first-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked.vertical > combobox:first-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode button.titlebutton:first-child, .selection-mode .inline-toolbar button.titlebutton:first-child, toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:backdrop, .inline-toolbar combobox.linked .selection-mode button.titlebutton:backdrop:nth-child(2):dir(rtl), combobox.linked .selection-mode .inline-toolbar button.titlebutton:backdrop:nth-child(2):dir(rtl), .inline-toolbar .selection-mode combobox.linked button.titlebutton:backdrop:nth-child(2):dir(rtl), .selection-mode combobox.linked .inline-toolbar button.titlebutton:backdrop:nth-child(2):dir(rtl), .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:first-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:first-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode .linked.vertical > combobox:first-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked.vertical > combobox:first-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode button.titlebutton:first-child:backdrop, .selection-mode .inline-toolbar button.titlebutton:first-child:backdrop, .inline-toolbar button:first-child, toolbar.inline-toolbar toolbutton:first-child > button.flat, toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:first-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton, .inline-toolbar combobox.linked button:nth-child(2):dir(rtl), combobox.linked .inline-toolbar button:nth-child(2):dir(rtl), .inline-toolbar .linked:not(.vertical) > combobox:first-child > box > button.combo, .inline-toolbar .linked.vertical > combobox:first-child > box > button.combo, .inline-toolbar button:first-child:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:backdrop, .inline-toolbar combobox.linked button:backdrop:nth-child(2):dir(rtl), combobox.linked .inline-toolbar button:backdrop:nth-child(2):dir(rtl), .inline-toolbar .linked:not(.vertical) > combobox:first-child > box > button.combo:backdrop, .inline-toolbar .linked.vertical > combobox:first-child > box > button.combo:backdrop, toolbar.inline-toolbar .selection-mode toolbutton.linked:first-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton, combobox.linked .selection-mode .linked > button.titlebutton:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked > button.titlebutton:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:first-child > box.linked > button.combo.titlebutton, .selection-mode .linked > button.titlebutton:first-child, toolbar.inline-toolbar .selection-mode toolbutton.linked:first-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:hover, combobox.linked .selection-mode .linked > button.titlebutton:hover:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked > button.titlebutton:hover:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:first-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked > button.titlebutton:first-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked:first-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:active, combobox.linked .selection-mode .linked > button.titlebutton:active:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked > button.titlebutton:active:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:first-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked > button.titlebutton:first-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked:first-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:checked, combobox.linked .selection-mode .linked > button.titlebutton:checked:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked > button.titlebutton:checked:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:first-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked > button.titlebutton:first-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked:first-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:backdrop, combobox.linked .selection-mode .linked > button.titlebutton:backdrop:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked > button.titlebutton:backdrop:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:first-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked > button.titlebutton:first-child:backdrop, .linked > button:first-child, toolbar.inline-toolbar toolbutton.linked:first-child > button.flat, toolbar.inline-toolbar toolbutton.linked:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton, combobox.linked .linked > button:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked > button.combo, .linked.vertical > combobox:first-child > box.linked > button.combo, .linked > button:first-child:hover, toolbar.inline-toolbar toolbutton.linked:first-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked:first-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:hover, combobox.linked .linked > button:hover:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked > button.combo:hover, .linked.vertical > combobox:first-child > box.linked > button.combo:hover, .linked > button:first-child:active, toolbar.inline-toolbar toolbutton.linked:first-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked:first-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:active, combobox.linked .linked > button:active:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked > button.combo:active, .linked.vertical > combobox:first-child > box.linked > button.combo:active, .linked > button:first-child:checked, toolbar.inline-toolbar toolbutton.linked:first-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked:first-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:checked, combobox.linked .linked > button:checked:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked > button.combo:checked, .linked.vertical > combobox:first-child > box.linked > button.combo:checked, .linked > button:first-child:backdrop, toolbar.inline-toolbar toolbutton.linked:first-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked:first-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked:first-child > button.titlebutton:backdrop, combobox.linked .linked > button:backdrop:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked > button.combo:backdrop, .linked.vertical > combobox:first-child > box.linked > button.combo:backdrop,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:first-child > button.titlebutton, toolbar.inline-toolbar toolbutton:last-child > button.flat:first-child, toolbar.inline-toolbar toolbutton:first-child:last-child > button.flat, toolbar.inline-toolbar toolbutton:first-child:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:first-child:last-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:first-child:last-child > button.titlebutton, toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.flat:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.flat:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.sidebar-button:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl), toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:first-child, toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:first-child, toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:first-child, toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat:first-child,
toolbar.inline-toolbar toolbutton:backdrop:first-child:last-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:first-child:last-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:first-child:last-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:first-child:last-child > button.titlebutton,
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:last-child > button.flat:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:last-child > button.sidebar-button:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:last-child > button.titlebutton:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:backdrop:last-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button:first-child,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton:first-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:last-child > button.titlebutton:first-child, toolbar.inline-toolbar toolbutton:only-child > button.flat:first-child, toolbar.inline-toolbar toolbutton:first-child:only-child > button.flat, toolbar.inline-toolbar toolbutton:first-child:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:first-child:only-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:first-child:only-child > button.titlebutton, toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.flat:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.flat:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.sidebar-button:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl), toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:first-child, toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:first-child, toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton:first-child, toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat:first-child,
toolbar.inline-toolbar toolbutton:backdrop:first-child:only-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:first-child:only-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:first-child:only-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:first-child:only-child > button.titlebutton,
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.flat:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.sidebar-button:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(rtl),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(rtl),
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button:first-child,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton:first-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:only-child > button.titlebutton:first-child, combobox.linked button:first-child:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.flat:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.flat:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.sidebar-button:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked button:nth-child(2):dir(ltr):dir(rtl), combobox.linked .linked:not(.vertical) > combobox:first-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked .linked.vertical > combobox:first-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked button:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo, .linked:not(.vertical) > combobox:last-child > box > button.combo:first-child, combobox.linked .linked:not(.vertical) > combobox:last-child > box > button.combo:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child:last-child > box > button.combo, .linked:not(.vertical) > combobox:only-child > box > button.combo:first-child, combobox.linked .linked:not(.vertical) > combobox:only-child > box > button.combo:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child:only-child > box > button.combo, combobox.linked .linked.vertical > combobox:first-child > box > button.combo:nth-child(2):dir(rtl), .linked.vertical:not(.vertical) > combobox:first-child > box > button.combo {
  border-top-left-radius: 0px;
  border-bottom-left-radius: 0px; }
.linked:not(.vertical) > spinbutton:last-child:not(.vertical), .linked:not(.vertical) >
entry:last-child, toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton:last-child > button.titlebutton, .inline-toolbar combobox.linked .selection-mode button.titlebutton:nth-child(2):dir(ltr), combobox.linked .selection-mode .inline-toolbar button.titlebutton:nth-child(2):dir(ltr), .inline-toolbar .selection-mode combobox.linked button.titlebutton:nth-child(2):dir(ltr), .selection-mode combobox.linked .inline-toolbar button.titlebutton:nth-child(2):dir(ltr), .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:last-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:last-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode .linked.vertical > combobox:last-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked.vertical > combobox:last-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode button.titlebutton:last-child, .selection-mode .inline-toolbar button.titlebutton:last-child, toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:backdrop, .inline-toolbar combobox.linked .selection-mode button.titlebutton:backdrop:nth-child(2):dir(ltr), combobox.linked .selection-mode .inline-toolbar button.titlebutton:backdrop:nth-child(2):dir(ltr), .inline-toolbar .selection-mode combobox.linked button.titlebutton:backdrop:nth-child(2):dir(ltr), .selection-mode combobox.linked .inline-toolbar button.titlebutton:backdrop:nth-child(2):dir(ltr), .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:last-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:last-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode .linked.vertical > combobox:last-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked.vertical > combobox:last-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode button.titlebutton:last-child:backdrop, .selection-mode .inline-toolbar button.titlebutton:last-child:backdrop, .inline-toolbar button:last-child, toolbar.inline-toolbar toolbutton:last-child > button.flat, toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:last-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton, .inline-toolbar combobox.linked button:nth-child(2):dir(ltr), combobox.linked .inline-toolbar button:nth-child(2):dir(ltr), .inline-toolbar .linked:not(.vertical) > combobox:last-child > box > button.combo, .inline-toolbar .linked.vertical > combobox:last-child > box > button.combo, .inline-toolbar button:last-child:backdrop, toolbar.inline-toolbar toolbutton:last-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:backdrop, .inline-toolbar combobox.linked button:backdrop:nth-child(2):dir(ltr), combobox.linked .inline-toolbar button:backdrop:nth-child(2):dir(ltr), .inline-toolbar .linked:not(.vertical) > combobox:last-child > box > button.combo:backdrop, .inline-toolbar .linked.vertical > combobox:last-child > box > button.combo:backdrop, toolbar.inline-toolbar .selection-mode toolbutton.linked:last-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton, combobox.linked .selection-mode .linked > button.titlebutton:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked > button.titlebutton:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:last-child > box.linked > button.combo.titlebutton, .selection-mode .linked > button.titlebutton:last-child, toolbar.inline-toolbar .selection-mode toolbutton.linked:last-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:hover, combobox.linked .selection-mode .linked > button.titlebutton:hover:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked > button.titlebutton:hover:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:last-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked > button.titlebutton:last-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked:last-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:active, combobox.linked .selection-mode .linked > button.titlebutton:active:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked > button.titlebutton:active:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:last-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked > button.titlebutton:last-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked:last-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:checked, combobox.linked .selection-mode .linked > button.titlebutton:checked:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked > button.titlebutton:checked:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:last-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked > button.titlebutton:last-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked:last-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:backdrop, combobox.linked .selection-mode .linked > button.titlebutton:backdrop:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked > button.titlebutton:backdrop:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:last-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked > button.titlebutton:last-child:backdrop, .linked > button:last-child, toolbar.inline-toolbar toolbutton.linked:last-child > button.flat, toolbar.inline-toolbar toolbutton.linked:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton, combobox.linked .linked > button:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked > button.combo, .linked.vertical > combobox:last-child > box.linked > button.combo, .linked > button:last-child:hover, toolbar.inline-toolbar toolbutton.linked:last-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked:last-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:hover, combobox.linked .linked > button:hover:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked > button.combo:hover, .linked.vertical > combobox:last-child > box.linked > button.combo:hover, .linked > button:last-child:active, toolbar.inline-toolbar toolbutton.linked:last-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked:last-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:active, combobox.linked .linked > button:active:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked > button.combo:active, .linked.vertical > combobox:last-child > box.linked > button.combo:active, .linked > button:last-child:checked, toolbar.inline-toolbar toolbutton.linked:last-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked:last-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:checked, combobox.linked .linked > button:checked:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked > button.combo:checked, .linked.vertical > combobox:last-child > box.linked > button.combo:checked, .linked > button:last-child:backdrop, toolbar.inline-toolbar toolbutton.linked:last-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked:last-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked:last-child > button.titlebutton:backdrop, combobox.linked .linked > button:backdrop:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked > button.combo:backdrop, .linked.vertical > combobox:last-child > box.linked > button.combo:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.flat:last-child, toolbar.inline-toolbar toolbutton:last-child:first-child > button.flat, toolbar.inline-toolbar toolbutton:last-child:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:last-child:first-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:last-child:first-child > button.titlebutton, toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.flat:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.flat:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.sidebar-button:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr), toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:last-child, toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:last-child, toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:last-child, toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:first-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat:last-child,
toolbar.inline-toolbar toolbutton:backdrop:last-child:first-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:last-child:first-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:last-child:first-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:last-child:first-child > button.titlebutton,
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:first-child > button.flat:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:first-child > button.sidebar-button:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:first-child > button.titlebutton:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:backdrop:first-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button:last-child,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton:last-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:first-child > button.titlebutton:last-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:last-child > button.titlebutton, toolbar.inline-toolbar toolbutton:only-child > button.flat:last-child, toolbar.inline-toolbar toolbutton:last-child:only-child > button.flat, toolbar.inline-toolbar toolbutton:last-child:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:last-child:only-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:last-child:only-child > button.titlebutton, toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.flat:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.flat:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.sidebar-button:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr), toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:last-child, toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:last-child, toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton:last-child, toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat:last-child,
toolbar.inline-toolbar toolbutton:backdrop:last-child:only-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:last-child:only-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:last-child:only-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:last-child:only-child > button.titlebutton,
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.flat:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.sidebar-button:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(ltr),
combobox.linked toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar combobox.linked .selection-mode toolbutton:backdrop:only-child > button.titlebutton:nth-child(2):dir(ltr),
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button:last-child,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton:last-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:only-child > button.titlebutton:last-child, combobox.linked button:nth-child(2):dir(ltr), combobox.linked button:last-child:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.flat:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.flat:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.sidebar-button:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked button:nth-child(2):dir(rtl):dir(ltr), combobox.linked .linked:not(.vertical) > combobox:last-child > box > button.combo:nth-child(2):dir(rtl), combobox.linked .linked.vertical > combobox:last-child > box > button.combo:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo:last-child, combobox.linked .linked:not(.vertical) > combobox:first-child > box > button.combo:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child:first-child > box > button.combo, .linked:not(.vertical) > combobox:last-child > box > button.combo, .linked:not(.vertical) > combobox:only-child > box > button.combo:last-child, combobox.linked .linked:not(.vertical) > combobox:only-child > box > button.combo:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child:only-child > box > button.combo, combobox.linked .linked.vertical > combobox:last-child > box > button.combo:nth-child(2):dir(ltr), .linked.vertical:not(.vertical) > combobox:last-child > box > button.combo {
  border-top-right-radius: 0px;
  border-bottom-right-radius: 0px;
  border-right-style: solid; }
.linked:not(.vertical) > spinbutton:only-child:not(.vertical), .linked:not(.vertical) >
entry:only-child, toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton:only-child > button.titlebutton, .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:only-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:only-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode .linked.vertical > combobox:only-child > box > button.combo.titlebutton, .selection-mode .inline-toolbar .linked.vertical > combobox:only-child > box > button.combo.titlebutton, .inline-toolbar .selection-mode button.titlebutton:only-child, .selection-mode .inline-toolbar button.titlebutton:only-child, toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:backdrop, .inline-toolbar .selection-mode .linked:not(.vertical) > combobox:only-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked:not(.vertical) > combobox:only-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode .linked.vertical > combobox:only-child > box > button.combo.titlebutton:backdrop, .selection-mode .inline-toolbar .linked.vertical > combobox:only-child > box > button.combo.titlebutton:backdrop, .inline-toolbar .selection-mode button.titlebutton:only-child:backdrop, .selection-mode .inline-toolbar button.titlebutton:only-child:backdrop, .inline-toolbar button:only-child, toolbar.inline-toolbar toolbutton:only-child > button.flat, toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:only-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:only-child > button.titlebutton, .inline-toolbar .linked:not(.vertical) > combobox:only-child > box > button.combo, .inline-toolbar .linked.vertical > combobox:only-child > box > button.combo, .inline-toolbar button:only-child:backdrop, toolbar.inline-toolbar toolbutton:only-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:backdrop, .inline-toolbar .linked:not(.vertical) > combobox:only-child > box > button.combo:backdrop, .inline-toolbar .linked.vertical > combobox:only-child > box > button.combo:backdrop, toolbar.inline-toolbar .selection-mode toolbutton.linked:only-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:only-child > box.linked > button.combo.titlebutton, .selection-mode .linked > button.titlebutton:only-child, toolbar.inline-toolbar .selection-mode toolbutton.linked:only-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:hover, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:only-child > box.linked > button.combo.titlebutton:hover, .selection-mode .linked > button.titlebutton:only-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked:only-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:active, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:only-child > box.linked > button.combo.titlebutton:active, .selection-mode .linked > button.titlebutton:only-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked:only-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:checked, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:only-child > box.linked > button.combo.titlebutton:checked, .selection-mode .linked > button.titlebutton:only-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked:only-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:backdrop, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:only-child > box.linked > button.combo.titlebutton:backdrop, .selection-mode .linked > button.titlebutton:only-child:backdrop, .linked > button:only-child, toolbar.inline-toolbar toolbutton.linked:only-child > button.flat, toolbar.inline-toolbar toolbutton.linked:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton, .linked:not(.vertical) > combobox:only-child > box.linked > button.combo, .linked.vertical > combobox:only-child > box.linked > button.combo, .linked > button:only-child:hover, toolbar.inline-toolbar toolbutton.linked:only-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked:only-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:hover, .linked:not(.vertical) > combobox:only-child > box.linked > button.combo:hover, .linked.vertical > combobox:only-child > box.linked > button.combo:hover, .linked > button:only-child:active, toolbar.inline-toolbar toolbutton.linked:only-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked:only-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:active, .linked:not(.vertical) > combobox:only-child > box.linked > button.combo:active, .linked.vertical > combobox:only-child > box.linked > button.combo:active, .linked > button:only-child:checked, toolbar.inline-toolbar toolbutton.linked:only-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked:only-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:checked, .linked:not(.vertical) > combobox:only-child > box.linked > button.combo:checked, .linked.vertical > combobox:only-child > box.linked > button.combo:checked, .linked > button:only-child:backdrop, toolbar.inline-toolbar toolbutton.linked:only-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked:only-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked:only-child > button.titlebutton:backdrop, .linked:not(.vertical) > combobox:only-child > box.linked > button.combo:backdrop, .linked.vertical > combobox:only-child > box.linked > button.combo:backdrop, toolbar.inline-toolbar toolbutton:first-child > button.flat:only-child, toolbar.inline-toolbar toolbutton:only-child:first-child > button.flat, toolbar.inline-toolbar toolbutton:only-child:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:only-child:first-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:only-child:first-child > button.titlebutton, toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:only-child, toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:only-child, toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:only-child,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat:only-child,
toolbar.inline-toolbar toolbutton:backdrop:only-child:first-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:only-child:first-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:only-child:first-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:only-child:first-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button:only-child,
toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton:only-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:first-child > button.titlebutton:only-child, toolbar.inline-toolbar toolbutton:last-child > button.flat:only-child, toolbar.inline-toolbar toolbutton:only-child:last-child > button.flat, toolbar.inline-toolbar toolbutton:only-child:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton:only-child:last-child > button.titlebutton, toolbar.inline-toolbar .selection-mode toolbutton:only-child:last-child > button.titlebutton, toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:only-child, toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:only-child, toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:only-child,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat:only-child,
toolbar.inline-toolbar toolbutton:backdrop:only-child:last-child > button.flat,
toolbar.inline-toolbar toolbutton:backdrop:only-child:last-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton:backdrop:only-child:last-child > button.titlebutton,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:only-child:last-child > button.titlebutton,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button:only-child,
toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton:only-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:last-child > button.titlebutton:only-child,
toolbar.inline-toolbar .selection-mode toolbutton:backdrop:only-child > button.titlebutton, combobox.linked button:only-child:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.flat:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.flat:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.sidebar-button:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton:nth-child(2):dir(ltr), combobox.linked .linked:not(.vertical) > combobox:only-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked .linked.vertical > combobox:only-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked button:only-child:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.flat:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.flat:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.sidebar-button:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.sidebar-button:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl), toolbar.inline-toolbar combobox.linked toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked toolbar.inline-toolbar .selection-mode toolbutton:only-child > button.titlebutton:nth-child(2):dir(rtl), combobox.linked .linked:not(.vertical) > combobox:only-child > box > button.combo:nth-child(2):dir(rtl), combobox.linked .linked.vertical > combobox:only-child > box > button.combo:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo:only-child, .linked:not(.vertical) > combobox:only-child:first-child > box > button.combo, .linked:not(.vertical) > combobox:last-child > box > button.combo:only-child, .linked:not(.vertical) > combobox:only-child:last-child > box > button.combo, .linked:not(.vertical) > combobox:only-child > box > button.combo, .linked.vertical:not(.vertical) > combobox:only-child > box > button.combo {
  border-radius: 0px;
  border-style: solid; }

.linked.vertical > spinbutton:not(.vertical), .linked.vertical >
entry, .selection-mode .linked.vertical > button.titlebutton, .linked.vertical > button, .linked.vertical > button:hover, .linked.vertical > button:active, .linked.vertical > button:checked, .linked.vertical > button:backdrop, .linked.vertical > combobox:first-child > box > button.combo, .linked.vertical > combobox:last-child > box > button.combo, .linked.vertical > combobox:only-child > box > button.combo, .linked.vertical > combobox > box > button.combo {
  border-style: solid solid none solid;
  border-radius: 0; }

.linked.vertical > spinbutton:first-child:not(.vertical), .linked.vertical >
entry:first-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:first-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton, combobox.linked .selection-mode .linked.vertical > button.titlebutton:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked.vertical > button.titlebutton:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:first-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > button.titlebutton:first-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:first-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:hover, combobox.linked .selection-mode .linked.vertical > button.titlebutton:hover:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked.vertical > button.titlebutton:hover:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:first-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > button.titlebutton:first-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:first-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:active, combobox.linked .selection-mode .linked.vertical > button.titlebutton:active:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked.vertical > button.titlebutton:active:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:first-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > button.titlebutton:first-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:first-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:checked, combobox.linked .selection-mode .linked.vertical > button.titlebutton:checked:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked.vertical > button.titlebutton:checked:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:first-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > button.titlebutton:first-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:first-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:backdrop, combobox.linked .selection-mode .linked.vertical > button.titlebutton:backdrop:nth-child(2):dir(rtl), .selection-mode combobox.linked .linked.vertical > button.titlebutton:backdrop:nth-child(2):dir(rtl), .selection-mode .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:first-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > button.titlebutton:first-child:backdrop, .linked.vertical > button:first-child, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.flat, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton, combobox.linked .linked.vertical > button:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo, .linked.vertical > combobox:first-child > box.linked.vertical > button.combo, .linked.vertical > button:first-child:hover, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:hover, combobox.linked .linked.vertical > button:hover:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo:hover, .linked.vertical > combobox:first-child > box.linked.vertical > button.combo:hover, .linked.vertical > button:first-child:active, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:active, combobox.linked .linked.vertical > button:active:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo:active, .linked.vertical > combobox:first-child > box.linked.vertical > button.combo:active, .linked.vertical > button:first-child:checked, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:checked, combobox.linked .linked.vertical > button:checked:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo:checked, .linked.vertical > combobox:first-child > box.linked.vertical > button.combo:checked, .linked.vertical > button:first-child:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:first-child > button.titlebutton:backdrop, combobox.linked .linked.vertical > button:backdrop:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box.linked.vertical > button.combo:backdrop, .linked.vertical > combobox:first-child > box.linked.vertical > button.combo:backdrop,
toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:backdrop:first-child > button.titlebutton,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:first-child > button.flat,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:first-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:first-child > button.titlebutton, combobox.linked .linked.vertical > combobox:first-child > box > button.combo:nth-child(2):dir(rtl), combobox.linked .linked.vertical > combobox:last-child > box > button.combo:nth-child(2):dir(rtl), combobox.linked .linked.vertical > combobox:only-child > box > button.combo:nth-child(2):dir(rtl), .linked.vertical:not(.vertical) > combobox:first-child > box > button.combo, .linked.vertical > combobox:first-child > box > button.combo, .linked.vertical > combobox:last-child > box > button.combo:first-child, .linked.vertical > combobox:first-child:last-child > box > button.combo, .linked.vertical > combobox:only-child > box > button.combo:first-child, .linked.vertical > combobox:first-child:only-child > box > button.combo {
  border-top-left-radius: 0px;
  border-top-right-radius: 0px; }
.linked.vertical > spinbutton:last-child:not(.vertical), .linked.vertical >
entry:last-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:last-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton, combobox.linked .selection-mode .linked.vertical > button.titlebutton:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked.vertical > button.titlebutton:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:last-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > button.titlebutton:last-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:last-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:hover, combobox.linked .selection-mode .linked.vertical > button.titlebutton:hover:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked.vertical > button.titlebutton:hover:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:last-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > button.titlebutton:last-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:last-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:active, combobox.linked .selection-mode .linked.vertical > button.titlebutton:active:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked.vertical > button.titlebutton:active:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:last-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > button.titlebutton:last-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:last-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:checked, combobox.linked .selection-mode .linked.vertical > button.titlebutton:checked:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked.vertical > button.titlebutton:checked:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:last-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > button.titlebutton:last-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:last-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:backdrop, combobox.linked .selection-mode .linked.vertical > button.titlebutton:backdrop:nth-child(2):dir(ltr), .selection-mode combobox.linked .linked.vertical > button.titlebutton:backdrop:nth-child(2):dir(ltr), .selection-mode .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:last-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > button.titlebutton:last-child:backdrop, .linked.vertical > button:last-child, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.flat, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton, combobox.linked .linked.vertical > button:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo, .linked.vertical > combobox:last-child > box.linked.vertical > button.combo, .linked.vertical > button:last-child:hover, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:hover, combobox.linked .linked.vertical > button:hover:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo:hover, .linked.vertical > combobox:last-child > box.linked.vertical > button.combo:hover, .linked.vertical > button:last-child:active, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:active, combobox.linked .linked.vertical > button:active:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo:active, .linked.vertical > combobox:last-child > box.linked.vertical > button.combo:active, .linked.vertical > button:last-child:checked, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:checked, combobox.linked .linked.vertical > button:checked:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo:checked, .linked.vertical > combobox:last-child > box.linked.vertical > button.combo:checked, .linked.vertical > button:last-child:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:last-child > button.titlebutton:backdrop, combobox.linked .linked.vertical > button:backdrop:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box.linked.vertical > button.combo:backdrop, .linked.vertical > combobox:last-child > box.linked.vertical > button.combo:backdrop,
toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:backdrop:last-child > button.titlebutton,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:last-child > button.flat,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:last-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:last-child > button.titlebutton, combobox.linked .linked.vertical > combobox:first-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked .linked.vertical > combobox:last-child > box > button.combo:nth-child(2):dir(ltr), combobox.linked .linked.vertical > combobox:only-child > box > button.combo:nth-child(2):dir(ltr), .linked.vertical:not(.vertical) > combobox:last-child > box > button.combo, .linked.vertical > combobox:first-child > box > button.combo:last-child, .linked.vertical > combobox:last-child:first-child > box > button.combo, .linked.vertical > combobox:last-child > box > button.combo, .linked.vertical > combobox:only-child > box > button.combo:last-child, .linked.vertical > combobox:last-child:only-child > box > button.combo {
  border-bottom-left-radius: 0px;
  border-bottom-right-radius: 0px;
  border-style: solid; }
.linked.vertical > spinbutton:only-child:not(.vertical), .linked.vertical >
entry:only-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:only-child > button.titlebutton, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > combobox:only-child > box.linked.vertical > button.combo.titlebutton, .selection-mode .linked.vertical > button.titlebutton:only-child, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:only-child > button.titlebutton:hover, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:hover, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > combobox:only-child > box.linked.vertical > button.combo.titlebutton:hover, .selection-mode .linked.vertical > button.titlebutton:only-child:hover, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:only-child > button.titlebutton:active, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:active, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > combobox:only-child > box.linked.vertical > button.combo.titlebutton:active, .selection-mode .linked.vertical > button.titlebutton:only-child:active, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:only-child > button.titlebutton:checked, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:checked, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > combobox:only-child > box.linked.vertical > button.combo.titlebutton:checked, .selection-mode .linked.vertical > button.titlebutton:only-child:checked, toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:only-child > button.titlebutton:backdrop, .selection-mode toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:backdrop, .selection-mode .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > combobox:only-child > box.linked.vertical > button.combo.titlebutton:backdrop, .selection-mode .linked.vertical > button.titlebutton:only-child:backdrop, .linked.vertical > button:only-child, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.flat, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.sidebar-button, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton, .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo, .linked.vertical > combobox:only-child > box.linked.vertical > button.combo, .linked.vertical > button:only-child:hover, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.flat:hover, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.sidebar-button:hover, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:hover, .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo:hover, .linked.vertical > combobox:only-child > box.linked.vertical > button.combo:hover, .linked.vertical > button:only-child:active, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.flat:active, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.sidebar-button:active, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:active, .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo:active, .linked.vertical > combobox:only-child > box.linked.vertical > button.combo:active, .linked.vertical > button:only-child:checked, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.flat:checked, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.sidebar-button:checked, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:checked, .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo:checked, .linked.vertical > combobox:only-child > box.linked.vertical > button.combo:checked, .linked.vertical > button:only-child:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.flat:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.sidebar-button:backdrop, toolbar.inline-toolbar toolbutton.linked.vertical:only-child > button.titlebutton:backdrop, .linked:not(.vertical) > combobox:only-child > box.linked.vertical > button.combo:backdrop, .linked.vertical > combobox:only-child > box.linked.vertical > button.combo:backdrop,
toolbar.inline-toolbar .selection-mode toolbutton.linked.vertical:backdrop:only-child > button.titlebutton,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:only-child > button.flat,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:only-child > button.sidebar-button,
toolbar.inline-toolbar toolbutton.linked.vertical:backdrop:only-child > button.titlebutton, .linked.vertical:not(.vertical) > combobox:only-child > box > button.combo, .linked.vertical > combobox:first-child > box > button.combo:only-child, .linked.vertical > combobox:only-child:first-child > box > button.combo, .linked.vertical > combobox:last-child > box > button.combo:only-child, .linked.vertical > combobox:only-child:last-child > box > button.combo, .linked.vertical > combobox:only-child > box > button.combo {
  border-radius: 0px;
  border-style: solid; }

modelbutton.flat,
.menuitem.button.flat,
notebook > header > tabs > arrow.menuitem.button,
button.menuitem.button.sidebar-button,
button.menuitem.button.titlebutton, modelbutton.flat:backdrop,
.menuitem.button.flat:backdrop,
label:backdrop selection.menuitem.button.flat,
notebook > header > tabs > arrow.menuitem.button:backdrop,
progressbar:backdrop trough.menuitem.button.flat,
progressbar:backdrop progress.menuitem.button.flat,
button.menuitem.button.sidebar-button:backdrop,
button.menuitem.button.titlebutton:backdrop, modelbutton.flat:backdrop:hover,
.menuitem.button.flat:backdrop:hover,
label:backdrop selection.menuitem.button.flat:hover,
notebook > header > tabs > arrow.menuitem.button:backdrop:hover,
progressbar:backdrop trough.menuitem.button.flat:hover,
progressbar:backdrop progress.menuitem.button.flat:hover,
button.menuitem.button.sidebar-button:backdrop:hover,
button.menuitem.button.titlebutton:backdrop:hover, calendar.button, .scale-popup button:hover, .scale-popup button:backdrop:hover, .scale-popup button:backdrop:disabled, .scale-popup button:backdrop, button:link, .selection-mode.titlebar:not(headerbar) button.subtitle:link,
headerbar.selection-mode button.subtitle:link,
button:visited, button:link:hover,
button:hover:visited, button:link:active,
button:active:visited, button:link:checked,
button:checked:visited {
  background-image: none;
  border-color: transparent;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0), 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none; }

/* menu buttons */
modelbutton.flat,
.menuitem.button.flat,
notebook > header > tabs > arrow.menuitem.button,
button.menuitem.button.sidebar-button,
button.menuitem.button.titlebutton {
  min-height: 26px;
  padding-left: 5px;
  padding-right: 5px;
  border-radius: 3px;
  outline-offset: -2px; }
  modelbutton.flat:hover,
  .menuitem.button.flat:hover,
  notebook > header > tabs > arrow.menuitem.button:hover,
  button.menuitem.button.sidebar-button:hover,
  button.menuitem.button.titlebutton:hover {
    color: #fff;
    background-color: #3790FA; }
  modelbutton.flat:backdrop,
  .menuitem.button.flat:backdrop,
  label:backdrop selection.menuitem.button.flat,
  notebook > header > tabs > arrow.menuitem.button:backdrop,
  progressbar:backdrop trough.menuitem.button.flat,
  progressbar:backdrop progress.menuitem.button.flat,
  button.menuitem.button.sidebar-button:backdrop,
  button.menuitem.button.titlebutton:backdrop {
    background-color: #0c0c0d; }
  modelbutton.flat:backdrop:hover,
  .menuitem.button.flat:backdrop:hover,
  label:backdrop selection.menuitem.button.flat:hover,
  notebook > header > tabs > arrow.menuitem.button:backdrop:hover,
  progressbar:backdrop trough.menuitem.button.flat:hover,
  progressbar:backdrop progress.menuitem.button.flat:hover,
  button.menuitem.button.sidebar-button:backdrop:hover,
  button.menuitem.button.titlebutton:backdrop:hover {
    color: #fff;
    background-color: #3790FA; }

modelbutton.flat arrow {
  background-color: #2D2E32; }
  modelbutton.flat arrow:hover {
    color: #fff;
    background-color: #3790FA; }
  modelbutton.flat arrow.left {
    -gtk-icon-source: -gtk-icontheme("pan-start-symbolic"); }
  modelbutton.flat arrow.right {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }

button.color {
  padding: 4px; }
  button.color colorswatch:only-child {
    box-shadow: 0 1px rgba(255, 255, 255, 0.9607843137); }
    button.color colorswatch:only-child, button.color colorswatch:only-child overlay {
      border-radius: 0; }
    .osd button.color colorswatch:only-child {
      box-shadow: none; }
  .osd button.color:disabled colorswatch:only-child, .osd button.color:backdrop colorswatch:only-child, .osd button.color:active colorswatch:only-child, .osd button.color:checked colorswatch:only-child, button.color:disabled colorswatch:only-child, button.color:backdrop colorswatch:only-child, button.color:active colorswatch:only-child, button.color:checked colorswatch:only-child {
    box-shadow: none; }

/*****************
 * GtkSpinButton *
 *****************/
spinbutton:not(.vertical) {
  padding: 0; }
  spinbutton:not(.vertical) entry {
    min-width: 28px;
    margin: 0;
    background: none;
    background-color: #242424;
    border: none;
    border-radius: 3px 0 0 3px;
    box-shadow: none; }
    spinbutton:not(.vertical) entry:backdrop:disabled {
      background-color: #1C1C1E; }
  spinbutton:not(.vertical) button {
    min-height: 16px;
    margin: 0;
    padding-bottom: 0;
    padding-top: 0;
    color: #e7e7e7;
    background-image: none;
    background-color: #393A3E;
    border-style: none none none solid;
    border-color: rgba(12, 12, 13, 0.3);
    border-radius: 0;
    box-shadow: none; }
    spinbutton:not(.vertical) button:dir(rtl) {
      border-style: none solid none none; }
    spinbutton:not(.vertical) button:hover {
      color: #fff;
      background-color: #3790FA; }
    spinbutton:not(.vertical) button:disabled {
      color: rgba(255, 255, 255, 0.3);
      background-color: #1C1C1E; }
    spinbutton:not(.vertical) button:active {
      background-color: rgba(0, 0, 0, 0.1);
      box-shadow: inset 0 2px 3px -1px rgba(0, 0, 0, 0.2); }
    spinbutton:not(.vertical) button:backdrop {
      color: #e7e7e7;
      background-color: transparent;
      border-color: rgba(12, 12, 13, 0.3);
      transition: 200ms ease-out; }
    spinbutton:not(.vertical) button:backdrop:disabled {
      color: rgba(255, 255, 255, 0.3);
      background-color: #1C1C1E;
      border-style: none none none solid; }
      spinbutton:not(.vertical) button:backdrop:disabled:dir(rtl) {
        border-style: none solid none none; }
    spinbutton:not(.vertical) button:dir(ltr):last-child, spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.flat:dir(ltr), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:last-child > button.flat:dir(ltr), spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:dir(ltr), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:last-child > button.sidebar-button:dir(ltr), spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:dir(ltr), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:last-child > button.titlebutton:dir(ltr), spinbutton:not(.vertical) toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:dir(ltr), spinbutton:not(.vertical) combobox.linked button:dir(ltr):nth-child(2), combobox.linked spinbutton:not(.vertical) button:dir(ltr):nth-child(2), spinbutton:not(.vertical) .linked:not(.vertical) > combobox:last-child > box > button.combo:dir(ltr), spinbutton:not(.vertical) .linked.vertical > combobox:last-child > box > button.combo:dir(ltr) {
      border-radius: 0 3px 3px 0; }
    spinbutton:not(.vertical) button:dir(rtl):first-child, spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.flat:dir(rtl), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:first-child > button.flat:dir(rtl), spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:dir(rtl), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:first-child > button.sidebar-button:dir(rtl), spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:dir(rtl), toolbar.inline-toolbar spinbutton:not(.vertical) toolbutton:first-child > button.titlebutton:dir(rtl), spinbutton:not(.vertical) toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:dir(rtl), spinbutton:not(.vertical) combobox.linked button:dir(rtl):nth-child(2), combobox.linked spinbutton:not(.vertical) button:dir(rtl):nth-child(2), spinbutton:not(.vertical) .linked:not(.vertical) > combobox:first-child > box > button.combo:dir(rtl), spinbutton:not(.vertical) .linked.vertical > combobox:first-child > box > button.combo:dir(rtl) {
      border-radius: 3px 0 0 3px; }
.osd spinbutton:not(.vertical) button {
  border-color: transparent;
  background-color: transparent;
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: #ffffff;
  border-style: none none none solid;
  border-color: rgba(0, 0, 0, 0.4);
  border-radius: 0;
  box-shadow: none;
  -gtk-icon-shadow: 0 1px black; }
  .osd spinbutton:not(.vertical) button:dir(rtl) {
    border-style: none solid none none; }
  .osd spinbutton:not(.vertical) button:hover {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none;
    color: #fff;
    border-color: rgba(0, 0, 0, 0.5);
    background-color: #3790FA;
    -gtk-icon-shadow: 0 1px black;
    box-shadow: none; }
  .osd spinbutton:not(.vertical) button:backdrop {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none;
    color: #ffffff;
    border-color: rgba(0, 0, 0, 0.5);
    -gtk-icon-shadow: none;
    box-shadow: none; }
  .osd spinbutton:not(.vertical) button:disabled {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none;
    color: rgba(255, 255, 255, 0.3);
    border-color: #1C1C1E;
    -gtk-icon-shadow: none;
    box-shadow: none; }
  .osd spinbutton:not(.vertical) button:dir(ltr):last-child, .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.flat:dir(ltr), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:last-child > button.flat:dir(ltr), .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:dir(ltr), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:last-child > button.sidebar-button:dir(ltr), .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:dir(ltr), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:last-child > button.titlebutton:dir(ltr), .osd spinbutton:not(.vertical) toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:dir(ltr), .osd spinbutton:not(.vertical) combobox.linked button:dir(ltr):nth-child(2), combobox.linked .osd spinbutton:not(.vertical) button:dir(ltr):nth-child(2), .osd spinbutton:not(.vertical) .linked:not(.vertical) > combobox:last-child > box > button.combo:dir(ltr), .osd spinbutton:not(.vertical) .linked.vertical > combobox:last-child > box > button.combo:dir(ltr) {
    border-radius: 0 3px 3px 0; }
  .osd spinbutton:not(.vertical) button:dir(rtl):first-child, .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.flat:dir(rtl), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:first-child > button.flat:dir(rtl), .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:dir(rtl), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:first-child > button.sidebar-button:dir(rtl), .osd spinbutton:not(.vertical) toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:dir(rtl), toolbar.inline-toolbar .osd spinbutton:not(.vertical) toolbutton:first-child > button.titlebutton:dir(rtl), .osd spinbutton:not(.vertical) toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:dir(rtl), .osd spinbutton:not(.vertical) combobox.linked button:dir(rtl):nth-child(2), combobox.linked .osd spinbutton:not(.vertical) button:dir(rtl):nth-child(2), .osd spinbutton:not(.vertical) .linked:not(.vertical) > combobox:first-child > box > button.combo:dir(rtl), .osd spinbutton:not(.vertical) .linked.vertical > combobox:first-child > box > button.combo:dir(rtl) {
    border-radius: 3px 0 0 3px; }
spinbutton.vertical:disabled {
  color: rgba(255, 255, 255, 0.3); }
spinbutton.vertical:backdrop:disabled {
  color: rgba(255, 255, 255, 0.3); }
spinbutton.vertical:drop(active) {
  border-color: transparent;
  box-shadow: none; }
spinbutton.vertical entry {
  min-height: 32px;
  min-width: 32px;
  padding: 0;
  border-radius: 0; }
spinbutton.vertical button {
  min-height: 32px;
  min-width: 32px;
  padding: 0; }
spinbutton.vertical button.up {
  border-radius: 3px 3px 0 0;
  border-style: solid solid none solid; }
spinbutton.vertical button.down {
  border-radius: 0 0 3px 3px;
  border-style: none solid solid solid; }
.osd spinbutton.vertical button:first-child, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.flat, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.flat, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.sidebar-button, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.titlebutton, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.titlebutton, .osd spinbutton.vertical toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton, .osd spinbutton.vertical combobox.linked button:nth-child(2):dir(rtl), combobox.linked .osd spinbutton.vertical button:nth-child(2):dir(rtl), .osd spinbutton.vertical .linked:not(.vertical) > combobox:first-child > box > button.combo, .osd spinbutton.vertical .linked.vertical > combobox:first-child > box > button.combo {
  color: #ffffff;
  border-color: rgba(0, 0, 0, 0.7);
  background-color: transparent;
  background-image: image(rgba(28, 31, 32, 0.7));
  background-clip: padding-box;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
  text-shadow: 0 1px white;
  -gtk-icon-shadow: 0 1px white;
  outline-color: rgba(255, 255, 255, 0.3); }
  .osd spinbutton.vertical button:first-child:hover, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.flat:hover, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.flat:hover, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:hover, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.sidebar-button:hover, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:hover, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.titlebutton:hover, .osd spinbutton.vertical toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:hover, .osd spinbutton.vertical combobox.linked button:hover:nth-child(2):dir(rtl), combobox.linked .osd spinbutton.vertical button:hover:nth-child(2):dir(rtl), .osd spinbutton.vertical .linked:not(.vertical) > combobox:first-child > box > button.combo:hover, .osd spinbutton.vertical .linked.vertical > combobox:first-child > box > button.combo:hover {
    color: white;
    border-color: rgba(0, 0, 0, 0.7);
    background-color: transparent;
    background-image: image(rgba(57, 63, 65, 0.7));
    background-clip: padding-box;
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
    text-shadow: 0 1px white;
    -gtk-icon-shadow: 0 1px white;
    outline-color: rgba(255, 255, 255, 0.3); }
  .osd spinbutton.vertical button:first-child:active, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.flat:active, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.flat:active, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:active, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.sidebar-button:active, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:active, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.titlebutton:active, .osd spinbutton.vertical toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:active, .osd spinbutton.vertical combobox.linked button:active:nth-child(2):dir(rtl), combobox.linked .osd spinbutton.vertical button:active:nth-child(2):dir(rtl), .osd spinbutton.vertical .linked:not(.vertical) > combobox:first-child > box > button.combo:active, .osd spinbutton.vertical .linked.vertical > combobox:first-child > box > button.combo:active {
    color: white;
    border-color: rgba(0, 0, 0, 0.7);
    background-color: transparent;
    background-image: image(rgba(0, 0, 0, 0.7));
    background-clip: padding-box;
    box-shadow: none;
    text-shadow: none;
    -gtk-icon-shadow: none;
    outline-color: rgba(255, 255, 255, 0.3); }
  .osd spinbutton.vertical button:first-child:disabled, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.flat:disabled, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.flat:disabled, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:disabled, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.sidebar-button:disabled, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:disabled, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.titlebutton:disabled, .osd spinbutton.vertical toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:disabled, .osd spinbutton.vertical combobox.linked button:disabled:nth-child(2):dir(rtl), combobox.linked .osd spinbutton.vertical button:disabled:nth-child(2):dir(rtl), .osd spinbutton.vertical .linked:not(.vertical) > combobox:first-child > box > button.combo:disabled, .osd spinbutton.vertical .linked.vertical > combobox:first-child > box > button.combo:disabled {
    color: rgba(255, 255, 255, 0.3);
    border-color: rgba(0, 0, 0, 0.7);
    background-color: transparent;
    background-image: image(#1C1C1E);
    background-clip: padding-box;
    box-shadow: none;
    text-shadow: none;
    -gtk-icon-shadow: none; }
  .osd spinbutton.vertical button:first-child:backdrop, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.flat:backdrop, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.flat:backdrop, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button:backdrop, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.sidebar-button:backdrop, .osd spinbutton.vertical toolbar.inline-toolbar toolbutton:first-child > button.titlebutton:backdrop, toolbar.inline-toolbar .osd spinbutton.vertical toolbutton:first-child > button.titlebutton:backdrop, .osd spinbutton.vertical toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton:backdrop, .osd spinbutton.vertical combobox.linked button:backdrop:nth-child(2):dir(rtl), combobox.linked .osd spinbutton.vertical button:backdrop:nth-child(2):dir(rtl), .osd spinbutton.vertical .linked:not(.vertical) > combobox:first-child > box > button.combo:backdrop, .osd spinbutton.vertical .linked.vertical > combobox:first-child > box > button.combo:backdrop {
    color: #ffffff;
    border-color: rgba(0, 0, 0, 0.7);
    background-color: transparent;
    background-image: image(rgba(28, 31, 32, 0.7));
    background-clip: padding-box;
    box-shadow: none;
    text-shadow: none;
    -gtk-icon-shadow: none; }
treeview spinbutton:not(.vertical) {
  min-height: 0;
  border-style: none;
  border-radius: 0; }
  treeview spinbutton:not(.vertical) entry {
    min-height: 0;
    padding: 1px 2px; }

/*******************
 * ComboBoxes *
 *******************/
combobox arrow {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
  min-height: 16px;
  min-width: 16px; }
combobox:drop(active) {
  box-shadow: none; }

/********************
 * Header bars *
 ********************/
.titlebar:not(headerbar), headerbar {
  padding: 0 6px;
  min-height: 46px;
  border-radius: 0;
  background: #18181a linear-gradient(to top, black, #131315 2px, #18181a 3px);
  box-shadow: none;
  border-style: none;
  box-shadow: none; }
  label:backdrop selection.titlebar:not(headerbar), .titlebar:backdrop:not(headerbar), progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop progress.titlebar:not(headerbar),
  headerbar:backdrop {
    border-style: none;
    background-image: none;
    box-shadow: inset 0 1px rgba(0, 0, 0, 0.8);
    transition: 200ms ease-out; }
  .titlebar:not(headerbar) .title,
  headerbar .title {
    border-style: none;
    padding-left: 12px;
    padding-right: 12px;
    font-weight: bold; }
  .titlebar:not(headerbar) .subtitle,
  headerbar .subtitle {
    font-size: smaller;
    padding-left: 12px;
    padding-right: 12px;
    border-style: none; }
  .selection-mode.titlebar:not(headerbar),
  headerbar.selection-mode {
    color: #fff;
    text-shadow: none;
    background: #3790FA linear-gradient(to top, #066de9, #2d8afa 2px, #3790FA 3px);
    box-shadow: none;
    border-style: none; }
    label:backdrop selection.selection-mode.titlebar:not(headerbar), .selection-mode.titlebar:backdrop:not(headerbar), progressbar:backdrop trough.selection-mode.titlebar:not(headerbar), progressbar:backdrop progress.selection-mode.titlebar:not(headerbar),
    headerbar.selection-mode:backdrop {
      background-color: #3790FA;
      background-image: none;
      border-style: none; }
    .selection-mode.titlebar:not(headerbar) .subtitle:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link,
    headerbar.selection-mode .subtitle:link,
    headerbar.selection-mode button.subtitle:visited {
      border-style: none; }
    .selection-mode.titlebar:not(headerbar) button,
    headerbar.selection-mode button {
      color: #fff;
      outline-color: rgba(255, 255, 255, 0.3);
      border-color: #0c0c0d;
      background-image: image(#0c0c0d);
      border-style: none; }
      .selection-mode.titlebar:not(headerbar) button.flat, .selection-mode.titlebar:not(headerbar) button.sidebar-button, .selection-mode.titlebar:not(headerbar) button.titlebutton,
      headerbar.selection-mode button.flat,
      headerbar.selection-mode button.sidebar-button,
      headerbar.selection-mode button.titlebutton {
        border-color: transparent;
        background-color: transparent;
        text-shadow: none;
        -gtk-icon-shadow: none;
        border-style: none; }
      .selection-mode.titlebar:not(headerbar) button:hover,
      headerbar.selection-mode button:hover {
        color: #fff;
        outline-color: rgba(255, 255, 255, 0.3);
        border-color: #0c0c0d;
        background-image: image(#0c0c0d);
        border-style: none;
        color: #fff;
        background-color: #3790FA; }
      .selection-mode.titlebar:not(headerbar) button:active, .selection-mode.titlebar:not(headerbar) button:checked,
      headerbar.selection-mode button:active,
      headerbar.selection-mode button:checked {
        color: #fff;
        outline-color: rgba(255, 255, 255, 0.3);
        border-color: #0c0c0d;
        background-image: image(#0c0c0d);
        text-shadow: none;
        -gtk-icon-shadow: none;
        border-style: none;
        color: #fff;
        background-color: #3790FA; }
      .selection-mode.titlebar:not(headerbar) button:backdrop.flat, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop, .selection-mode.titlebar:not(headerbar) button:backdrop,
      headerbar.selection-mode button:backdrop.flat,
      headerbar.selection-mode button.sidebar-button:backdrop,
      headerbar.selection-mode button.titlebutton:backdrop,
      headerbar.selection-mode button:backdrop {
        border-color: #0c0c0d;
        background-image: image(#3790FA);
        text-shadow: none;
        -gtk-icon-shadow: none;
        -gtk-icon-effect: none;
        border-color: #3790FA;
        border-style: none; }
        .selection-mode.titlebar:not(headerbar) button:backdrop.flat label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop label, .selection-mode.titlebar:not(headerbar) button:backdrop.flat, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop, .selection-mode.titlebar:not(headerbar) button:backdrop label, .selection-mode.titlebar:not(headerbar) button:backdrop,
        headerbar.selection-mode button:backdrop.flat label,
        headerbar.selection-mode button.sidebar-button:backdrop label,
        headerbar.selection-mode button.titlebutton:backdrop label,
        headerbar.selection-mode button:backdrop.flat,
        headerbar.selection-mode button.sidebar-button:backdrop,
        headerbar.selection-mode button.titlebutton:backdrop,
        headerbar.selection-mode button:backdrop label,
        headerbar.selection-mode button:backdrop {
          color: #ffffff; }
        .selection-mode.titlebar:not(headerbar) button:backdrop.flat:active, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:active, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:active, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:checked, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:checked, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:checked, .selection-mode.titlebar:not(headerbar) button:backdrop:active, .selection-mode.titlebar:not(headerbar) button:backdrop:checked,
        headerbar.selection-mode button:backdrop.flat:active,
        headerbar.selection-mode button.sidebar-button:backdrop:active,
        headerbar.selection-mode button.titlebutton:backdrop:active,
        headerbar.selection-mode button:backdrop.flat:checked,
        headerbar.selection-mode button.sidebar-button:backdrop:checked,
        headerbar.selection-mode button.titlebutton:backdrop:checked,
        headerbar.selection-mode button:backdrop:active,
        headerbar.selection-mode button:backdrop:checked {
          border-color: #2568b9;
          background-image: image(#2568b9);
          color: #fff;
          background-color: #3790FA;
          border-color: #3790FA;
          border-style: none; }
          .selection-mode.titlebar:not(headerbar) button:backdrop.flat:active label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:active label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:active label, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:active, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:active, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:active, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:checked label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:checked label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:checked label, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:checked, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:checked, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:checked, .selection-mode.titlebar:not(headerbar) button:backdrop:active label, .selection-mode.titlebar:not(headerbar) button:backdrop:active, .selection-mode.titlebar:not(headerbar) button:backdrop:checked label, .selection-mode.titlebar:not(headerbar) button:backdrop:checked,
          headerbar.selection-mode button:backdrop.flat:active label,
          headerbar.selection-mode button.sidebar-button:backdrop:active label,
          headerbar.selection-mode button.titlebutton:backdrop:active label,
          headerbar.selection-mode button:backdrop.flat:active,
          headerbar.selection-mode button.sidebar-button:backdrop:active,
          headerbar.selection-mode button.titlebutton:backdrop:active,
          headerbar.selection-mode button:backdrop.flat:checked label,
          headerbar.selection-mode button.sidebar-button:backdrop:checked label,
          headerbar.selection-mode button.titlebutton:backdrop:checked label,
          headerbar.selection-mode button:backdrop.flat:checked,
          headerbar.selection-mode button.sidebar-button:backdrop:checked,
          headerbar.selection-mode button.titlebutton:backdrop:checked,
          headerbar.selection-mode button:backdrop:active label,
          headerbar.selection-mode button:backdrop:active,
          headerbar.selection-mode button:backdrop:checked label,
          headerbar.selection-mode button:backdrop:checked {
            color: #ffffff; }
        .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled,
        headerbar.selection-mode button:backdrop.flat:disabled,
        headerbar.selection-mode button.sidebar-button:backdrop:disabled,
        headerbar.selection-mode button.titlebutton:backdrop:disabled,
        headerbar.selection-mode button:backdrop:disabled {
          border-color: #317cd6;
          background-image: image(#317cd6);
          text-shadow: none;
          -gtk-icon-shadow: none;
          border-color: #1C1C1E;
          border-style: none; }
          .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled label, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled label, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled,
          headerbar.selection-mode button:backdrop.flat:disabled label,
          headerbar.selection-mode button.sidebar-button:backdrop:disabled label,
          headerbar.selection-mode button.titlebutton:backdrop:disabled label,
          headerbar.selection-mode button:backdrop.flat:disabled,
          headerbar.selection-mode button.sidebar-button:backdrop:disabled,
          headerbar.selection-mode button.titlebutton:backdrop:disabled,
          headerbar.selection-mode button:backdrop:disabled label,
          headerbar.selection-mode button:backdrop:disabled {
            color: #79aae4; }
          .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled:active, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled:active, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled:active, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled:checked, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled:checked, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled:checked, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:active, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:checked,
          headerbar.selection-mode button:backdrop.flat:disabled:active,
          headerbar.selection-mode button.sidebar-button:backdrop:disabled:active,
          headerbar.selection-mode button.titlebutton:backdrop:disabled:active,
          headerbar.selection-mode button:backdrop.flat:disabled:checked,
          headerbar.selection-mode button.sidebar-button:backdrop:disabled:checked,
          headerbar.selection-mode button.titlebutton:backdrop:disabled:checked,
          headerbar.selection-mode button:backdrop:disabled:active,
          headerbar.selection-mode button:backdrop:disabled:checked {
            border-color: #2568b9;
            background-image: image(#2568b9);
            color: #fff;
            background-color: #3790FA;
            border-color: #3790FA;
            border-style: none; }
            .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled:active label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled:active label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled:active label, .selection-mode.titlebar:not(headerbar) button:backdrop.flat:disabled:checked label, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled:checked label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled:checked label, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:active label, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:checked label,
            headerbar.selection-mode button:backdrop.flat:disabled:active label,
            headerbar.selection-mode button.sidebar-button:backdrop:disabled:active label,
            headerbar.selection-mode button.titlebutton:backdrop:disabled:active label,
            headerbar.selection-mode button:backdrop.flat:disabled:checked label,
            headerbar.selection-mode button.sidebar-button:backdrop:disabled:checked label,
            headerbar.selection-mode button.titlebutton:backdrop:disabled:checked label,
            headerbar.selection-mode button:backdrop:disabled:active label,
            headerbar.selection-mode button:backdrop:disabled:checked label {
              color: #719dd2; }
      .selection-mode.titlebar:not(headerbar) button.flat:backdrop, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop, .selection-mode.titlebar:not(headerbar) button.flat:disabled, .selection-mode.titlebar:not(headerbar) button.sidebar-button:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:disabled, .selection-mode.titlebar:not(headerbar) button.flat:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.sidebar-button:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled,
      headerbar.selection-mode button.flat:backdrop,
      headerbar.selection-mode button.sidebar-button:backdrop,
      headerbar.selection-mode button.titlebutton:backdrop,
      headerbar.selection-mode button.flat:disabled,
      headerbar.selection-mode button.sidebar-button:disabled,
      headerbar.selection-mode button.titlebutton:disabled,
      headerbar.selection-mode button.flat:backdrop:disabled,
      headerbar.selection-mode button.sidebar-button:backdrop:disabled,
      headerbar.selection-mode button.titlebutton:backdrop:disabled {
        border-color: transparent;
        background-color: transparent;
        text-shadow: none;
        -gtk-icon-shadow: none;
        border-style: none; }
      .selection-mode.titlebar:not(headerbar) button:disabled,
      headerbar.selection-mode button:disabled {
        border-color: #1C1C1E;
        background-image: image(#317cd6);
        text-shadow: none;
        -gtk-icon-shadow: none; }
        .selection-mode.titlebar:not(headerbar) button:disabled label, .selection-mode.titlebar:not(headerbar) button:disabled,
        headerbar.selection-mode button:disabled label,
        headerbar.selection-mode button:disabled {
          color: rgba(255, 255, 255, 0.3); }
        .selection-mode.titlebar:not(headerbar) button:disabled:active, .selection-mode.titlebar:not(headerbar) button:disabled:checked,
        headerbar.selection-mode button:disabled:active,
        headerbar.selection-mode button:disabled:checked {
          border-color: #055dc6;
          background-image: image(#2568b9); }
          .selection-mode.titlebar:not(headerbar) button:disabled:active label, .selection-mode.titlebar:not(headerbar) button:disabled:active, .selection-mode.titlebar:not(headerbar) button:disabled:checked label, .selection-mode.titlebar:not(headerbar) button:disabled:checked,
          headerbar.selection-mode button:disabled:active label,
          headerbar.selection-mode button:disabled:active,
          headerbar.selection-mode button:disabled:checked label,
          headerbar.selection-mode button:disabled:checked {
            color: #a8c3e3; }
      .selection-mode.titlebar:not(headerbar) button.suggested-action, .selection-mode.titlebar:not(headerbar) button.titlebutton,
      headerbar.selection-mode button.suggested-action,
      headerbar.selection-mode button.titlebutton {
        color: #ffffff;
        outline-color: rgba(255, 255, 255, 0.3);
        border-color: #0c0c0d;
        background-image: image(#0c0c0d);
        border-color: #3790FA; }
        .selection-mode.titlebar:not(headerbar) button.suggested-action:hover, .selection-mode.titlebar:not(headerbar) button.titlebutton:hover,
        headerbar.selection-mode button.suggested-action:hover,
        headerbar.selection-mode button.titlebutton:hover {
          color: #ffffff;
          outline-color: rgba(255, 255, 255, 0.3);
          border-color: #0c0c0d;
          background-image: image(#0c0c0d);
          border-color: #3790FA; }
        .selection-mode.titlebar:not(headerbar) button.suggested-action:active, .selection-mode.titlebar:not(headerbar) button.titlebutton:active,
        headerbar.selection-mode button.suggested-action:active,
        headerbar.selection-mode button.titlebutton:active {
          color: #ffffff;
          outline-color: rgba(255, 255, 255, 0.3);
          border-color: #0c0c0d;
          background-image: image(#0c0c0d);
          text-shadow: none;
          -gtk-icon-shadow: none;
          border-color: #3790FA; }
        .selection-mode.titlebar:not(headerbar) button.suggested-action:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:disabled,
        headerbar.selection-mode button.suggested-action:disabled,
        headerbar.selection-mode button.titlebutton:disabled {
          border-color: #1C1C1E;
          background-image: image(#1C1C1E);
          text-shadow: none;
          -gtk-icon-shadow: none;
          border-color: #1C1C1E; }
          .selection-mode.titlebar:not(headerbar) button.suggested-action:disabled label, .selection-mode.titlebar:not(headerbar) button.titlebutton:disabled label, .selection-mode.titlebar:not(headerbar) button.suggested-action:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:disabled,
          headerbar.selection-mode button.suggested-action:disabled label,
          headerbar.selection-mode button.titlebutton:disabled label,
          headerbar.selection-mode button.suggested-action:disabled,
          headerbar.selection-mode button.titlebutton:disabled {
            color: rgba(255, 255, 255, 0.3); }
        .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop,
        headerbar.selection-mode button.suggested-action:backdrop,
        headerbar.selection-mode button.titlebutton:backdrop {
          border-color: #0c0c0d;
          background-image: image(#2D2E32);
          text-shadow: none;
          -gtk-icon-shadow: none;
          border-color: #3790FA; }
          .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop label, .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop,
          headerbar.selection-mode button.suggested-action:backdrop label,
          headerbar.selection-mode button.titlebutton:backdrop label,
          headerbar.selection-mode button.suggested-action:backdrop,
          headerbar.selection-mode button.titlebutton:backdrop {
            color: #ffffff; }
        .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled,
        headerbar.selection-mode button.suggested-action:backdrop:disabled,
        headerbar.selection-mode button.titlebutton:backdrop:disabled {
          border-color: #0c0c0d;
          background-image: image(#1C1C1E);
          text-shadow: none;
          -gtk-icon-shadow: none;
          border-color: #3790FA; }
          .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop:disabled label, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled label, .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.titlebutton:backdrop:disabled,
          headerbar.selection-mode button.suggested-action:backdrop:disabled label,
          headerbar.selection-mode button.titlebutton:backdrop:disabled label,
          headerbar.selection-mode button.suggested-action:backdrop:disabled,
          headerbar.selection-mode button.titlebutton:backdrop:disabled {
            color: rgba(255, 255, 255, 0.3); }
    .selection-mode.titlebar:not(headerbar) .selection-menu:backdrop, .selection-mode.titlebar:not(headerbar) label:backdrop selection.selection-menu, label:backdrop .selection-mode.titlebar:not(headerbar) selection.selection-menu, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.selection-menu, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.selection-menu, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.selection-menu, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.selection-menu, .selection-mode.titlebar:not(headerbar) .selection-menu,
    headerbar.selection-mode .selection-menu:backdrop,
    headerbar.selection-mode label:backdrop selection.selection-menu,
    label:backdrop headerbar.selection-mode selection.selection-menu,
    headerbar.selection-mode progressbar:backdrop trough.selection-menu,
    progressbar:backdrop headerbar.selection-mode trough.selection-menu,
    headerbar.selection-mode progressbar:backdrop progress.selection-menu,
    progressbar:backdrop headerbar.selection-mode progress.selection-menu,
    headerbar.selection-mode .selection-menu {
      border-color: rgba(61, 62, 66, 0);
      background-color: rgba(61, 62, 66, 0);
      background-image: none;
      box-shadow: none;
      padding-left: 10px;
      padding-right: 10px; }
      .selection-mode.titlebar:not(headerbar) .selection-menu:backdrop arrow, .selection-mode.titlebar:not(headerbar) label:backdrop selection.selection-menu arrow, label:backdrop .selection-mode.titlebar:not(headerbar) selection.selection-menu arrow, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.selection-menu arrow, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.selection-menu arrow, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.selection-menu arrow, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.selection-menu arrow, .selection-mode.titlebar:not(headerbar) .selection-menu arrow,
      headerbar.selection-mode .selection-menu:backdrop arrow,
      headerbar.selection-mode label:backdrop selection.selection-menu arrow,
      label:backdrop headerbar.selection-mode selection.selection-menu arrow,
      headerbar.selection-mode progressbar:backdrop trough.selection-menu arrow,
      progressbar:backdrop headerbar.selection-mode trough.selection-menu arrow,
      headerbar.selection-mode progressbar:backdrop progress.selection-menu arrow,
      progressbar:backdrop headerbar.selection-mode progress.selection-menu arrow,
      headerbar.selection-mode .selection-menu arrow {
        -GtkArrow-arrow-scaling: 1; }
      .selection-mode.titlebar:not(headerbar) .selection-menu:backdrop .arrow, .selection-mode.titlebar:not(headerbar) label:backdrop selection.selection-menu .arrow, label:backdrop .selection-mode.titlebar:not(headerbar) selection.selection-menu .arrow, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.selection-menu .arrow, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.selection-menu .arrow, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.selection-menu .arrow, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.selection-menu .arrow, .selection-mode.titlebar:not(headerbar) .selection-menu .arrow,
      headerbar.selection-mode .selection-menu:backdrop .arrow,
      headerbar.selection-mode label:backdrop selection.selection-menu .arrow,
      label:backdrop headerbar.selection-mode selection.selection-menu .arrow,
      headerbar.selection-mode progressbar:backdrop trough.selection-menu .arrow,
      progressbar:backdrop headerbar.selection-mode trough.selection-menu .arrow,
      headerbar.selection-mode progressbar:backdrop progress.selection-menu .arrow,
      progressbar:backdrop headerbar.selection-mode progress.selection-menu .arrow,
      headerbar.selection-mode .selection-menu .arrow {
        -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
        color: rgba(255, 255, 255, 0.5);
        -gtk-icon-shadow: none; }
  .tiled label:backdrop selection.titlebar:not(headerbar), label:backdrop .tiled selection.titlebar:not(headerbar), .tiled .titlebar:backdrop:not(headerbar), .tiled progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .tiled trough.titlebar:not(headerbar), .tiled progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .tiled progress.titlebar:not(headerbar), .tiled .titlebar:not(headerbar), .tiled-top label:backdrop selection.titlebar:not(headerbar), label:backdrop .tiled-top selection.titlebar:not(headerbar), .tiled-top .titlebar:backdrop:not(headerbar), .tiled-top progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .tiled-top trough.titlebar:not(headerbar), .tiled-top progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .tiled-top progress.titlebar:not(headerbar), .tiled-top .titlebar:not(headerbar), .tiled-right label:backdrop selection.titlebar:not(headerbar), label:backdrop .tiled-right selection.titlebar:not(headerbar), .tiled-right .titlebar:backdrop:not(headerbar), .tiled-right progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .tiled-right trough.titlebar:not(headerbar), .tiled-right progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .tiled-right progress.titlebar:not(headerbar), .tiled-right .titlebar:not(headerbar), .tiled-bottom label:backdrop selection.titlebar:not(headerbar), label:backdrop .tiled-bottom selection.titlebar:not(headerbar), .tiled-bottom .titlebar:backdrop:not(headerbar), .tiled-bottom progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .tiled-bottom trough.titlebar:not(headerbar), .tiled-bottom progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .tiled-bottom progress.titlebar:not(headerbar), .tiled-bottom .titlebar:not(headerbar), .tiled-left label:backdrop selection.titlebar:not(headerbar), label:backdrop .tiled-left selection.titlebar:not(headerbar), .tiled-left .titlebar:backdrop:not(headerbar), .tiled-left progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .tiled-left trough.titlebar:not(headerbar), .tiled-left progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .tiled-left progress.titlebar:not(headerbar), .tiled-left .titlebar:not(headerbar), .maximized label:backdrop selection.titlebar:not(headerbar), label:backdrop .maximized selection.titlebar:not(headerbar), .maximized .titlebar:backdrop:not(headerbar), .maximized progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .maximized trough.titlebar:not(headerbar), .maximized progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .maximized progress.titlebar:not(headerbar), .maximized .titlebar:not(headerbar), .fullscreen label:backdrop selection.titlebar:not(headerbar), label:backdrop .fullscreen selection.titlebar:not(headerbar), .fullscreen .titlebar:backdrop:not(headerbar), .fullscreen progressbar:backdrop trough.titlebar:not(headerbar), progressbar:backdrop .fullscreen trough.titlebar:not(headerbar), .fullscreen progressbar:backdrop progress.titlebar:not(headerbar), progressbar:backdrop .fullscreen progress.titlebar:not(headerbar), .fullscreen .titlebar:not(headerbar),
  .tiled headerbar:backdrop,
  .tiled headerbar, .tiled-top headerbar:backdrop, .tiled-top headerbar, .tiled-right headerbar:backdrop, .tiled-right headerbar, .tiled-bottom headerbar:backdrop, .tiled-bottom headerbar, .tiled-left headerbar:backdrop, .tiled-left headerbar, .maximized headerbar:backdrop, .maximized headerbar, .fullscreen headerbar:backdrop, .fullscreen headerbar {
    border-radius: 0; }
  .default-decoration.titlebar:not(headerbar), headerbar.default-decoration {
    min-height: 30px;
    padding: 0px; }
    .default-decoration.titlebar:not(headerbar) button.titlebutton, headerbar.default-decoration button.titlebutton {
      min-height: 30px;
      min-width: 45px;
      margin: 0;
      margin-right: -1px;
      padding: 0; }
  .solid-csd label:backdrop selection.titlebar:dir(rtl):not(headerbar), label:backdrop .solid-csd selection.titlebar:dir(rtl):not(headerbar), .solid-csd .titlebar:backdrop:dir(rtl):not(headerbar), .solid-csd progressbar:backdrop trough.titlebar:dir(rtl):not(headerbar), progressbar:backdrop .solid-csd trough.titlebar:dir(rtl):not(headerbar), .solid-csd progressbar:backdrop progress.titlebar:dir(rtl):not(headerbar), progressbar:backdrop .solid-csd progress.titlebar:dir(rtl):not(headerbar), .solid-csd label:backdrop selection.titlebar:dir(ltr):not(headerbar), label:backdrop .solid-csd selection.titlebar:dir(ltr):not(headerbar), .solid-csd .titlebar:backdrop:dir(ltr):not(headerbar), .solid-csd progressbar:backdrop trough.titlebar:dir(ltr):not(headerbar), progressbar:backdrop .solid-csd trough.titlebar:dir(ltr):not(headerbar), .solid-csd progressbar:backdrop progress.titlebar:dir(ltr):not(headerbar), progressbar:backdrop .solid-csd progress.titlebar:dir(ltr):not(headerbar), .solid-csd .titlebar:dir(rtl):not(headerbar), .solid-csd .titlebar:dir(ltr):not(headerbar), .solid-csd headerbar:backdrop:dir(rtl), .solid-csd headerbar:backdrop:dir(ltr), .solid-csd headerbar:dir(rtl), .solid-csd headerbar:dir(ltr) {
    margin-left: -1px;
    margin-right: -1px;
    margin-top: -1px;
    border-radius: 0;
    box-shadow: none; }

headerbar entry,
headerbar spinbutton,
headerbar separator,
headerbar button {
  margin-top: 6px;
  margin-bottom: 6px;
  border-radius: 0;
  box-shadow: none; }
headerbar switch {
  margin-top: 9px;
  margin-bottom: 9px; }

.background.tiled .titlebar:backdrop, .background.tiled label:backdrop selection.titlebar, label:backdrop .background.tiled selection.titlebar, .background.tiled progressbar:backdrop trough.titlebar, progressbar:backdrop .background.tiled trough.titlebar, .background.tiled progressbar:backdrop progress.titlebar, progressbar:backdrop .background.tiled progress.titlebar, .background.tiled .titlebar,
.background.tiled-top .titlebar:backdrop,
.background.tiled-top label:backdrop selection.titlebar,
label:backdrop .background.tiled-top selection.titlebar,
.background.tiled-top progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.tiled-top trough.titlebar,
.background.tiled-top progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.tiled-top progress.titlebar,
.background.tiled-top .titlebar,
.background.tiled-right .titlebar:backdrop,
.background.tiled-right label:backdrop selection.titlebar,
label:backdrop .background.tiled-right selection.titlebar,
.background.tiled-right progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.tiled-right trough.titlebar,
.background.tiled-right progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.tiled-right progress.titlebar,
.background.tiled-right .titlebar,
.background.tiled-bottom .titlebar:backdrop,
.background.tiled-bottom label:backdrop selection.titlebar,
label:backdrop .background.tiled-bottom selection.titlebar,
.background.tiled-bottom progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.tiled-bottom trough.titlebar,
.background.tiled-bottom progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.tiled-bottom progress.titlebar,
.background.tiled-bottom .titlebar,
.background.tiled-left .titlebar:backdrop,
.background.tiled-left label:backdrop selection.titlebar,
label:backdrop .background.tiled-left selection.titlebar,
.background.tiled-left progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.tiled-left trough.titlebar,
.background.tiled-left progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.tiled-left progress.titlebar,
.background.tiled-left .titlebar,
.background.maximized .titlebar:backdrop,
.background.maximized label:backdrop selection.titlebar,
label:backdrop .background.maximized selection.titlebar,
.background.maximized progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.maximized trough.titlebar,
.background.maximized progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.maximized progress.titlebar,
.background.maximized .titlebar,
.background.solid-csd .titlebar:backdrop,
.background.solid-csd label:backdrop selection.titlebar,
label:backdrop .background.solid-csd selection.titlebar,
.background.solid-csd progressbar:backdrop trough.titlebar,
progressbar:backdrop .background.solid-csd trough.titlebar,
.background.solid-csd progressbar:backdrop progress.titlebar,
progressbar:backdrop .background.solid-csd progress.titlebar,
.background.solid-csd .titlebar {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
  border-style: none;
  box-shadow: none; }

window separator:first-child + headerbar:backdrop, window separator:first-child + headerbar, window headerbar:first-child:backdrop, window headerbar:first-child {
  border-top-left-radius: 0;
  border-style: none;
  box-shadow: none; }
window headerbar:last-child:backdrop, window headerbar:last-child {
  border-top-right-radius: 0;
  border-style: none;
  box-shadow: none; }
window stack headerbar:first-child:backdrop, window stack headerbar:first-child, window stack headerbar:last-child:backdrop, window stack headerbar:last-child {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
  border-style: none;
  box-shadow: none; }
window.tiled headerbar, window.tiled headerbar:first-child, window.tiled headerbar:last-child, window.tiled headerbar:only-child, window.tiled headerbar:backdrop, window.tiled headerbar:backdrop:first-child, window.tiled headerbar:backdrop:last-child, window.tiled headerbar:backdrop:only-child, window.tiled-top headerbar, window.tiled-top headerbar:first-child, window.tiled-top headerbar:last-child, window.tiled-top headerbar:only-child, window.tiled-top headerbar:backdrop, window.tiled-top headerbar:backdrop:first-child, window.tiled-top headerbar:backdrop:last-child, window.tiled-top headerbar:backdrop:only-child, window.tiled-right headerbar, window.tiled-right headerbar:first-child, window.tiled-right headerbar:last-child, window.tiled-right headerbar:only-child, window.tiled-right headerbar:backdrop, window.tiled-right headerbar:backdrop:first-child, window.tiled-right headerbar:backdrop:last-child, window.tiled-right headerbar:backdrop:only-child, window.tiled-bottom headerbar, window.tiled-bottom headerbar:first-child, window.tiled-bottom headerbar:last-child, window.tiled-bottom headerbar:only-child, window.tiled-bottom headerbar:backdrop, window.tiled-bottom headerbar:backdrop:first-child, window.tiled-bottom headerbar:backdrop:last-child, window.tiled-bottom headerbar:backdrop:only-child, window.tiled-left headerbar, window.tiled-left headerbar:first-child, window.tiled-left headerbar:last-child, window.tiled-left headerbar:only-child, window.tiled-left headerbar:backdrop, window.tiled-left headerbar:backdrop:first-child, window.tiled-left headerbar:backdrop:last-child, window.tiled-left headerbar:backdrop:only-child, window.maximized headerbar, window.maximized headerbar:first-child, window.maximized headerbar:last-child, window.maximized headerbar:only-child, window.maximized headerbar:backdrop, window.maximized headerbar:backdrop:first-child, window.maximized headerbar:backdrop:last-child, window.maximized headerbar:backdrop:only-child, window.fullscreen headerbar, window.fullscreen headerbar:first-child, window.fullscreen headerbar:last-child, window.fullscreen headerbar:only-child, window.fullscreen headerbar:backdrop, window.fullscreen headerbar:backdrop:first-child, window.fullscreen headerbar:backdrop:last-child, window.fullscreen headerbar:backdrop:only-child, window.solid-csd headerbar, window.solid-csd headerbar:first-child, window.solid-csd headerbar:last-child, window.solid-csd headerbar:only-child, window.solid-csd headerbar:backdrop, window.solid-csd headerbar:backdrop:first-child, window.solid-csd headerbar:backdrop:last-child, window.solid-csd headerbar:backdrop:only-child {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
  border-style: none;
  box-shadow: none; }

window.csd > .titlebar:not(headerbar) {
  padding: 0;
  background-color: transparent;
  background-image: none;
  border-style: none;
  border-color: transparent;
  box-shadow: none; }
.titlebar:not(headerbar) > separator {
  background-color: #0c0c0d; }

/*****************
 * Pathbars *
 *****************/
.path-bar button.text-button, .path-bar button.image-button, .path-bar button {
  padding-left: 4px;
  padding-right: 4px; }
.path-bar button.text-button.image-button label {
  padding-left: 0;
  padding-right: 0; }
.path-bar button.text-button.image-button label:last-child, .path-bar button label:last-child {
  padding-right: 8px; }
.path-bar button.text-button.image-button label:first-child, .path-bar button label:first-child {
  padding-left: 8px; }
.path-bar button image {
  padding-left: 4px;
  padding-right: 4px; }
.path-bar button.slider-button {
  padding-left: 0;
  padding-right: 0; }

/********************
 * Popovers    *
 ********************/
popover.background {
  padding: 2px;
  border-radius: 5px;
  background-color: #0c0c0d;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.3); }
  .csd popover.background, popover.background {
    border: 1px solid #0c0c0d; }
  popover.background:backdrop {
    background-color: #0c0c0d;
    box-shadow: none; }
  popover.background > list,
  popover.background > .view,
  popover.background > iconview,
  popover.background > toolbar {
    border-style: none;
    background-color: transparent; }
  .csd popover.background.touch-selection, .csd popover.background.magnifier, popover.background.touch-selection, popover.background.magnifier {
    border: 1px solid rgba(255, 255, 255, 0.1); }
  popover.background separator {
    margin: 3px; }
  popover.background list separator {
    margin: 0px; }

/******************
 * Notebooks *
 ******************/
notebook > header {
  padding-bottom: 2px;
  background-color: #2D2E32; }
  notebook > header:backdrop {
    border-color: #0c0c0d;
    background-color: #2D2E32; }
  notebook > header tabs {
    margin: 0px; }
  notebook > header.top > tabs {
    margin-bottom: -2px; }
    notebook > header.top > tabs > tab:hover {
      box-shadow: none; }
    notebook > header.top > tabs > tab:backdrop {
      box-shadow: none; }
    notebook > header.top > tabs > tab:checked {
      box-shadow: none; }
  notebook > header.bottom > tabs {
    margin-top: -2px; }
    notebook > header.bottom > tabs > tab:hover {
      box-shadow: none; }
    notebook > header.bottom > tabs > tab:backdrop {
      box-shadow: none; }
    notebook > header.bottom > tabs > tab:checked {
      box-shadow: none; }
  notebook > header.left > tabs {
    margin-right: -2px; }
    notebook > header.left > tabs > tab:hover {
      box-shadow: none; }
    notebook > header.left > tabs > tab:backdrop {
      box-shadow: none; }
    notebook > header.left > tabs > tab:checked {
      box-shadow: none; }
  notebook > header.right > tabs {
    margin-left: -2px; }
    notebook > header.right > tabs > tab:hover {
      box-shadow: none; }
    notebook > header.right > tabs > tab:backdrop {
      box-shadow: none; }
    notebook > header.right > tabs > tab:checked {
      box-shadow: none; }
  notebook > header.top > tabs > arrow {
    border-top-style: none; }
  notebook > header.bottom > tabs > arrow {
    border-bottom-style: none; }
  notebook > header.top > tabs > arrow, notebook > header.bottom > tabs > arrow {
    margin-left: -5px;
    margin-right: -5px;
    padding-left: 4px;
    padding-right: 4px; }
    notebook > header.top > tabs > arrow.down, notebook > header.bottom > tabs > arrow.down {
      -gtk-icon-source: -gtk-icontheme("pan-start-symbolic"); }
    notebook > header.top > tabs > arrow.up, notebook > header.bottom > tabs > arrow.up {
      -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }
  notebook > header.left > tabs > arrow {
    border-left-style: none; }
  notebook > header.right > tabs > arrow {
    border-right-style: none; }
  notebook > header.left > tabs > arrow, notebook > header.right > tabs > arrow {
    margin-top: -5px;
    margin-bottom: -5px;
    padding-top: 4px;
    padding-bottom: 4px; }
    notebook > header.left > tabs > arrow.down, notebook > header.right > tabs > arrow.down {
      -gtk-icon-source: -gtk-icontheme("pan-up-symbolic"); }
    notebook > header.left > tabs > arrow.up, notebook > header.right > tabs > arrow.up {
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
  notebook > header > tabs > arrow {
    min-height: 16px;
    min-width: 16px;
    border-radius: 0; }
    notebook > header > tabs > arrow:hover:not(:active):not(:backdrop) {
      background-clip: padding-box;
      background-image: none;
      background-color: rgba(255, 255, 255, 0.3);
      border-color: transparent;
      box-shadow: none; }
  notebook > header tab {
    margin-top: 4px;
    border-radius: 6px 6px 0 0;
    min-height: 30px;
    min-width: 30px;
    padding: 3px 12px;
    outline-offset: -5px;
    color: #ffffff;
    background-color: #2D2E32;
    border-color: #0c0c0d; }
    notebook > header tab:hover {
      color: #ffffff; }
      notebook > header tab:hover, notebook > header tab:hover.reorderable-page {
        background-color: #2D2E32; }
    notebook > header tab:backdrop {
      color: #ffffff; }
      notebook > header tab:backdrop, notebook > header tab:backdrop.reorderable-page {
        border-color: #0c0c0d;
        background-color: #2D2E32; }
    notebook > header tab:checked {
      color: #ffffff; }
      notebook > header tab:checked, notebook > header tab:checked.reorderable-page {
        border-color: #0c0c0d;
        background-color: #0c0c0d; }
        notebook > header tab:checked:hover, notebook > header tab:checked.reorderable-page:hover {
          background-color: #0c0c0d; }
    notebook > header tab:backdrop:checked {
      color: #ffffff; }
      notebook > header tab:backdrop:checked, notebook > header tab:backdrop:checked.reorderable-page {
        border-color: #0c0c0d;
        background-color: #0c0c0d; }
    notebook > header tab button.flat, notebook > header tab button.sidebar-button, notebook > header tab button.titlebutton, notebook > header tab .selection-mode button.titlebutton, .selection-mode notebook > header tab button.titlebutton {
      padding: 0;
      margin-top: 4px;
      margin-bottom: 4px;
      min-width: 20px;
      min-height: 20px; }
      notebook > header tab button.flat:hover, notebook > header tab button.sidebar-button:hover, notebook > header tab button.titlebutton:hover {
        color: currentColor; }
      notebook > header tab button.flat, notebook > header tab button.sidebar-button, notebook > header tab button.titlebutton, notebook > header tab .selection-mode button.titlebutton, .selection-mode notebook > header tab button.titlebutton, notebook > header tab button.flat:backdrop, notebook > header tab button.sidebar-button:backdrop, notebook > header tab button.titlebutton:backdrop {
        color: alpha(currentColor,0.3); }
      notebook > header tab button.flat:last-child, notebook > header tab toolbar.inline-toolbar toolbutton:last-child > button.flat, toolbar.inline-toolbar notebook > header tab toolbutton:last-child > button.flat, notebook > header tab toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button, toolbar.inline-toolbar notebook > header tab toolbutton:last-child > button.sidebar-button, notebook > header tab toolbar.inline-toolbar toolbutton:last-child > button.titlebutton, toolbar.inline-toolbar notebook > header tab toolbutton:last-child > button.titlebutton, notebook > header tab toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:last-child > button.flat,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:last-child > button.sidebar-button,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:last-child > button.sidebar-button,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:last-child > button.titlebutton,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:last-child > button.titlebutton, notebook > header tab combobox.linked button.flat:nth-child(2):dir(ltr), combobox.linked notebook > header tab button.flat:nth-child(2):dir(ltr), notebook > header tab combobox.linked button.sidebar-button:nth-child(2):dir(ltr), combobox.linked notebook > header tab button.sidebar-button:nth-child(2):dir(ltr), notebook > header tab combobox.linked button.titlebutton:nth-child(2):dir(ltr), combobox.linked notebook > header tab button.titlebutton:nth-child(2):dir(ltr), notebook > header tab .linked:not(.vertical) > combobox:last-child > box > button.flat.combo, notebook > header tab .linked:not(.vertical) > combobox:last-child > box > button.combo.sidebar-button, notebook > header tab .linked:not(.vertical) > combobox:last-child > box > button.combo.titlebutton, notebook > header tab .linked.vertical > combobox:last-child > box > button.flat.combo, notebook > header tab .linked.vertical > combobox:last-child > box > button.combo.sidebar-button, notebook > header tab .linked.vertical > combobox:last-child > box > button.combo.titlebutton, notebook > header tab button.sidebar-button:last-child, notebook > header tab button.titlebutton:last-child {
        margin-left: 4px;
        margin-right: -4px; }
      notebook > header tab button.flat:first-child, notebook > header tab toolbar.inline-toolbar toolbutton:first-child > button.flat, toolbar.inline-toolbar notebook > header tab toolbutton:first-child > button.flat, notebook > header tab toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button, toolbar.inline-toolbar notebook > header tab toolbutton:first-child > button.sidebar-button, notebook > header tab toolbar.inline-toolbar toolbutton:first-child > button.titlebutton, toolbar.inline-toolbar notebook > header tab toolbutton:first-child > button.titlebutton, notebook > header tab toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:first-child > button.flat,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:first-child > button.sidebar-button,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:first-child > button.sidebar-button,
      notebook > header tab toolbar.inline-toolbar toolbutton:backdrop:first-child > button.titlebutton,
      toolbar.inline-toolbar notebook > header tab toolbutton:backdrop:first-child > button.titlebutton, notebook > header tab combobox.linked button.flat:nth-child(2):dir(rtl), combobox.linked notebook > header tab button.flat:nth-child(2):dir(rtl), notebook > header tab combobox.linked button.sidebar-button:nth-child(2):dir(rtl), combobox.linked notebook > header tab button.sidebar-button:nth-child(2):dir(rtl), notebook > header tab combobox.linked button.titlebutton:nth-child(2):dir(rtl), combobox.linked notebook > header tab button.titlebutton:nth-child(2):dir(rtl), notebook > header tab .linked:not(.vertical) > combobox:first-child > box > button.flat.combo, notebook > header tab .linked:not(.vertical) > combobox:first-child > box > button.combo.sidebar-button, notebook > header tab .linked:not(.vertical) > combobox:first-child > box > button.combo.titlebutton, notebook > header tab .linked.vertical > combobox:first-child > box > button.flat.combo, notebook > header tab .linked.vertical > combobox:first-child > box > button.combo.sidebar-button, notebook > header tab .linked.vertical > combobox:first-child > box > button.combo.titlebutton, notebook > header tab button.sidebar-button:first-child, notebook > header tab button.titlebutton:first-child {
        margin-left: -4px;
        margin-right: 4px; }
  notebook > header.top tabs, notebook > header.bottom tabs {
    padding-left: 0px;
    padding-right: 0px; }
    notebook > header.top tabs:not(:only-child), notebook > header.bottom tabs:not(:only-child) {
      margin-left: 3px;
      margin-right: 3px; }
      notebook > header.top tabs:not(:only-child):first-child, notebook > header.bottom tabs:not(:only-child):first-child {
        margin-left: -1px; }
      notebook > header.top tabs:not(:only-child):last-child, notebook > header.bottom tabs:not(:only-child):last-child {
        margin-right: -1px; }
    notebook > header.top tabs tab, notebook > header.bottom tabs tab {
      margin-left: 0px;
      margin-right: 0px; }
      notebook > header.top tabs tab.reorderable-page, notebook > header.bottom tabs tab.reorderable-page {
        border-style: none solid; }
  notebook > header.left tabs, notebook > header.right tabs {
    padding-top: 0px;
    padding-bottom: 0px; }
    notebook > header.left tabs:not(:only-child), notebook > header.right tabs:not(:only-child) {
      margin-top: 3px;
      margin-bottom: 3px; }
      notebook > header.left tabs:not(:only-child):first-child, notebook > header.right tabs:not(:only-child):first-child {
        margin-top: -1px; }
      notebook > header.left tabs:not(:only-child):last-child, notebook > header.right tabs:not(:only-child):last-child {
        margin-bottom: -1px; }
    notebook > header.left tabs tab, notebook > header.right tabs tab {
      margin-top: 0px;
      margin-bottom: 0px; }
      notebook > header.left tabs tab.reorderable-page, notebook > header.right tabs tab.reorderable-page {
        border-style: solid none; }
  notebook > header.top tab {
    padding-bottom: 4px; }
  notebook > header.bottom tab {
    padding-top: 4px; }
notebook > stack:not(:only-child) {
  background-color: #0c0c0d; }
  notebook > stack:not(:only-child):backdrop {
    background-color: #0c0c0d; }

/*******************
 * Scrollbars *
 *******************/
scrollbar {
  transition: 300ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  min-width: 10px;
  min-height: 10px; }
  *, .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode button.subtitle:visited, button:link,
  button:visited {
    -GtkScrollbar-has-backward-stepper: false;
    -GtkScrollbar-has-forward-stepper: false; }
  scrollbar:hover {
    background-color: #303033; }
  scrollbar slider {
    min-width: 8px;
    min-height: 8px;
    margin: -1px;
    border-style: none;
    border-radius: 6px;
    background-clip: padding-box;
    background-color: #3D3D41; }
    scrollbar slider:hover {
      background-color: #616166;
      min-width: 6px;
      min-height: 6px; }
    scrollbar slider:hover:active {
      background-color: #85858C;
      min-width: 6px; }
    scrollbar slider:backdrop {
      background-color: #6d6d6e; }
    scrollbar slider:disabled {
      background-color: #1C1C1E; }
  scrollbar.horizontal slider {
    min-width: 40px; }
  scrollbar.vertical slider {
    min-height: 40px; }

/************
 * Switch*
 ************/
switch {
  font-weight: bold;
  font-size: smaller;
  outline-offset: -4px;
  border: 1px solid #0c0c0d;
  border-radius: 0px;
  color: #ffffff;
  background-color: #0c0c0d;
  text-shadow: none; }
  switch:checked {
    color: #fff;
    border-color: #3790FA;
    background-color: #3790FA;
    text-shadow: none; }
  switch:disabled {
    color: rgba(255, 255, 255, 0.3);
    border-color: #1C1C1E;
    background-color: #1C1C1E;
    text-shadow: none; }
  switch:backdrop {
    color: #ffffff;
    border-color: #0c0c0d;
    background-color: #1C1C1E;
    text-shadow: none;
    transition: 200ms ease-out; }
    switch:backdrop:checked {
      color: #0c0c0d;
      border-color: #3790FA;
      background-color: #3790FA; }
    switch:backdrop:disabled {
      color: rgba(255, 255, 255, 0.3);
      border-color: #1C1C1E;
      background-color: #1C1C1E; }
  switch slider {
    margin: -1px;
    min-width: 44px;
    min-height: 26px;
    border: 1px solid;
    border-radius: 0px;
    transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
    color: #ffffff;
    outline-color: rgba(255, 255, 255, 0.3);
    border-color: black;
    background-image: linear-gradient(to bottom, #18181a 20%, #0c0c0d 90%); }
  switch:hover slider {
    color: #ffffff;
    outline-color: rgba(255, 255, 255, 0.3);
    border-color: black;
    background-image: linear-gradient(to bottom, #222225 10%, #161618 90%); }
  switch:checked slider {
    border: 1px solid #3790FA; }
  switch:disabled slider {
    border-color: #1C1C1E;
    background-image: image(#1C1C1E);
    text-shadow: none;
    -gtk-icon-shadow: none; }
    switch:disabled slider label, switch:disabled slider {
      color: rgba(255, 255, 255, 0.3); }
  switch:backdrop slider {
    transition: 200ms ease-out;
    border-color: #0c0c0d;
    background-image: image(#2D2E32);
    text-shadow: none;
    -gtk-icon-shadow: none; }
    switch:backdrop slider label, switch:backdrop slider {
      color: #ffffff; }
  switch:backdrop:checked slider {
    border-color: #3790FA; }
  switch:backdrop:disabled slider {
    border-color: #0c0c0d;
    background-image: image(#1C1C1E);
    text-shadow: none;
    -gtk-icon-shadow: none; }
    switch:backdrop:disabled slider label, switch:backdrop:disabled slider {
      color: rgba(255, 255, 255, 0.3); }
  row:selected switch, .selection-mode.titlebar:not(headerbar) row.subtitle:link switch,
  headerbar.selection-mode row.subtitle:link switch {
    box-shadow: none;
    border-color: #3790FA; }
    row:selected switch:backdrop, .selection-mode.titlebar:not(headerbar) row.subtitle:link switch:backdrop,
    headerbar.selection-mode row.subtitle:link switch:backdrop {
      border-color: #3790FA; }
    row:selected switch slider:dir(rtl), .selection-mode.titlebar:not(headerbar) row.subtitle:link switch slider:dir(rtl),
    headerbar.selection-mode row.subtitle:link switch slider:dir(rtl) {
      border-left-color: #0c0c0d; }
    row:selected switch slider:dir(ltr), .selection-mode.titlebar:not(headerbar) row.subtitle:link switch slider:dir(ltr),
    headerbar.selection-mode row.subtitle:link switch slider:dir(ltr) {
      border-right-color: #0c0c0d; }
    row:selected switch slider:checked, .selection-mode.titlebar:not(headerbar) row.subtitle:link switch slider:checked,
    headerbar.selection-mode row.subtitle:link switch slider:checked, row:selected switch slider, .selection-mode.titlebar:not(headerbar) row.subtitle:link switch slider,
    headerbar.selection-mode row.subtitle:link switch slider {
      border-color: #3790FA; }

/************
 * GtkScale *
 ************/
scale trough, scale fill, progressbar trough, progressbar:backdrop trough {
  border: 1px solid #0c0c0d;
  border-radius: 0px;
  background-color: #0c0c0d; }
  scale trough:disabled, scale fill:disabled, progressbar trough:disabled {
    background-color: #1C1C1E; }
  scale trough:backdrop, scale fill:backdrop, progressbar trough:backdrop, progressbar:backdrop trough {
    background-color: #1C1C1E;
    border-color: #0c0c0d;
    transition: 200ms ease-out; }
    scale trough:backdrop:disabled, scale fill:backdrop:disabled, progressbar trough:backdrop:disabled, progressbar:backdrop trough:disabled {
      background-color: #1C1C1E; }
  row:selected scale trough:disabled, scale row:selected trough:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale trough:disabled, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link trough:disabled,
  headerbar.selection-mode row.subtitle:link scale trough:disabled,
  scale headerbar.selection-mode row.subtitle:link trough:disabled, row:selected scale fill:disabled, scale row:selected fill:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale fill:disabled, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link fill:disabled,
  headerbar.selection-mode row.subtitle:link scale fill:disabled,
  scale headerbar.selection-mode row.subtitle:link fill:disabled, row:selected progressbar trough:disabled, progressbar row:selected trough:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link progressbar trough:disabled, progressbar .selection-mode.titlebar:not(headerbar) row.subtitle:link trough:disabled,
  headerbar.selection-mode row.subtitle:link progressbar trough:disabled,
  progressbar headerbar.selection-mode row.subtitle:link trough:disabled, row:selected scale trough, scale row:selected trough, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale trough, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link trough,
  headerbar.selection-mode row.subtitle:link scale trough,
  scale headerbar.selection-mode row.subtitle:link trough, row:selected scale fill, scale row:selected fill, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale fill, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link fill,
  headerbar.selection-mode row.subtitle:link scale fill,
  scale headerbar.selection-mode row.subtitle:link fill, row:selected progressbar trough, progressbar row:selected trough, .selection-mode.titlebar:not(headerbar) row.subtitle:link progressbar trough, progressbar .selection-mode.titlebar:not(headerbar) row.subtitle:link trough,
  headerbar.selection-mode row.subtitle:link progressbar trough,
  progressbar headerbar.selection-mode row.subtitle:link trough {
    border-color: #3790FA; }
  .osd scale trough, scale .osd trough, .osd scale fill, scale .osd fill, .osd progressbar trough, progressbar .osd trough {
    border-color: rgba(0, 0, 0, 0.7);
    background-color: rgba(0, 0, 0, 0.5); }
    .osd scale trough:disabled, scale .osd trough:disabled, .osd scale fill:disabled, scale .osd fill:disabled, .osd progressbar trough:disabled, progressbar .osd trough:disabled {
      background-color: #1C1C1E; }

scale highlight, progressbar progress, progressbar:backdrop progress {
  border: 1px solid #3790FA;
  border-radius: 0px;
  background-color: #3790FA; }
  scale highlight:disabled, progressbar progress:disabled {
    background-color: #1C1C1E;
    border-color: #1C1C1E; }
  scale highlight:backdrop, progressbar progress:backdrop, progressbar:backdrop progress {
    border-color: #3790FA; }
    scale highlight:backdrop:disabled, progressbar progress:backdrop:disabled, progressbar:backdrop progress:disabled {
      background-color: #1C1C1E;
      border-color: #1C1C1E; }
  row:selected scale highlight:disabled, scale row:selected highlight:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale highlight:disabled, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link highlight:disabled,
  headerbar.selection-mode row.subtitle:link scale highlight:disabled,
  scale headerbar.selection-mode row.subtitle:link highlight:disabled, row:selected progressbar progress:disabled, progressbar row:selected progress:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link progressbar progress:disabled, progressbar .selection-mode.titlebar:not(headerbar) row.subtitle:link progress:disabled,
  headerbar.selection-mode row.subtitle:link progressbar progress:disabled,
  progressbar headerbar.selection-mode row.subtitle:link progress:disabled, row:selected scale highlight, scale row:selected highlight, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale highlight, scale .selection-mode.titlebar:not(headerbar) row.subtitle:link highlight,
  headerbar.selection-mode row.subtitle:link scale highlight,
  scale headerbar.selection-mode row.subtitle:link highlight, row:selected progressbar progress, progressbar row:selected progress, .selection-mode.titlebar:not(headerbar) row.subtitle:link progressbar progress, progressbar .selection-mode.titlebar:not(headerbar) row.subtitle:link progress,
  headerbar.selection-mode row.subtitle:link progressbar progress,
  progressbar headerbar.selection-mode row.subtitle:link progress {
    border-color: #1C1C1E; }
  .osd scale highlight, scale .osd highlight, .osd progressbar progress, progressbar .osd progress {
    border-color: rgba(0, 0, 0, 0.7); }
    .osd scale highlight:disabled, scale .osd highlight:disabled, .osd progressbar progress:disabled, progressbar .osd progress:disabled {
      border-color: #1C1C1E; }

scale {
  min-height: 10px;
  min-width: 10px;
  padding: 12px; }
  scale fill,
  scale highlight {
    margin: -1px; }
  scale slider {
    min-height: 18px;
    min-width: 18px;
    margin: -9px; }
  scale.fine-tune.horizontal {
    padding-top: 9px;
    padding-bottom: 9px;
    min-height: 16px; }
  scale.fine-tune.vertical {
    padding-left: 9px;
    padding-right: 9px;
    min-width: 16px; }
  scale.fine-tune slider {
    margin: -6px; }
  scale.fine-tune fill,
  scale.fine-tune highlight,
  scale.fine-tune trough {
    border-radius: 5px;
    -gtk-outline-radius: 0; }
  scale trough {
    outline-offset: 2px;
    -gtk-outline-radius: 5px; }
  scale fill:backdrop, scale fill {
    background-color: #0c0c0d; }
  scale fill:disabled:backdrop, scale fill:disabled {
    border-color: #1C1C1E;
    background-color: #1C1C1E; }
  .osd scale fill {
    background-color: rgba(98, 98, 98, 0.775); }
    .osd scale fill:disabled:backdrop, .osd scale fill:disabled {
      border-color: #1C1C1E;
      background-color: #1C1C1E; }
  scale slider {
    color: #ffffff;
    outline-color: rgba(255, 255, 255, 0.3);
    border-color: black;
    background-image: linear-gradient(to bottom, #18181a 20%, #0c0c0d 90%);
    border: 1px solid;
    border-color: #3790FA;
    border-radius: 100%;
    transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
    transition-property: background, border, box-shadow; }
    scale slider:hover {
      color: #ffffff;
      outline-color: rgba(255, 255, 255, 0.3);
      border-color: black;
      background-image: linear-gradient(to bottom, #222225 10%, #161618 90%);
      border-color: #3790FA; }
    scale slider:active {
      border-color: #3790FA; }
    scale slider:disabled {
      border-color: #1C1C1E;
      background-image: image(#1C1C1E);
      text-shadow: none;
      -gtk-icon-shadow: none; }
      scale slider:disabled label, scale slider:disabled {
        color: rgba(255, 255, 255, 0.3); }
    row:selected scale slider:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale slider:disabled,
    headerbar.selection-mode row.subtitle:link scale slider:disabled, row:selected scale slider, .selection-mode.titlebar:not(headerbar) row.subtitle:link scale slider,
    headerbar.selection-mode row.subtitle:link scale slider {
      border-color: #1C1C1E; }
    .osd scale slider {
      color: #ffffff;
      border-color: rgba(0, 0, 0, 0.7);
      background-color: transparent;
      background-image: image(rgba(28, 31, 32, 0.7));
      background-clip: padding-box;
      box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
      text-shadow: 0 1px white;
      -gtk-icon-shadow: 0 1px white;
      outline-color: rgba(255, 255, 255, 0.3);
      border-color: rgba(0, 0, 0, 0.7);
      background-color: #1c1f20; }
      .osd scale slider:hover {
        color: white;
        border-color: rgba(0, 0, 0, 0.7);
        background-color: transparent;
        background-image: image(rgba(57, 63, 65, 0.7));
        background-clip: padding-box;
        box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
        text-shadow: 0 1px white;
        -gtk-icon-shadow: 0 1px white;
        outline-color: rgba(255, 255, 255, 0.3);
        background-color: #1c1f20; }
      .osd scale slider:active {
        color: white;
        border-color: rgba(0, 0, 0, 0.7);
        background-color: transparent;
        background-image: image(rgba(0, 0, 0, 0.7));
        background-clip: padding-box;
        box-shadow: none;
        text-shadow: none;
        -gtk-icon-shadow: none;
        outline-color: rgba(255, 255, 255, 0.3);
        background-color: #1c1f20; }
      .osd scale slider:disabled {
        color: rgba(255, 255, 255, 0.3);
        border-color: rgba(0, 0, 0, 0.7);
        background-color: transparent;
        background-image: image(#1C1C1E);
        background-clip: padding-box;
        box-shadow: none;
        text-shadow: none;
        -gtk-icon-shadow: none;
        background-color: #1C1C1E; }
      .osd scale slider:backdrop {
        color: #ffffff;
        border-color: rgba(0, 0, 0, 0.7);
        background-color: transparent;
        background-image: image(rgba(28, 31, 32, 0.7));
        background-clip: padding-box;
        box-shadow: none;
        text-shadow: none;
        -gtk-icon-shadow: none;
        background-color: #1c1f20; }
        .osd scale slider:backdrop:disabled {
          background-color: #1C1C1E; }
  scale value {
    color: alpha(currentColor,0.55); }
  scale marks {
    color: alpha(currentColor,0.55); }
    scale marks.top {
      margin-bottom: 6px;
      margin-top: -12px; }
    scale marks.bottom {
      margin-top: 6px;
      margin-bottom: -12px; }
    scale marks.top {
      margin-right: 6px;
      margin-left: -12px; }
    scale marks.bottom {
      margin-left: 6px;
      margin-right: -12px; }
  scale.fine-tune marks.top {
    margin-bottom: 6px;
    margin-top: -9px; }
  scale.fine-tune marks.bottom {
    margin-top: 6px;
    margin-bottom: -9px; }
  scale.fine-tune marks.top {
    margin-right: 6px;
    margin-left: -9px; }
  scale.fine-tune marks.bottom {
    margin-left: 6px;
    margin-right: -9px; }
  scale.horizontal indicator {
    min-height: 6px;
    min-width: 1px; }
  scale.horizontal.fine-tune indicator {
    min-height: 3px; }
  scale.vertical indicator {
    min-height: 1px;
    min-width: 6px; }
  scale.vertical.fine-tune indicator {
    min-width: 3px; }
  scale.horizontal.marks-before:not(.marks-after) slider {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-before:not(.marks-after) slider:hover {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-hover.png"), url("assets/slider-horz-scale-has-marks-above-hover@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-before:not(.marks-after) slider:active {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-active.png"), url("assets/slider-horz-scale-has-marks-above-active@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-before:not(.marks-after) slider:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-insensitive.png"), url("assets/slider-horz-scale-has-marks-above-insensitive@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-before:not(.marks-after) slider:backdrop {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-backdrop.png"), url("assets/slider-horz-scale-has-marks-above-backdrop@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-before:not(.marks-after) slider:backdrop:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-backdrop-insensitive.png"), url("assets/slider-horz-scale-has-marks-above-backdrop-insensitive@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-top: -14px;
    background-position: top;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-top: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below.png"), url("assets/slider-horz-scale-has-marks-below@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider:hover {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-hover.png"), url("assets/slider-horz-scale-has-marks-below-hover@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider:active {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-active.png"), url("assets/slider-horz-scale-has-marks-below-active@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-insensitive.png"), url("assets/slider-horz-scale-has-marks-below-insensitive@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider:backdrop {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-backdrop.png"), url("assets/slider-horz-scale-has-marks-below-backdrop@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.horizontal.marks-after:not(.marks-before) slider:backdrop:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-backdrop-insensitive.png"), url("assets/slider-horz-scale-has-marks-below-backdrop-insensitive@2.png"));
    min-height: 26px;
    min-width: 22px;
    margin-bottom: -14px;
    background-position: bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-bottom: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above.png"), url("assets/slider-vert-scale-has-marks-above@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider:hover {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-hover.png"), url("assets/slider-vert-scale-has-marks-above-hover@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider:active {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-active.png"), url("assets/slider-vert-scale-has-marks-above-active@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-insensitive.png"), url("assets/slider-vert-scale-has-marks-above-insensitive@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider:backdrop {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-backdrop.png"), url("assets/slider-vert-scale-has-marks-above-backdrop@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-before:not(.marks-after) slider:backdrop:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-backdrop-insensitive.png"), url("assets/slider-vert-scale-has-marks-above-backdrop-insensitive@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-left: -14px;
    background-position: left bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-before:not(.marks-after).fine-tune slider {
    margin: -7px;
    margin-left: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below.png"), url("assets/slider-vert-scale-has-marks-below@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider:hover {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-hover.png"), url("assets/slider-vert-scale-has-marks-below-hover@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider:active {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-active.png"), url("assets/slider-vert-scale-has-marks-below-active@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-insensitive.png"), url("assets/slider-vert-scale-has-marks-below-insensitive@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider:backdrop {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-backdrop.png"), url("assets/slider-vert-scale-has-marks-below-backdrop@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.vertical.marks-after:not(.marks-before) slider:backdrop:disabled {
    margin: -10px;
    border-style: none;
    border-radius: 0;
    background-color: transparent;
    background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-backdrop-insensitive.png"), url("assets/slider-vert-scale-has-marks-below-backdrop-insensitive@2.png"));
    min-height: 22px;
    min-width: 26px;
    margin-right: -14px;
    background-position: right bottom;
    background-repeat: no-repeat;
    box-shadow: none; }
  scale.vertical.marks-after:not(.marks-before).fine-tune slider {
    margin: -7px;
    margin-right: -11px; }
  scale.color {
    min-height: 0;
    min-width: 0; }
    scale.color trough {
      background-image: image(#0c0c0d);
      background-repeat: no-repeat; }
    scale.color.horizontal {
      padding: 0 0 15px 0; }
      scale.color.horizontal trough {
        padding-bottom: 4px;
        background-position: 0 -3px;
        border-top-left-radius: 0;
        border-top-right-radius: 0; }
      scale.color.horizontal slider:dir(ltr):hover, scale.color.horizontal slider:dir(ltr):backdrop, scale.color.horizontal slider:dir(ltr):disabled, scale.color.horizontal slider:dir(ltr):backdrop:disabled, scale.color.horizontal slider:dir(ltr), scale.color.horizontal slider:dir(rtl):hover, scale.color.horizontal slider:dir(rtl):backdrop, scale.color.horizontal slider:dir(rtl):disabled, scale.color.horizontal slider:dir(rtl):backdrop:disabled, scale.color.horizontal slider:dir(rtl) {
        margin-bottom: -15px;
        margin-top: 6px; }
    scale.color.vertical:dir(ltr) {
      padding: 0 0 0 15px; }
      scale.color.vertical:dir(ltr) trough {
        padding-left: 4px;
        background-position: 3px 0;
        border-bottom-right-radius: 0;
        border-top-right-radius: 0; }
      scale.color.vertical:dir(ltr) slider:hover, scale.color.vertical:dir(ltr) slider:backdrop, scale.color.vertical:dir(ltr) slider:disabled, scale.color.vertical:dir(ltr) slider:backdrop:disabled, scale.color.vertical:dir(ltr) slider {
        margin-left: -15px;
        margin-right: 6px; }
    scale.color.vertical:dir(rtl) {
      padding: 0 15px 0 0; }
      scale.color.vertical:dir(rtl) trough {
        padding-right: 4px;
        background-position: -3px 0;
        border-bottom-left-radius: 0;
        border-top-left-radius: 0; }
      scale.color.vertical:dir(rtl) slider:hover, scale.color.vertical:dir(rtl) slider:backdrop, scale.color.vertical:dir(rtl) slider:disabled, scale.color.vertical:dir(rtl) slider:backdrop:disabled, scale.color.vertical:dir(rtl) slider {
        margin-right: -15px;
        margin-left: 6px; }
    scale.color.fine-tune.horizontal:dir(ltr), scale.color.fine-tune.horizontal:dir(rtl) {
      padding: 0 0 12px 0; }
      scale.color.fine-tune.horizontal:dir(ltr) trough, scale.color.fine-tune.horizontal:dir(rtl) trough {
        padding-bottom: 7px;
        background-position: 0 -6px; }
      scale.color.fine-tune.horizontal:dir(ltr) slider, scale.color.fine-tune.horizontal:dir(rtl) slider {
        margin-bottom: -15px;
        margin-top: 6px; }
    scale.color.fine-tune.vertical:dir(ltr) {
      padding: 0 0 0 12px; }
      scale.color.fine-tune.vertical:dir(ltr) trough {
        padding-left: 7px;
        background-position: 6px 0; }
      scale.color.fine-tune.vertical:dir(ltr) slider {
        margin-left: -15px;
        margin-right: 6px; }
    scale.color.fine-tune.vertical:dir(rtl) {
      padding: 0 12px 0 0; }
      scale.color.fine-tune.vertical:dir(rtl) trough {
        padding-right: 7px;
        background-position: -6px 0; }
      scale.color.fine-tune.vertical:dir(rtl) slider {
        margin-right: -15px;
        margin-left: 6px; }

/**********************
 * Progress bars *
 **********************/
progressbar {
  font-size: smaller;
  color: rgba(255, 255, 255, 0.4); }
  progressbar.horizontal trough,
  progressbar.horizontal progress {
    min-height: 2px; }
  progressbar.vertical trough,
  progressbar.vertical progress {
    min-width: 2px; }
  progressbar.horizontal progress {
    margin: 0 -1px; }
  progressbar.vertical progress {
    margin: -1px 0; }
  progressbar:backdrop {
    box-shadow: none;
    transition: 200ms ease-out; }
  progressbar progress {
    border-radius: 1.5px; }
    progressbar progress.left {
      border-top-left-radius: 2px;
      border-bottom-left-radius: 2px; }
    progressbar progress.right {
      border-top-right-radius: 2px;
      border-bottom-right-radius: 2px; }
    progressbar progress.top {
      border-top-right-radius: 2px;
      border-top-left-radius: 2px; }
    progressbar progress.bottom {
      border-bottom-right-radius: 2px;
      border-bottom-left-radius: 2px; }
  progressbar.osd {
    min-width: 3px;
    min-height: 3px;
    background-color: transparent; }
    progressbar.osd trough {
      border-style: none;
      border-radius: 0;
      background-color: transparent;
      box-shadow: none; }
    progressbar.osd progress {
      border-style: none;
      border-radius: 0; }
  progressbar trough.empty progress {
    all: unset; }

/******************
 * Level Bar*
 ******************/
levelbar.horizontal block {
  min-height: 1px; }
levelbar.horizontal.discrete block {
  margin: 0 1px;
  min-width: 32px; }
levelbar.vertical block {
  min-width: 1px; }
levelbar.vertical.discrete block {
  margin: 1px 0;
  min-height: 32px; }
levelbar:backdrop {
  transition: 200ms ease-out; }
levelbar trough {
  border: 1px solid;
  padding: 2px;
  border-radius: 0px;
  color: #ffffff;
  border-color: #0c0c0d;
  background-color: #0c0c0d; }
  levelbar trough:backdrop, levelbar progressbar:backdrop trough, progressbar:backdrop levelbar trough {
    color: #ffffff;
    border-color: #0c0c0d;
    background-color: #0c0c0d;
    box-shadow: none; }
levelbar block {
  border: 1px solid;
  border-radius: 1px; }
  levelbar block.low {
    border-color: #7b1413;
    background-color: #d42320; }
    levelbar block.low:backdrop {
      border-color: #d42320; }
  levelbar block.high, levelbar block:not(.empty) {
    border-color: #055dc6;
    background-color: #3790FA; }
    levelbar block.high:backdrop, levelbar block:not(.empty):backdrop {
      border-color: #3790FA; }
  levelbar block.full {
    border-color: #055dc6;
    background-color: #3790FA; }
    levelbar block.full:backdrop {
      border-color: #3790FA; }
  levelbar block.empty {
    background-color: transparent;
    border-color: rgba(255, 255, 255, 0.2); }
    levelbar block.empty:backdrop {
      border-color: rgba(255, 255, 255, 0.15); }

/************************
 * Print dialog*
*************************/
printdialog paper {
  color: #ffffff;
  border: 1px solid #0c0c0d;
  background: white;
  padding: 0; }
  printdialog paper:backdrop {
    color: #ffffff;
    border-color: #0c0c0d; }
printdialog .dialog-action-box {
  margin: 12px; }

/*************
 * Frames *
 *************/
frame > border,
.frame {
  box-shadow: none;
  margin: 0;
  padding: 0;
  border-radius: 0;
  border: 1px solid #0c0c0d; }
  frame > border.flat,
  .frame.flat,
  notebook > header > tabs > arrow.frame,
  button.frame.sidebar-button,
  button.frame.titlebutton {
    border-style: none; }
  frame > border:backdrop,
  .frame:backdrop,
  label:backdrop selection.frame,
  progressbar:backdrop trough.frame,
  progressbar:backdrop progress.frame {
    border-color: #0c0c0d; }

actionbar > revealer > box {
  padding: 6px;
  border-top: 1px solid #0c0c0d; }
  actionbar > revealer > box:backdrop {
    border-color: #0c0c0d; }

scrolledwindow viewport.frame {
  border-style: none; }
scrolledwindow overshoot.top {
  background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(black), to(rgba(0, 0, 0, 0))), -gtk-gradient(radial, center top, 0, center top, 0.6, from(rgba(255, 255, 255, 0.07)), to(rgba(255, 255, 255, 0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center top;
  background-color: transparent;
  border: none;
  box-shadow: none; }
  scrolledwindow overshoot.top:backdrop {
    background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(#0c0c0d), to(rgba(12, 12, 13, 0)));
    background-size: 100% 5%;
    background-repeat: no-repeat;
    background-position: center top;
    background-color: transparent;
    border: none;
    box-shadow: none; }
scrolledwindow overshoot.bottom {
  background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(black), to(rgba(0, 0, 0, 0))), -gtk-gradient(radial, center bottom, 0, center bottom, 0.6, from(rgba(255, 255, 255, 0.07)), to(rgba(255, 255, 255, 0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center bottom;
  background-color: transparent;
  border: none;
  box-shadow: none; }
  scrolledwindow overshoot.bottom:backdrop {
    background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(#0c0c0d), to(rgba(12, 12, 13, 0)));
    background-size: 100% 5%;
    background-repeat: no-repeat;
    background-position: center bottom;
    background-color: transparent;
    border: none;
    box-shadow: none; }
scrolledwindow overshoot.left {
  background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(black), to(rgba(0, 0, 0, 0))), -gtk-gradient(radial, left center, 0, left center, 0.6, from(rgba(255, 255, 255, 0.07)), to(rgba(255, 255, 255, 0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: left center;
  background-color: transparent;
  border: none;
  box-shadow: none; }
  scrolledwindow overshoot.left:backdrop {
    background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(#0c0c0d), to(rgba(12, 12, 13, 0)));
    background-size: 5% 100%;
    background-repeat: no-repeat;
    background-position: left center;
    background-color: transparent;
    border: none;
    box-shadow: none; }
scrolledwindow overshoot.right {
  background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(black), to(rgba(0, 0, 0, 0))), -gtk-gradient(radial, right center, 0, right center, 0.6, from(rgba(255, 255, 255, 0.07)), to(rgba(255, 255, 255, 0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: right center;
  background-color: transparent;
  border: none;
  box-shadow: none; }
  scrolledwindow overshoot.right:backdrop {
    background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(#0c0c0d), to(rgba(12, 12, 13, 0)));
    background-size: 5% 100%;
    background-repeat: no-repeat;
    background-position: right center;
    background-color: transparent;
    border: none;
    box-shadow: none; }
scrolledwindow undershoot.top, scrolledwindow undershoot.right, scrolledwindow undershoot.bottom, scrolledwindow undershoot.left {
  background-image: none; }
scrolledwindow junction {
  border-color: transparent;
  border-image: linear-gradient(to bottom, #0c0c0d 1px, transparent 1px) 0 0 0 1/0 1px stretch;
  background-color: #3d3d3d; }
  scrolledwindow junction:dir(rtl) {
    border-image-slice: 0 1 0 0; }
  scrolledwindow junction:backdrop {
    border-image-source: linear-gradient(to bottom, #0c0c0d 1px, transparent 1px);
    background-color: #050505;
    transition: 200ms ease-out; }

separator {
  background-color: #0c0c0d;
  min-width: 1px;
  min-height: 1px; }

/************
 * Lists *
 ************/
list {
  color: #ffffff;
  background-color: #0c0c0d;
  border-color: #0c0c0d; }
  list:backdrop {
    background-color: #0c0c0d;
    border-color: #0c0c0d; }
  list row {
    padding: 2px; }

row {
  transition: all 150ms cubic-bezier(0.25, 0.46, 0.45, 0.94); }
  row:hover {
    transition: none; }
  row:backdrop {
    transition: 200ms ease-out; }
  row.activatable.has-open-popup, row.activatable:hover {
    background-color: rgba(255, 255, 255, 0.1); }
  row.activatable:active {
    box-shadow: inset 0 2px 2px -2px rgba(0, 0, 0, 0.2); }
  row.activatable:backdrop:hover {
    background-color: transparent; }
  row.activatable:selected:active, .selection-mode.titlebar:not(headerbar) row.activatable.subtitle:active:link,
  headerbar.selection-mode row.activatable.subtitle:active:link {
    box-shadow: inset 0 2px 3px -1px rgba(0, 0, 0, 0.5); }
  row.activatable:selected.has-open-popup, .selection-mode.titlebar:not(headerbar) row.activatable.has-open-popup.subtitle:link,
  headerbar.selection-mode row.activatable.has-open-popup.subtitle:link, row.activatable:selected:hover, .selection-mode.titlebar:not(headerbar) row.activatable.subtitle:hover:link,
  headerbar.selection-mode row.activatable.subtitle:hover:link {
    background-color: #4b9bfb; }
  row.activatable:selected:backdrop, .selection-mode.titlebar:not(headerbar) row.activatable.subtitle:backdrop:link,
  headerbar.selection-mode row.activatable.subtitle:backdrop:link {
    background-color: #3790FA; }

/*******************************
 * App Notifications*
 *******************************/
.app-notification,
.app-notification.frame {
  padding: 10px;
  border-radius: 0 0 5px 5px;
  background-color: rgba(28, 31, 32, 0.7);
  background-image: linear-gradient(to bottom, rgba(0, 0, 0, 0.2), transparent 2px);
  background-clip: padding-box; }
  .app-notification:backdrop, label:backdrop selection.app-notification, progressbar:backdrop trough.app-notification, progressbar:backdrop progress.app-notification,
  .app-notification.frame:backdrop,
  label:backdrop selection.app-notification.frame,
  progressbar:backdrop trough.app-notification.frame,
  progressbar:backdrop progress.app-notification.frame {
    background-image: none;
    transition: 200ms ease-out; }
  .app-notification border,
  .app-notification.frame border {
    border: none; }

/*****************************
 * Expanders *
 *****************************/
expander arrow {
  min-width: 16px;
  min-height: 16px;
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }
  expander arrow:dir(rtl) {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl"); }
  expander arrow:hover {
    color: white; }
  expander arrow:checked {
    -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }

/************
 * Calendar*
 ***********/
calendar {
  color: #ffffff;
  border: 1px solid #0c0c0d; }
  calendar:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:link,
  headerbar.selection-mode calendar.subtitle:link {
    border-radius: 3px; }
  calendar.header {
    border-bottom-color: rgba(0, 0, 0, 0.1); }
    calendar.header:backdrop {
      border-bottom-color: rgba(0, 0, 0, 0.1); }
  calendar.button {
    color: rgba(255, 255, 255, 0.45); }
    calendar.button:hover {
      color: #ffffff; }
    calendar.button:backdrop {
      color: rgba(255, 255, 255, 0.45); }
    calendar.button:disabled {
      color: rgba(255, 255, 255, 0.3); }
  calendar.highlight {
    color: rgba(255, 255, 255, 0.3); }
    calendar.highlight:backdrop {
      color: rgba(255, 255, 255, 0.3); }
  calendar:backdrop {
    color: #ffffff;
    border-color: #0c0c0d; }
  calendar:indeterminate {
    color: alpha(currentColor,0.1); }

/***************
 * Dialogs*
 ***************/
messagedialog .titlebar {
  min-height: 20px;
  background-image: none;
  background-color: #0c0c0d;
  border-style: none;
  border-top-left-radius: 7px;
  border-top-right-radius: 7px; }
messagedialog.csd.background {
  border-bottom-left-radius: 9px;
  border-bottom-right-radius: 9px; }
messagedialog.csd .dialog-action-area button {
  padding: 10px 14px;
  border-radius: 0;
  border-left-style: solid;
  border-right-style: none;
  border-bottom-style: none; }
  messagedialog.csd .dialog-action-area button:first-child, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:first-child > button.flat, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:first-child > button.flat, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:first-child > button.sidebar-button, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:first-child > button.sidebar-button, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:first-child > button.titlebutton, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:first-child > button.titlebutton, messagedialog.csd .dialog-action-area toolbar.inline-toolbar .selection-mode toolbutton:first-child > button.titlebutton, messagedialog.csd .dialog-action-area combobox.linked button:nth-child(2):dir(rtl), combobox.linked messagedialog.csd .dialog-action-area button:nth-child(2):dir(rtl), messagedialog.csd .dialog-action-area .linked:not(.vertical) > combobox:first-child > box > button.combo, messagedialog.csd .dialog-action-area .linked.vertical > combobox:first-child > box > button.combo {
    border-left-style: none;
    border-bottom-left-radius: 7px;
    -gtk-outline-bottom-left-radius: 5px; }
  messagedialog.csd .dialog-action-area button:last-child, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:last-child > button.flat, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:last-child > button.flat, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:last-child > button.sidebar-button, messagedialog.csd .dialog-action-area toolbar.inline-toolbar toolbutton:last-child > button.titlebutton, toolbar.inline-toolbar messagedialog.csd .dialog-action-area toolbutton:last-child > button.titlebutton, messagedialog.csd .dialog-action-area toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton, messagedialog.csd .dialog-action-area combobox.linked button:nth-child(2):dir(ltr), combobox.linked messagedialog.csd .dialog-action-area button:nth-child(2):dir(ltr), messagedialog.csd .dialog-action-area .linked:not(.vertical) > combobox:last-child > box > button.combo, messagedialog.csd .dialog-action-area .linked.vertical > combobox:last-child > box > button.combo {
    border-bottom-right-radius: 7px;
    -gtk-outline-bottom-right-radius: 5px; }

filechooser .dialog-action-box {
  border-top: 1px solid #0c0c0d; }
  filechooser .dialog-action-box:backdrop, filechooser label:backdrop selection.dialog-action-box, label:backdrop filechooser selection.dialog-action-box, filechooser progressbar:backdrop trough.dialog-action-box, progressbar:backdrop filechooser trough.dialog-action-box, filechooser progressbar:backdrop progress.dialog-action-box, progressbar:backdrop filechooser progress.dialog-action-box {
    border-top-color: #0c0c0d; }
filechooser #pathbarbox {
  border-bottom: 1px solid #0c0c0d; }

filechooserbutton:drop(active) {
  box-shadow: none;
  border-color: transparent; }

/****************
 * Sidebar*
 ****************/
.sidebar {
  border-style: none;
  background-color: #0c0c0d; }
  stacksidebar.sidebar:dir(ltr) list, stacksidebar.sidebar.left list, stacksidebar.sidebar.left:dir(rtl) list, .sidebar:dir(ltr), .sidebar.left, .sidebar.left:dir(rtl) {
    border-right: 1px solid #0c0c0d;
    border-left-style: none; }
  stacksidebar.sidebar:dir(rtl) list
  .sidebar:dir(rtl), stacksidebar.sidebar.right list
  .sidebar:dir(rtl), .sidebar.right {
    border-left: 1px solid #0c0c0d;
    border-right-style: none; }
  .sidebar:backdrop, label:backdrop selection.sidebar, progressbar:backdrop trough.sidebar, progressbar:backdrop progress.sidebar {
    background-color: #0c0c0d;
    border-color: #0c0c0d;
    transition: 200ms ease-out; }
  .sidebar list {
    background-color: transparent; }
  paned .sidebar.left, paned .sidebar.right, paned .sidebar.left:dir(rtl), paned .sidebar:dir(rtl), paned .sidebar:dir(ltr), paned .sidebar {
    border-style: none; }

stacksidebar row {
  padding: 10px 4px; }
  stacksidebar row > label {
    padding-left: 6px;
    padding-right: 6px; }
  stacksidebar row.needs-attention > label {
    background-size: 6px 6px, 0 0; }

/**************************
 * File chooser *
 *************************/
row image.sidebar-icon {
  opacity: 0.7; }

placessidebar > viewport.frame {
  border-style: none; }
placessidebar row {
  min-height: 36px;
  padding: 0px; }
  placessidebar row > revealer {
    padding: 0 14px; }
  placessidebar row:selected, placessidebar .selection-mode.titlebar:not(headerbar) row.subtitle:link, .selection-mode.titlebar:not(headerbar) placessidebar row.subtitle:link,
  placessidebar headerbar.selection-mode row.subtitle:link,
  headerbar.selection-mode placessidebar row.subtitle:link {
    color: #fff; }
  placessidebar row:disabled {
    color: rgba(255, 255, 255, 0.3); }
  placessidebar row:backdrop {
    color: #ffffff; }
    placessidebar row:backdrop:selected, placessidebar .selection-mode.titlebar:not(headerbar) row.subtitle:backdrop:link, .selection-mode.titlebar:not(headerbar) placessidebar row.subtitle:backdrop:link,
    placessidebar headerbar.selection-mode row.subtitle:backdrop:link,
    headerbar.selection-mode placessidebar row.subtitle:backdrop:link {
      color: #0c0c0d; }
    placessidebar row:backdrop:disabled {
      color: rgba(255, 255, 255, 0.3); }
  placessidebar row image.sidebar-icon:dir(ltr) {
    padding-right: 8px; }
  placessidebar row image.sidebar-icon:dir(rtl) {
    padding-left: 8px; }
  placessidebar row label.sidebar-label:dir(ltr) {
    padding-right: 2px; }
  placessidebar row label.sidebar-label:dir(rtl) {
    padding-left: 2px; }
  button.sidebar-button {
    min-height: 26px;
    min-width: 26px;
    margin-top: 3px;
    margin-bottom: 3px;
    padding: 0;
    border-radius: 100%;
    -gtk-outline-radius: 100%; }
    button.sidebar-button:not(:hover):not(:active) > image, button.sidebar-button:backdrop > image {
      opacity: 0.7; }
  placessidebar row:selected:active, placessidebar .selection-mode.titlebar:not(headerbar) row.subtitle:active:link, .selection-mode.titlebar:not(headerbar) placessidebar row.subtitle:active:link,
  placessidebar headerbar.selection-mode row.subtitle:active:link,
  headerbar.selection-mode placessidebar row.subtitle:active:link {
    box-shadow: none; }
  placessidebar row.sidebar-placeholder-row {
    padding: 0 8px;
    min-height: 2px;
    background-image: image(#0c0c0d);
    background-clip: content-box; }
  placessidebar row.sidebar-new-bookmark-row {
    color: #3790FA; }
  placessidebar row:drop(active):not(:disabled) {
    color: #0c0c0d;
    box-shadow: inset 0 1px #0c0c0d, inset 0 -1px #0c0c0d; }
    placessidebar row:drop(active):not(:disabled):selected, placessidebar .selection-mode.titlebar:not(headerbar) row.subtitle:drop(active):not(:disabled):link, .selection-mode.titlebar:not(headerbar) placessidebar row.subtitle:drop(active):not(:disabled):link,
    placessidebar headerbar.selection-mode row.subtitle:drop(active):not(:disabled):link,
    headerbar.selection-mode placessidebar row.subtitle:drop(active):not(:disabled):link {
      color: #fff;
      background-color: #0c0c0d; }

placesview .server-list-button > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(0turn); }
placesview .server-list-button:checked > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(-0.5turn); }
placesview row.activatable:hover {
  background-color: transparent; }
placesview > actionbar > revealer > box > label {
  padding-left: 8px;
  padding-right: 8px; }

/*********
 * Paned *
 *********/
paned > separator {
  min-width: 1px;
  min-height: 1px;
  -gtk-icon-source: none;
  border-style: none;
  background-color: transparent;
  background-image: image(#0c0c0d);
  background-size: 1px 1px; }
  paned > separator:selected, .selection-mode.titlebar:not(headerbar) paned > separator.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode paned > separator.subtitle:link,
  headerbar.selection-mode paned > separator.subtitle:link {
    background-image: image(#3790FA); }
  paned > separator:backdrop {
    background-image: image(#0c0c0d); }
  paned > separator.wide {
    min-width: 5px;
    min-height: 5px;
    background-color: #0c0c0d;
    background-image: image(#0c0c0d), image(#0c0c0d);
    background-size: 1px 1px, 1px 1px; }
    paned > separator.wide:backdrop {
      background-color: #0c0c0d;
      background-image: image(#0c0c0d), image(#0c0c0d); }
paned.horizontal > separator {
  background-repeat: repeat-y; }
  paned.horizontal > separator:dir(ltr) {
    margin: 0 -8px 0 0;
    padding: 0 8px 0 0;
    background-position: left; }
  paned.horizontal > separator:dir(rtl) {
    margin: 0 0 0 -8px;
    padding: 0 0 0 8px;
    background-position: right; }
  paned.horizontal > separator.wide {
    margin: 0;
    padding: 0;
    background-repeat: repeat-y, repeat-y;
    background-position: left, right; }
paned.vertical > separator {
  margin: 0 0 -8px 0;
  padding: 0 0 8px 0;
  background-repeat: repeat-x;
  background-position: top; }
  paned.vertical > separator.wide {
    margin: 0;
    padding: 0;
    background-repeat: repeat-x, repeat-x;
    background-position: bottom, top; }

/******************
 * GtkInfoBar*
 ******************/
infobar {
  border-style: none; }
  infobar.info, infobar.question, infobar.warning, infobar.error {
    text-shadow: none; }
    infobar.info:backdrop, infobar.info, infobar.question:backdrop, infobar.question, infobar.warning:backdrop, infobar.warning, infobar.error:backdrop, infobar.error {
      background-color: #2D2E32;
      border-color: #0674f8; }
      infobar.info:backdrop label, infobar.info:backdrop, infobar.info label, infobar.info, infobar.question:backdrop label, infobar.question:backdrop, infobar.question label, infobar.question, infobar.warning:backdrop label, infobar.warning:backdrop, infobar.warning label, infobar.warning, infobar.error:backdrop label, infobar.error:backdrop, infobar.error label, infobar.error {
        color: #ffffff; }
    infobar.info:backdrop, infobar.question:backdrop, infobar.warning:backdrop, infobar.error:backdrop {
      text-shadow: none; }
    infobar.info button, infobar.question button, infobar.warning button, infobar.error button {
      box-shadow: none;
      border-radius: 4px;
      color: #fff;
      background-image: image(#393A3E);
      border-style: none; }
      infobar.info button:hover, infobar.question button:hover, infobar.warning button:hover, infobar.error button:hover {
        color: #fff;
        background-image: image(#3790FA); }
      infobar.info button:active, infobar.question button:active, infobar.warning button:active, infobar.error button:active {
        color: #fff;
        background-image: image(#3790FA); }
      infobar.info button:checked, infobar.question button:checked, infobar.warning button:checked, infobar.error button:checked {
        color: #fff;
        background-image: image(#3790FA); }
      infobar.info button:disabled, infobar.question button:disabled, infobar.warning button:disabled, infobar.error button:disabled {
        color: #474747;
        background-image: image(#1C1C1E); }
      infobar.info button:backdrop, infobar.question button:backdrop, infobar.warning button:backdrop, infobar.error button:backdrop {
        color: #ffffff;
        background-image: image(#1C1C1E); }
      infobar.info button:backdrop:disabled, infobar.question button:backdrop:disabled, infobar.warning button:backdrop:disabled, infobar.error button:backdrop:disabled {
        color: #474747;
        background-image: image(#1C1C1E); }
    infobar.info selection, infobar.question selection, infobar.warning selection, infobar.error selection {
      background-color: #0674f8; }

/*****************
 * Tooltips *
 *****************/
tooltip {
  background-color: #0c0c0d;
  border: 1px solid #ffffff;
  border-radius: 4px; }
  tooltip label {
    color: #ffffff;
    text-shadow: none;
    padding: 0px; }

/************************
 * Color Chooser*
 ************************/
colorswatch {
  background-color: transparent; }
  colorswatch:drop(active), colorswatch {
    border-style: none; }
  colorswatch.top {
    border-top-left-radius: 5.5px;
    border-top-right-radius: 5.5px; }
    colorswatch.top overlay {
      border-top-left-radius: 5px;
      border-top-right-radius: 5px; }
  colorswatch.bottom {
    border-bottom-left-radius: 5.5px;
    border-bottom-right-radius: 5.5px; }
    colorswatch.bottom overlay {
      border-bottom-left-radius: 5px;
      border-bottom-right-radius: 5px; }
  colorswatch.left, colorswatch:first-child:not(.top) {
    border-top-left-radius: 5.5px;
    border-bottom-left-radius: 5.5px; }
    colorswatch.left overlay, colorswatch:first-child:not(.top) overlay {
      border-top-left-radius: 5px;
      border-bottom-left-radius: 5px; }
  colorswatch.right, colorswatch:last-child:not(.bottom) {
    border-top-right-radius: 5.5px;
    border-bottom-right-radius: 5.5px; }
    colorswatch.right overlay, colorswatch:last-child:not(.bottom) overlay {
      border-top-right-radius: 5px;
      border-bottom-right-radius: 5px; }
  colorswatch.dark {
    outline-color: rgba(255, 255, 255, 0.6); }
    colorswatch.dark overlay {
      color: white; }
      colorswatch.dark overlay:hover {
        border-color: rgba(0, 0, 0, 0.8); }
      colorswatch.dark overlay:backdrop {
        color: rgba(255, 255, 255, 0.5); }
  colorswatch.light {
    outline-color: rgba(0, 0, 0, 0.6); }
    colorswatch.light overlay {
      color: black; }
      colorswatch.light overlay:hover {
        border-color: rgba(0, 0, 0, 0.5); }
      colorswatch.light overlay:backdrop {
        color: rgba(0, 0, 0, 0.5); }
  colorswatch:drop(active) {
    box-shadow: none; }
    colorswatch:drop(active).light overlay {
      border-color: #0c0c0d;
      box-shadow: inset 0 0 0 2px black, inset 0 0 0 1px #0c0c0d; }
    colorswatch:drop(active).dark overlay {
      border-color: #0c0c0d;
      box-shadow: inset 0 0 0 2px rgba(0, 0, 0, 0.3), inset 0 0 0 1px #0c0c0d; }
  colorswatch overlay {
    border: 1px solid rgba(0, 0, 0, 0.3); }
    colorswatch overlay:hover {
      box-shadow: inset 0 1px rgba(255, 255, 255, 0.4), inset 0 -1px rgba(0, 0, 0, 0.2); }
    colorswatch overlay:backdrop, colorswatch overlay:backdrop:hover {
      border-color: rgba(0, 0, 0, 0.3);
      box-shadow: none; }
  colorswatch#add-color-button {
    border-radius: 5px 5px 0 0; }
    colorswatch#add-color-button:only-child {
      border-radius: 5px; }
    colorswatch#add-color-button overlay {
      color: #ffffff;
      outline-color: rgba(255, 255, 255, 0.3);
      border-color: #0c0c0d;
      background-image: image(#0c0c0d); }
      colorswatch#add-color-button overlay:hover {
        color: #ffffff;
        outline-color: rgba(255, 255, 255, 0.3);
        border-color: #0c0c0d;
        background-image: image(#0c0c0d); }
      colorswatch#add-color-button overlay:backdrop {
        border-color: #0c0c0d;
        background-image: image(#2D2E32);
        text-shadow: none;
        -gtk-icon-shadow: none; }
        colorswatch#add-color-button overlay:backdrop label, colorswatch#add-color-button overlay:backdrop {
          color: #ffffff; }
  colorswatch:disabled {
    opacity: 0.5; }
    colorswatch:disabled overlay {
      border-color: #1C1C1E;
      box-shadow: none; }
  row:selected colorswatch, .selection-mode.titlebar:not(headerbar) row.subtitle:link colorswatch,
  headerbar.selection-mode row.subtitle:link colorswatch {
    box-shadow: 0 0 0 2px #fff; }
  colorswatch#editor-color-sample {
    border-radius: 4px; }
    colorswatch#editor-color-sample overlay {
      border-radius: 4.5px; }

colorchooser .popover.osd {
  border-radius: 5px; }

/********
 * Misc *
 ********/
.content-view {
  background-color: transparent; }
  .content-view:hover {
    -gtk-icon-effect: highlight; }
  .content-view:backdrop, label:backdrop selection.content-view, progressbar:backdrop trough.content-view, progressbar:backdrop progress.content-view {
    background-color: transparent; }

.osd .scale-popup button.flat, .osd .scale-popup button.sidebar-button, .osd .scale-popup button.titlebutton {
  background-color: transparent;
  border-style: none;
  border-radius: 15px; }
.scale-popup button:hover {
  background-color: rgba(255, 255, 255, 0.1);
  border-radius: 5px; }

/**********************
 * Window Decorations *
 *********************/
decoration {
  box-shadow: 0 0 5px #888; }
  .maximized decoration, .fullscreen decoration, .tiled decoration, .tiled-top decoration, .tiled-right decoration, .tiled-bottom decoration, .tiled-left decoration {
    border-radius: 0px; }
  .popup decoration {
    box-shadow: none; }
  .ssd decoration {
    box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.23);
    border-style: solid;
    /* both border-color and background-color must be set to take effect */
    background-color: #0C0C0D; }
    .ssd decoration:backdrop {
      background-color: #0C0C0D; }
  .csd.popup decoration {
    box-shadow: 0 0 2px #0c0c0d; }
  tooltip.csd decoration {
    border-radius: 0;
    box-shadow: none; }
  messagedialog.csd decoration {
    border-radius: 0px;
    box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px rgba(0, 0, 0, 0.13); }
  .solid-csd decoration {
    margin: 0;
    padding: 4px;
    background-color: #0c0c0d;
    border-radius: 0;
    box-shadow: inset 0 0 0 3px #18181a, inset 0 1px rgba(0, 0, 0, 0.8); }
    .solid-csd decoration:backdrop {
      box-shadow: inset 0 0 0 3px #0c0c0d, inset 0 1px rgba(0, 0, 0, 0.8); }

button.titlebutton:backdrop {
  -gtk-icon-shadow: none; }

headerbar.selection-mode button.titlebutton:backdrop,
.titlebar.selection-mode button.titlebutton:backdrop {
  -gtk-icon-shadow: none; }

.view:selected:focus, iconview:selected:focus, .selection-mode.titlebar:not(headerbar) iconview.subtitle:focus:link,
headerbar.selection-mode iconview.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:focus:link,
.selection-mode.titlebar:not(headerbar) button.view.subtitle:focus:visited,
headerbar.selection-mode .view.subtitle:focus:link,
headerbar.selection-mode button.view.subtitle:focus:visited, .view:selected, iconview:selected, .selection-mode.titlebar:not(headerbar) iconview.subtitle:link,
headerbar.selection-mode iconview.subtitle:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:link,
.selection-mode.titlebar:not(headerbar) button.view.subtitle:visited,
headerbar.selection-mode .view.subtitle:link,
headerbar.selection-mode button.view.subtitle:visited,
.view text:selected:focus,
iconview text:selected:focus,
.view .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
.selection-mode.titlebar:not(headerbar) .view text.subtitle:focus:link,
iconview .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
.selection-mode.titlebar:not(headerbar) iconview text.subtitle:focus:link,
.view headerbar.selection-mode text.subtitle:focus:link,
headerbar.selection-mode .view text.subtitle:focus:link,
iconview headerbar.selection-mode text.subtitle:focus:link,
headerbar.selection-mode iconview text.subtitle:focus:link,
textview text:selected:focus,
textview .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
.selection-mode.titlebar:not(headerbar) textview text.subtitle:focus:link,
textview headerbar.selection-mode text.subtitle:focus:link,
headerbar.selection-mode textview text.subtitle:focus:link,
.view text:selected,
iconview text:selected,
.view .selection-mode.titlebar:not(headerbar) text.subtitle:link,
.selection-mode.titlebar:not(headerbar) .view text.subtitle:link,
iconview .selection-mode.titlebar:not(headerbar) text.subtitle:link,
.selection-mode.titlebar:not(headerbar) iconview text.subtitle:link,
.view headerbar.selection-mode text.subtitle:link,
headerbar.selection-mode .view text.subtitle:link,
iconview headerbar.selection-mode text.subtitle:link,
headerbar.selection-mode iconview text.subtitle:link,
textview text:selected,
textview .selection-mode.titlebar:not(headerbar) text.subtitle:link,
.selection-mode.titlebar:not(headerbar) textview text.subtitle:link,
textview headerbar.selection-mode text.subtitle:link,
headerbar.selection-mode textview text.subtitle:link, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
textview text selection:focus,
textview text selection, flowbox flowboxchild:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:link,
flowbox headerbar.selection-mode flowboxchild.subtitle:link,
headerbar.selection-mode flowbox flowboxchild.subtitle:link, label:disabled selection, label:backdrop selection, spinbutton:not(.vertical) selection,
entry selection, modelbutton.flat:selected, .selection-mode.titlebar:not(headerbar) modelbutton.flat.subtitle:link,
headerbar.selection-mode modelbutton.flat.subtitle:link,
.menuitem.button.flat:selected,
.selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:link,
.selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:link,
.selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:link,
.selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:visited,
.selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:link,
.selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:visited,
.selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:visited,
headerbar.selection-mode .menuitem.button.flat.subtitle:link,
headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:link,
headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:link,
headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:visited,
headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:link,
headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:visited,
headerbar.selection-mode button.menuitem.button.flat.subtitle:visited,
notebook > header > tabs > arrow.menuitem.button:selected,
button.menuitem.button.sidebar-button:selected,
button.menuitem.button.titlebutton:selected, row:selected, .selection-mode.titlebar:not(headerbar) row.subtitle:link,
headerbar.selection-mode row.subtitle:link, calendar:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:link,
headerbar.selection-mode calendar.subtitle:link, treeview.view:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:focus:link,
headerbar.selection-mode treeview.view.subtitle:focus:link, treeview.view:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:link,
headerbar.selection-mode treeview.view.subtitle:link {
  background-color: #3790FA; }
  row:selected label, .selection-mode.titlebar:not(headerbar) row.subtitle:link label,
  headerbar.selection-mode row.subtitle:link label, label:selected, .selection-mode.titlebar:not(headerbar) label.subtitle:link,
  headerbar.selection-mode label.subtitle:link, .selection-mode button.titlebutton, .view:selected:focus, iconview:selected:focus, .selection-mode.titlebar:not(headerbar) iconview.subtitle:focus:link,
  headerbar.selection-mode iconview.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:focus:link,
  .selection-mode.titlebar:not(headerbar) button.view.subtitle:focus:visited,
  headerbar.selection-mode .view.subtitle:focus:link,
  headerbar.selection-mode button.view.subtitle:focus:visited, .view:selected, iconview:selected, .selection-mode.titlebar:not(headerbar) iconview.subtitle:link,
  headerbar.selection-mode iconview.subtitle:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.view.subtitle:visited,
  headerbar.selection-mode .view.subtitle:link,
  headerbar.selection-mode button.view.subtitle:visited,
  .view text:selected:focus,
  iconview text:selected:focus,
  .view .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
  .selection-mode.titlebar:not(headerbar) .view text.subtitle:focus:link,
  iconview .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
  .selection-mode.titlebar:not(headerbar) iconview text.subtitle:focus:link,
  .view headerbar.selection-mode text.subtitle:focus:link,
  headerbar.selection-mode .view text.subtitle:focus:link,
  iconview headerbar.selection-mode text.subtitle:focus:link,
  headerbar.selection-mode iconview text.subtitle:focus:link,
  textview text:selected:focus,
  textview .selection-mode.titlebar:not(headerbar) text.subtitle:focus:link,
  .selection-mode.titlebar:not(headerbar) textview text.subtitle:focus:link,
  textview headerbar.selection-mode text.subtitle:focus:link,
  headerbar.selection-mode textview text.subtitle:focus:link,
  .view text:selected,
  iconview text:selected,
  .view .selection-mode.titlebar:not(headerbar) text.subtitle:link,
  .selection-mode.titlebar:not(headerbar) .view text.subtitle:link,
  iconview .selection-mode.titlebar:not(headerbar) text.subtitle:link,
  .selection-mode.titlebar:not(headerbar) iconview text.subtitle:link,
  .view headerbar.selection-mode text.subtitle:link,
  headerbar.selection-mode .view text.subtitle:link,
  iconview headerbar.selection-mode text.subtitle:link,
  headerbar.selection-mode iconview text.subtitle:link,
  textview text:selected,
  textview .selection-mode.titlebar:not(headerbar) text.subtitle:link,
  .selection-mode.titlebar:not(headerbar) textview text.subtitle:link,
  textview headerbar.selection-mode text.subtitle:link,
  headerbar.selection-mode textview text.subtitle:link, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
  textview text selection:focus,
  textview text selection, flowbox flowboxchild:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:link,
  flowbox headerbar.selection-mode flowboxchild.subtitle:link,
  headerbar.selection-mode flowbox flowboxchild.subtitle:link, label:disabled selection, label:backdrop selection, spinbutton:not(.vertical) selection,
  entry selection, modelbutton.flat:selected, .selection-mode.titlebar:not(headerbar) modelbutton.flat.subtitle:link,
  headerbar.selection-mode modelbutton.flat.subtitle:link,
  .menuitem.button.flat:selected,
  .selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:link,
  .selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:link,
  .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:visited,
  .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:link,
  .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:visited,
  .selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:visited,
  headerbar.selection-mode .menuitem.button.flat.subtitle:link,
  headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:link,
  headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:link,
  headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:visited,
  headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:link,
  headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:visited,
  headerbar.selection-mode button.menuitem.button.flat.subtitle:visited,
  notebook > header > tabs > arrow.menuitem.button:selected,
  button.menuitem.button.sidebar-button:selected,
  button.menuitem.button.titlebutton:selected, row:selected, .selection-mode.titlebar:not(headerbar) row.subtitle:link,
  headerbar.selection-mode row.subtitle:link, calendar:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:link,
  headerbar.selection-mode calendar.subtitle:link, treeview.view:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:focus:link,
  headerbar.selection-mode treeview.view.subtitle:focus:link, treeview.view:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:link,
  headerbar.selection-mode treeview.view.subtitle:link {
    color: #fff; }
    row:selected label:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link label:disabled,
    headerbar.selection-mode row.subtitle:link label:disabled, label:disabled:selected, .selection-mode.titlebar:not(headerbar) label.subtitle:disabled:link,
    headerbar.selection-mode label.subtitle:disabled:link, .selection-mode button.titlebutton:disabled, .view:disabled:selected:focus, iconview:disabled:selected:focus, .selection-mode.titlebar:not(headerbar) iconview.subtitle:disabled:focus:link,
    headerbar.selection-mode iconview.subtitle:disabled:focus:link, label:disabled selection.view:selected:focus, label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:focus:link,
    label:disabled headerbar.selection-mode selection.view.subtitle:focus:link,
    headerbar.selection-mode label:disabled selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:disabled:focus:link,
    .selection-mode.titlebar:not(headerbar) button.view.subtitle:disabled:focus:visited,
    headerbar.selection-mode .view.subtitle:disabled:focus:link,
    headerbar.selection-mode button.view.subtitle:disabled:focus:visited, .view:disabled:selected, iconview:disabled:selected, .selection-mode.titlebar:not(headerbar) iconview.subtitle:disabled:link,
    headerbar.selection-mode iconview.subtitle:disabled:link, label:disabled selection.view:selected, label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:link,
    label:disabled headerbar.selection-mode selection.view.subtitle:link,
    headerbar.selection-mode label:disabled selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) button.view.subtitle:disabled:visited,
    headerbar.selection-mode .view.subtitle:disabled:link,
    headerbar.selection-mode button.view.subtitle:disabled:visited,
    .view text:disabled:selected:focus,
    iconview text:disabled:selected:focus,
    .view .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:focus:link,
    .selection-mode.titlebar:not(headerbar) .view text.subtitle:disabled:focus:link,
    iconview .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:focus:link,
    .selection-mode.titlebar:not(headerbar) iconview text.subtitle:disabled:focus:link,
    .view headerbar.selection-mode text.subtitle:disabled:focus:link,
    headerbar.selection-mode .view text.subtitle:disabled:focus:link,
    iconview headerbar.selection-mode text.subtitle:disabled:focus:link,
    headerbar.selection-mode iconview text.subtitle:disabled:focus:link,
    textview text:disabled:selected:focus,
    textview .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:focus:link,
    .selection-mode.titlebar:not(headerbar) textview text.subtitle:disabled:focus:link,
    textview headerbar.selection-mode text.subtitle:disabled:focus:link,
    headerbar.selection-mode textview text.subtitle:disabled:focus:link,
    .view text:disabled:selected,
    iconview text:disabled:selected,
    .view .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) .view text.subtitle:disabled:link,
    iconview .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) iconview text.subtitle:disabled:link,
    .view headerbar.selection-mode text.subtitle:disabled:link,
    headerbar.selection-mode .view text.subtitle:disabled:link,
    iconview headerbar.selection-mode text.subtitle:disabled:link,
    headerbar.selection-mode iconview text.subtitle:disabled:link,
    textview text:disabled:selected,
    textview .selection-mode.titlebar:not(headerbar) text.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) textview text.subtitle:disabled:link,
    textview headerbar.selection-mode text.subtitle:disabled:link,
    headerbar.selection-mode textview text.subtitle:disabled:link, .view text selection:disabled:focus, iconview text selection:disabled:focus, .view text label:disabled selection:focus, label:disabled .view text selection:focus, iconview text label:disabled selection:focus, label:disabled iconview text selection:focus, .view text selection:disabled, iconview text selection:disabled, .view text label:disabled selection, label:disabled .view text selection, iconview text label:disabled selection, label:disabled iconview text selection,
    textview text selection:disabled:focus,
    textview text label:disabled selection:focus,
    label:disabled textview text selection:focus,
    textview text selection:disabled,
    textview text label:disabled selection,
    label:disabled textview text selection, flowbox flowboxchild:disabled:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:disabled:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:disabled:link,
    flowbox headerbar.selection-mode flowboxchild.subtitle:disabled:link,
    headerbar.selection-mode flowbox flowboxchild.subtitle:disabled:link, label:disabled selection, label:backdrop selection:disabled, label:backdrop label:disabled selection, label:disabled label:backdrop selection, spinbutton:not(.vertical) selection:disabled, spinbutton:not(.vertical) label:disabled selection, label:disabled spinbutton:not(.vertical) selection,
    entry selection:disabled,
    entry label:disabled selection,
    label:disabled entry selection, modelbutton.flat:disabled:selected, .selection-mode.titlebar:not(headerbar) modelbutton.flat.subtitle:disabled:link,
    headerbar.selection-mode modelbutton.flat.subtitle:disabled:link,
    .menuitem.button.flat:disabled:selected,
    label:disabled selection.menuitem.button.flat:selected,
    label:disabled .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) label:disabled selection.menuitem.button.flat.subtitle:link,
    label:disabled headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
    headerbar.selection-mode label:disabled selection.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:disabled:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:disabled:visited,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:disabled:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:disabled:visited,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:disabled:visited,
    headerbar.selection-mode .menuitem.button.flat.subtitle:disabled:link,
    headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:disabled:link,
    headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:disabled:link,
    headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:disabled:visited,
    headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:disabled:link,
    headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:disabled:visited,
    headerbar.selection-mode button.menuitem.button.flat.subtitle:disabled:visited,
    notebook > header > tabs > arrow.menuitem.button:disabled:selected,
    button.menuitem.button.sidebar-button:disabled:selected,
    button.menuitem.button.titlebutton:disabled:selected, row:disabled:selected, .selection-mode.titlebar:not(headerbar) row.subtitle:disabled:link,
    headerbar.selection-mode row.subtitle:disabled:link, calendar:disabled:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:disabled:link,
    headerbar.selection-mode calendar.subtitle:disabled:link, treeview.view:disabled:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:disabled:focus:link,
    headerbar.selection-mode treeview.view.subtitle:disabled:focus:link, treeview.view:disabled:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:disabled:link,
    headerbar.selection-mode treeview.view.subtitle:disabled:link {
      color: rgba(255, 255, 255, 0.3); }
    row:selected label:backdrop, .selection-mode.titlebar:not(headerbar) row.subtitle:link label:backdrop,
    headerbar.selection-mode row.subtitle:link label:backdrop, label:backdrop:selected, .selection-mode.titlebar:not(headerbar) label.subtitle:backdrop:link,
    headerbar.selection-mode label.subtitle:backdrop:link, .selection-mode button.titlebutton:backdrop, .view:backdrop:selected:focus, iconview:backdrop:selected:focus, .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:focus:link,
    headerbar.selection-mode iconview.subtitle:backdrop:focus:link, label:backdrop selection.view:selected:focus, label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:focus:link,
    label:backdrop headerbar.selection-mode selection.view.subtitle:focus:link,
    headerbar.selection-mode label:backdrop selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:focus:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.view.subtitle:focus:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.view.subtitle:focus:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:focus:link,
    .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:focus:visited,
    headerbar.selection-mode .view.subtitle:backdrop:focus:link,
    headerbar.selection-mode progressbar:backdrop trough.view.subtitle:focus:link,
    progressbar:backdrop headerbar.selection-mode trough.view.subtitle:focus:link,
    headerbar.selection-mode progressbar:backdrop progress.view.subtitle:focus:link,
    progressbar:backdrop headerbar.selection-mode progress.view.subtitle:focus:link,
    headerbar.selection-mode button.view.subtitle:backdrop:focus:visited, progressbar:backdrop trough.view:selected:focus, progressbar:backdrop progress.view:selected:focus, .view:backdrop:selected, iconview:backdrop:selected, .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:link,
    headerbar.selection-mode iconview.subtitle:backdrop:link, label:backdrop selection.view:selected, label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:link,
    label:backdrop headerbar.selection-mode selection.view.subtitle:link,
    headerbar.selection-mode label:backdrop selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.view.subtitle:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.view.subtitle:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:link,
    .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:visited,
    headerbar.selection-mode .view.subtitle:backdrop:link,
    headerbar.selection-mode progressbar:backdrop trough.view.subtitle:link,
    progressbar:backdrop headerbar.selection-mode trough.view.subtitle:link,
    headerbar.selection-mode progressbar:backdrop progress.view.subtitle:link,
    progressbar:backdrop headerbar.selection-mode progress.view.subtitle:link,
    headerbar.selection-mode button.view.subtitle:backdrop:visited, progressbar:backdrop trough.view:selected, progressbar:backdrop progress.view:selected,
    .view text:backdrop:selected:focus,
    iconview text:backdrop:selected:focus,
    .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:focus:link,
    .selection-mode.titlebar:not(headerbar) .view text.subtitle:backdrop:focus:link,
    iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:focus:link,
    .selection-mode.titlebar:not(headerbar) iconview text.subtitle:backdrop:focus:link,
    .view headerbar.selection-mode text.subtitle:backdrop:focus:link,
    headerbar.selection-mode .view text.subtitle:backdrop:focus:link,
    iconview headerbar.selection-mode text.subtitle:backdrop:focus:link,
    headerbar.selection-mode iconview text.subtitle:backdrop:focus:link,
    textview text:backdrop:selected:focus,
    textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:focus:link,
    .selection-mode.titlebar:not(headerbar) textview text.subtitle:backdrop:focus:link,
    textview headerbar.selection-mode text.subtitle:backdrop:focus:link,
    headerbar.selection-mode textview text.subtitle:backdrop:focus:link,
    .view text:backdrop:selected,
    iconview text:backdrop:selected,
    .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:link,
    .selection-mode.titlebar:not(headerbar) .view text.subtitle:backdrop:link,
    iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:link,
    .selection-mode.titlebar:not(headerbar) iconview text.subtitle:backdrop:link,
    .view headerbar.selection-mode text.subtitle:backdrop:link,
    headerbar.selection-mode .view text.subtitle:backdrop:link,
    iconview headerbar.selection-mode text.subtitle:backdrop:link,
    headerbar.selection-mode iconview text.subtitle:backdrop:link,
    textview text:backdrop:selected,
    textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:link,
    .selection-mode.titlebar:not(headerbar) textview text.subtitle:backdrop:link,
    textview headerbar.selection-mode text.subtitle:backdrop:link,
    headerbar.selection-mode textview text.subtitle:backdrop:link, .view text selection:backdrop:focus, iconview text selection:backdrop:focus, .view text label:backdrop selection:focus, label:backdrop .view text selection:focus, iconview text label:backdrop selection:focus, label:backdrop iconview text selection:focus, .view text selection:backdrop, iconview text selection:backdrop, .view text label:backdrop selection, label:backdrop .view text selection, iconview text label:backdrop selection, label:backdrop iconview text selection,
    textview text selection:backdrop:focus,
    textview text label:backdrop selection:focus,
    label:backdrop textview text selection:focus,
    textview text selection:backdrop,
    textview text label:backdrop selection,
    label:backdrop textview text selection, flowbox flowboxchild:backdrop:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:backdrop:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:backdrop:link,
    flowbox headerbar.selection-mode flowboxchild.subtitle:backdrop:link,
    headerbar.selection-mode flowbox flowboxchild.subtitle:backdrop:link, label:disabled selection:backdrop, label:disabled label:backdrop selection, label:backdrop label:disabled selection, label:backdrop selection, spinbutton:not(.vertical) selection:backdrop, spinbutton:not(.vertical) label:backdrop selection, label:backdrop spinbutton:not(.vertical) selection,
    entry selection:backdrop,
    entry label:backdrop selection,
    label:backdrop entry selection, modelbutton.flat:backdrop:selected, .selection-mode.titlebar:not(headerbar) modelbutton.flat.subtitle:backdrop:link,
    headerbar.selection-mode modelbutton.flat.subtitle:backdrop:link,
    .menuitem.button.flat:backdrop:selected,
    label:backdrop selection.menuitem.button.flat:selected,
    label:backdrop .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) label:backdrop selection.menuitem.button.flat.subtitle:link,
    label:backdrop headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
    headerbar.selection-mode label:backdrop selection.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:backdrop:link,
    .selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:link,
    .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.menuitem.button.flat.subtitle:link,
    progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.menuitem.button.flat.subtitle:link,
    progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.menuitem.button.flat.subtitle:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:backdrop:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:backdrop:visited,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:backdrop:link,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:backdrop:visited,
    .selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:backdrop:visited,
    headerbar.selection-mode .menuitem.button.flat.subtitle:backdrop:link,
    headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:link,
    headerbar.selection-mode progressbar:backdrop trough.menuitem.button.flat.subtitle:link,
    progressbar:backdrop headerbar.selection-mode trough.menuitem.button.flat.subtitle:link,
    headerbar.selection-mode progressbar:backdrop progress.menuitem.button.flat.subtitle:link,
    progressbar:backdrop headerbar.selection-mode progress.menuitem.button.flat.subtitle:link,
    headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:backdrop:link,
    headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:backdrop:visited,
    headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:backdrop:link,
    headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:backdrop:visited,
    headerbar.selection-mode button.menuitem.button.flat.subtitle:backdrop:visited,
    notebook > header > tabs > arrow.menuitem.button:backdrop:selected,
    progressbar:backdrop trough.menuitem.button.flat:selected,
    progressbar:backdrop progress.menuitem.button.flat:selected,
    button.menuitem.button.sidebar-button:backdrop:selected,
    button.menuitem.button.titlebutton:backdrop:selected, row:backdrop:selected, .selection-mode.titlebar:not(headerbar) row.subtitle:backdrop:link,
    headerbar.selection-mode row.subtitle:backdrop:link, calendar:backdrop:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:backdrop:link,
    headerbar.selection-mode calendar.subtitle:backdrop:link, treeview.view:backdrop:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:backdrop:focus:link,
    headerbar.selection-mode treeview.view.subtitle:backdrop:focus:link, treeview.view:backdrop:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:backdrop:link,
    headerbar.selection-mode treeview.view.subtitle:backdrop:link {
      color: #0c0c0d; }
      row:selected label:backdrop:disabled, .selection-mode.titlebar:not(headerbar) row.subtitle:link label:backdrop:disabled,
      headerbar.selection-mode row.subtitle:link label:backdrop:disabled, label:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) label.subtitle:backdrop:disabled:link,
      headerbar.selection-mode label.subtitle:backdrop:disabled:link, .selection-mode button.titlebutton:backdrop:disabled, .view:backdrop:disabled:selected:focus, iconview:backdrop:disabled:selected:focus, .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode iconview.subtitle:backdrop:disabled:focus:link, label:disabled selection.view:backdrop:selected:focus, label:disabled label:backdrop selection.view:selected:focus, label:backdrop label:disabled selection.view:selected:focus, label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link, label:backdrop .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:focus:link, label:disabled .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled selection.view.subtitle:focus:link,
      label:disabled label:backdrop headerbar.selection-mode selection.view.subtitle:focus:link,
      label:backdrop headerbar.selection-mode label:disabled selection.view.subtitle:focus:link,
      label:disabled headerbar.selection-mode label:backdrop selection.view.subtitle:focus:link,
      headerbar.selection-mode label:backdrop label:disabled selection.view.subtitle:focus:link, label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:focus:link, .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:backdrop:focus:link,
      label:disabled headerbar.selection-mode selection.view.subtitle:backdrop:focus:link,
      headerbar.selection-mode label:disabled selection.view.subtitle:backdrop:focus:link, label:backdrop selection.view:disabled:selected:focus, label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link, .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop selection.view.subtitle:focus:link,
      label:backdrop label:disabled headerbar.selection-mode selection.view.subtitle:focus:link,
      headerbar.selection-mode label:disabled label:backdrop selection.view.subtitle:focus:link, label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:focus:link, .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:disabled:focus:link,
      label:backdrop headerbar.selection-mode selection.view.subtitle:disabled:focus:link,
      headerbar.selection-mode label:backdrop selection.view.subtitle:disabled:focus:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:disabled:focus:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.view.subtitle:disabled:focus:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:focus:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.view.subtitle:disabled:focus:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .view.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode iconview.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled selection.view.subtitle:backdrop:focus:link,
      headerbar.selection-mode label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:focus:link,
      .selection-mode.titlebar:not(headerbar) label:disabled headerbar.selection-mode selection.view.subtitle:backdrop:focus:link,
      label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled label:backdrop selection.view.subtitle:focus:link,
      headerbar.selection-mode label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link,
      .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop headerbar.selection-mode selection.view.subtitle:focus:link,
      label:disabled label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop label:disabled selection.view.subtitle:focus:link,
      headerbar.selection-mode label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled headerbar.selection-mode selection.view.subtitle:focus:link,
      label:backdrop label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop selection.view.subtitle:disabled:focus:link,
      headerbar.selection-mode label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop headerbar.selection-mode selection.view.subtitle:disabled:focus:link,
      label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop trough.view.subtitle:disabled:focus:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode trough.view.subtitle:disabled:focus:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop progress.view.subtitle:disabled:focus:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode progress.view.subtitle:disabled:focus:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:focus:link, .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:disabled:focus:visited,
      headerbar.selection-mode .view.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode progressbar:backdrop trough.view.subtitle:disabled:focus:link,
      progressbar:backdrop headerbar.selection-mode trough.view.subtitle:disabled:focus:link,
      headerbar.selection-mode progressbar:backdrop progress.view.subtitle:disabled:focus:link,
      progressbar:backdrop headerbar.selection-mode progress.view.subtitle:disabled:focus:link,
      headerbar.selection-mode button.view.subtitle:backdrop:disabled:focus:visited, progressbar:backdrop trough.view:disabled:selected:focus, progressbar:backdrop progress.view:disabled:selected:focus, .view:backdrop:disabled:selected, iconview:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:disabled:link,
      headerbar.selection-mode iconview.subtitle:backdrop:disabled:link, label:disabled selection.view:backdrop:selected, label:disabled label:backdrop selection.view:selected, label:backdrop label:disabled selection.view:selected, label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link, label:backdrop .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:link, label:disabled .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled selection.view.subtitle:link,
      label:disabled label:backdrop headerbar.selection-mode selection.view.subtitle:link,
      label:backdrop headerbar.selection-mode label:disabled selection.view.subtitle:link,
      label:disabled headerbar.selection-mode label:backdrop selection.view.subtitle:link,
      headerbar.selection-mode label:backdrop label:disabled selection.view.subtitle:link, label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:link, .selection-mode.titlebar:not(headerbar) label:disabled selection.view.subtitle:backdrop:link,
      label:disabled headerbar.selection-mode selection.view.subtitle:backdrop:link,
      headerbar.selection-mode label:disabled selection.view.subtitle:backdrop:link, label:backdrop selection.view:disabled:selected, label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link, .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop selection.view.subtitle:link,
      label:backdrop label:disabled headerbar.selection-mode selection.view.subtitle:link,
      headerbar.selection-mode label:disabled label:backdrop selection.view.subtitle:link, label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:link, .selection-mode.titlebar:not(headerbar) label:backdrop selection.view.subtitle:disabled:link,
      label:backdrop headerbar.selection-mode selection.view.subtitle:disabled:link,
      headerbar.selection-mode label:backdrop selection.view.subtitle:disabled:link, .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:disabled:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.view.subtitle:disabled:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:link, .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.view.subtitle:disabled:link, progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .view.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .view.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode iconview.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) iconview.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled selection.view.subtitle:backdrop:link,
      headerbar.selection-mode label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:link,
      .selection-mode.titlebar:not(headerbar) label:disabled headerbar.selection-mode selection.view.subtitle:backdrop:link,
      label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:backdrop:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled label:backdrop selection.view.subtitle:link,
      headerbar.selection-mode label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop headerbar.selection-mode selection.view.subtitle:link,
      label:disabled label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop label:disabled selection.view.subtitle:link,
      headerbar.selection-mode label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled headerbar.selection-mode selection.view.subtitle:link,
      label:backdrop label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop selection.view.subtitle:disabled:link,
      headerbar.selection-mode label:backdrop .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop headerbar.selection-mode selection.view.subtitle:disabled:link,
      label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop trough.view.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode trough.view.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) trough.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop progress.view.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode progress.view.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) progress.view.subtitle:disabled:link, .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) button.view.subtitle:backdrop:disabled:visited,
      headerbar.selection-mode .view.subtitle:backdrop:disabled:link,
      headerbar.selection-mode progressbar:backdrop trough.view.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode trough.view.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop progress.view.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode progress.view.subtitle:disabled:link,
      headerbar.selection-mode button.view.subtitle:backdrop:disabled:visited, progressbar:backdrop trough.view:disabled:selected, progressbar:backdrop progress.view:disabled:selected,
      .view text:backdrop:disabled:selected:focus,
      iconview text:backdrop:disabled:selected:focus,
      .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) .view text.subtitle:backdrop:disabled:focus:link,
      iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) iconview text.subtitle:backdrop:disabled:focus:link,
      .view headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode .view text.subtitle:backdrop:disabled:focus:link,
      iconview headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode iconview text.subtitle:backdrop:disabled:focus:link,
      textview text:backdrop:disabled:selected:focus,
      textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) textview text.subtitle:backdrop:disabled:focus:link,
      textview headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode textview text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .view text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) .view headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      .view headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode iconview text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) iconview headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      iconview headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode textview text.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .selection-mode.titlebar:not(headerbar) textview headerbar.selection-mode text.subtitle:backdrop:disabled:focus:link,
      textview headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:focus:link,
      .view text:backdrop:disabled:selected,
      iconview text:backdrop:disabled:selected,
      .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) .view text.subtitle:backdrop:disabled:link,
      iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) iconview text.subtitle:backdrop:disabled:link,
      .view headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .view text.subtitle:backdrop:disabled:link,
      iconview headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode iconview text.subtitle:backdrop:disabled:link,
      textview text:backdrop:disabled:selected,
      textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) textview text.subtitle:backdrop:disabled:link,
      textview headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode textview text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .view text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .view .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) .view headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      .view headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode iconview text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode iconview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) iconview headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      iconview headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode textview text.subtitle:backdrop:disabled:link,
      headerbar.selection-mode textview .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) textview headerbar.selection-mode text.subtitle:backdrop:disabled:link,
      textview headerbar.selection-mode .selection-mode.titlebar:not(headerbar) text.subtitle:backdrop:disabled:link, .view text selection:backdrop:disabled:focus, iconview text selection:backdrop:disabled:focus, .view text label:disabled selection:backdrop:focus, label:disabled .view text selection:backdrop:focus, iconview text label:disabled selection:backdrop:focus, label:disabled iconview text selection:backdrop:focus, .view text label:disabled label:backdrop selection:focus, label:disabled label:backdrop .view text selection:focus, iconview text label:disabled label:backdrop selection:focus, label:disabled label:backdrop iconview text selection:focus, .view text label:backdrop label:disabled selection:focus, label:backdrop label:disabled .view text selection:focus, iconview text label:backdrop label:disabled selection:focus, label:backdrop label:disabled iconview text selection:focus, .view text label:backdrop selection:disabled:focus, label:backdrop .view text selection:disabled:focus, iconview text label:backdrop selection:disabled:focus, label:backdrop iconview text selection:disabled:focus, .view text selection:backdrop:disabled, iconview text selection:backdrop:disabled, .view text label:disabled selection:backdrop, label:disabled .view text selection:backdrop, iconview text label:disabled selection:backdrop, label:disabled iconview text selection:backdrop, .view text label:disabled label:backdrop selection, label:disabled label:backdrop .view text selection, iconview text label:disabled label:backdrop selection, label:disabled label:backdrop iconview text selection, .view text label:backdrop label:disabled selection, label:backdrop label:disabled .view text selection, iconview text label:backdrop label:disabled selection, label:backdrop label:disabled iconview text selection, .view text label:backdrop selection:disabled, label:backdrop .view text selection:disabled, iconview text label:backdrop selection:disabled, label:backdrop iconview text selection:disabled,
      textview text selection:backdrop:disabled:focus,
      textview text label:disabled selection:backdrop:focus,
      label:disabled textview text selection:backdrop:focus,
      textview text label:disabled label:backdrop selection:focus,
      label:disabled label:backdrop textview text selection:focus,
      textview text label:backdrop label:disabled selection:focus,
      label:backdrop label:disabled textview text selection:focus,
      textview text label:backdrop selection:disabled:focus,
      label:backdrop textview text selection:disabled:focus,
      textview text selection:backdrop:disabled,
      textview text label:disabled selection:backdrop,
      label:disabled textview text selection:backdrop,
      textview text label:disabled label:backdrop selection,
      label:disabled label:backdrop textview text selection,
      textview text label:backdrop label:disabled selection,
      label:backdrop label:disabled textview text selection,
      textview text label:backdrop selection:disabled,
      label:backdrop textview text selection:disabled, flowbox flowboxchild:backdrop:disabled:selected, flowbox .selection-mode.titlebar:not(headerbar) flowboxchild.subtitle:backdrop:disabled:link, .selection-mode.titlebar:not(headerbar) flowbox flowboxchild.subtitle:backdrop:disabled:link,
      flowbox headerbar.selection-mode flowboxchild.subtitle:backdrop:disabled:link,
      headerbar.selection-mode flowbox flowboxchild.subtitle:backdrop:disabled:link, label:disabled selection:backdrop, label:disabled label:backdrop selection, label:backdrop label:disabled selection, label:backdrop selection:disabled, spinbutton:not(.vertical) selection:backdrop:disabled, spinbutton:not(.vertical) label:disabled selection:backdrop, label:disabled spinbutton:not(.vertical) selection:backdrop, spinbutton:not(.vertical) label:disabled label:backdrop selection, label:disabled label:backdrop spinbutton:not(.vertical) selection, spinbutton:not(.vertical) label:backdrop label:disabled selection, label:backdrop label:disabled spinbutton:not(.vertical) selection, spinbutton:not(.vertical) label:backdrop selection:disabled, label:backdrop spinbutton:not(.vertical) selection:disabled,
      entry selection:backdrop:disabled,
      entry label:disabled selection:backdrop,
      label:disabled entry selection:backdrop,
      entry label:disabled label:backdrop selection,
      label:disabled label:backdrop entry selection,
      entry label:backdrop label:disabled selection,
      label:backdrop label:disabled entry selection,
      entry label:backdrop selection:disabled,
      label:backdrop entry selection:disabled, modelbutton.flat:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) modelbutton.flat.subtitle:backdrop:disabled:link,
      headerbar.selection-mode modelbutton.flat.subtitle:backdrop:disabled:link,
      .menuitem.button.flat:backdrop:disabled:selected,
      label:disabled selection.menuitem.button.flat:backdrop:selected,
      label:disabled label:backdrop selection.menuitem.button.flat:selected,
      label:backdrop label:disabled selection.menuitem.button.flat:selected,
      label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      label:backdrop .selection-mode.titlebar:not(headerbar) label:disabled selection.menuitem.button.flat.subtitle:link,
      label:disabled .selection-mode.titlebar:not(headerbar) label:backdrop selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled selection.menuitem.button.flat.subtitle:link,
      label:disabled label:backdrop headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
      label:backdrop headerbar.selection-mode label:disabled selection.menuitem.button.flat.subtitle:link,
      label:disabled headerbar.selection-mode label:backdrop selection.menuitem.button.flat.subtitle:link,
      headerbar.selection-mode label:backdrop label:disabled selection.menuitem.button.flat.subtitle:link,
      label:disabled .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:backdrop:link,
      .selection-mode.titlebar:not(headerbar) label:disabled selection.menuitem.button.flat.subtitle:backdrop:link,
      label:disabled headerbar.selection-mode selection.menuitem.button.flat.subtitle:backdrop:link,
      headerbar.selection-mode label:disabled selection.menuitem.button.flat.subtitle:backdrop:link,
      label:backdrop selection.menuitem.button.flat:disabled:selected,
      label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop selection.menuitem.button.flat.subtitle:link,
      label:backdrop label:disabled headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
      headerbar.selection-mode label:disabled label:backdrop selection.menuitem.button.flat.subtitle:link,
      label:backdrop .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop selection.menuitem.button.flat.subtitle:disabled:link,
      label:backdrop headerbar.selection-mode selection.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode label:backdrop selection.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .menuitem.button.flat.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .menuitem.button.flat.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled selection.menuitem.button.flat.subtitle:backdrop:link,
      headerbar.selection-mode label:disabled .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:backdrop:link,
      .selection-mode.titlebar:not(headerbar) label:disabled headerbar.selection-mode selection.menuitem.button.flat.subtitle:backdrop:link,
      label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:backdrop:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:disabled label:backdrop selection.menuitem.button.flat.subtitle:link,
      headerbar.selection-mode label:disabled label:backdrop .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:disabled label:backdrop headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
      label:disabled label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop label:disabled selection.menuitem.button.flat.subtitle:link,
      headerbar.selection-mode label:backdrop label:disabled .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop label:disabled headerbar.selection-mode selection.menuitem.button.flat.subtitle:link,
      label:backdrop label:disabled headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode label:backdrop selection.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode label:backdrop .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) label:backdrop headerbar.selection-mode selection.menuitem.button.flat.subtitle:disabled:link,
      label:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) selection.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop trough.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode trough.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) trough.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode progressbar:backdrop progress.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) progressbar:backdrop headerbar.selection-mode progress.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode .selection-mode.titlebar:not(headerbar) progress.menuitem.button.flat.subtitle:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:backdrop:disabled:link,
      headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:visited,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.subtitle.titlebutton:backdrop:disabled:visited,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:backdrop:disabled:link,
      .selection-mode.titlebar:not(headerbar) button.menuitem.button.flat.subtitle:backdrop:disabled:visited,
      headerbar.selection-mode .menuitem.button.flat.subtitle:backdrop:disabled:link,
      headerbar.selection-mode notebook > header > tabs > arrow.menuitem.button.subtitle:backdrop:disabled:link,
      headerbar.selection-mode progressbar:backdrop trough.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode trough.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode progressbar:backdrop progress.menuitem.button.flat.subtitle:disabled:link,
      progressbar:backdrop headerbar.selection-mode progress.menuitem.button.flat.subtitle:disabled:link,
      headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:link,
      headerbar.selection-mode button.menuitem.button.subtitle.sidebar-button:backdrop:disabled:visited,
      headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:backdrop:disabled:link,
      headerbar.selection-mode button.menuitem.button.subtitle.titlebutton:backdrop:disabled:visited,
      headerbar.selection-mode button.menuitem.button.flat.subtitle:backdrop:disabled:visited,
      notebook > header > tabs > arrow.menuitem.button:backdrop:disabled:selected,
      progressbar:backdrop trough.menuitem.button.flat:disabled:selected,
      progressbar:backdrop progress.menuitem.button.flat:disabled:selected,
      button.menuitem.button.sidebar-button:backdrop:disabled:selected,
      button.menuitem.button.titlebutton:backdrop:disabled:selected,
      .selection-mode button.menuitem.button.titlebutton:backdrop:disabled:selected, row:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) row.subtitle:backdrop:disabled:link,
      headerbar.selection-mode row.subtitle:backdrop:disabled:link, calendar:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) calendar.subtitle:backdrop:disabled:link,
      headerbar.selection-mode calendar.subtitle:backdrop:disabled:link, treeview.view:backdrop:disabled:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:backdrop:disabled:focus:link,
      headerbar.selection-mode treeview.view.subtitle:backdrop:disabled:focus:link, treeview.view:backdrop:disabled:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:backdrop:disabled:link,
      headerbar.selection-mode treeview.view.subtitle:backdrop:disabled:link {
        color: rgba(255, 255, 255, 0.3); }

.monospace {
  font-family: monospace; }

/**********************
 * Touch Copy & Paste *
 *********************/
cursor-handle {
  background-color: transparent;
  background-image: none;
  box-shadow: none;
  border-style: none; }
  cursor-handle.top:dir(ltr), cursor-handle.bottom:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-start.png"), url("assets/text-select-start@2.png"));
    padding-left: 10px; }
  cursor-handle.bottom:dir(ltr), cursor-handle.top:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-end.png"), url("assets/text-select-end@2.png"));
    padding-right: 10px; }
  cursor-handle.insertion-cursor:dir(ltr), cursor-handle.insertion-cursor:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png")); }
  cursor-handle.top:hover:dir(ltr), cursor-handle.bottom:hover:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-start-hover.png"), url("assets/text-select-start-hover@2.png"));
    padding-left: 10px; }
  cursor-handle.bottom:hover:dir(ltr), cursor-handle.top:hover:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-end-hover.png"), url("assets/text-select-end-hover@2.png"));
    padding-right: 10px; }
  cursor-handle.insertion-cursor:hover:dir(ltr), cursor-handle.insertion-cursor:hover:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-hover.png"), url("assets/slider-horz-scale-has-marks-above-hover@2.png")); }
  cursor-handle.top:active:dir(ltr), cursor-handle.bottom:active:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-start-active.png"), url("assets/text-select-start-active@2.png"));
    padding-left: 10px; }
  cursor-handle.bottom:active:dir(ltr), cursor-handle.top:active:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/text-select-end-active.png"), url("assets/text-select-end-active@2.png"));
    padding-right: 10px; }
  cursor-handle.insertion-cursor:active:dir(ltr), cursor-handle.insertion-cursor:active:dir(rtl) {
    -gtk-icon-source: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-active.png"), url("assets/slider-horz-scale-has-marks-above-active@2.png")); }

.context-menu {
  font: initial; }

.keycap {
  min-width: 20px;
  min-height: 25px;
  margin-top: 2px;
  padding-bottom: 3px;
  padding-left: 6px;
  padding-right: 6px;
  color: #ffffff;
  background-color: #0c0c0d;
  border: 1px solid;
  border-color: #0c0c0d;
  border-radius: 5px;
  box-shadow: inset 0 -3px #0c0c0d;
  font-size: smaller; }
  .keycap:backdrop, label:backdrop selection.keycap, progressbar:backdrop trough.keycap, progressbar:backdrop progress.keycap {
    background-color: #0c0c0d;
    color: #ffffff;
    transition: 200ms ease-out; }

:not(decoration):not(window):drop(active):focus, combobox:drop(active) button.combo:not(decoration):not(window):focus,
:not(decoration):not(window):drop(active),
combobox:drop(active) button.combo:not(decoration):not(window) {
  border-color: #0c0c0d;
  box-shadow: inset 0 0 0 1px #0c0c0d;
  caret-color: #0c0c0d; }

stackswitcher button.text-button {
  min-width: 100px; }

stackswitcher button.circular,
stackswitcher button.text-button.circular {
  min-width: 32px;
  min-height: 32px;
  padding: 0; }

/**********************
 * Base States *
 **********************/
button, combobox button, menubar,
.menubar, combobox, switch, slider, spinbutton {
  border-radius: 4px;
  color: #ffffff;
  box-shadow: none;
  background-color: #0c0c0d;
  border-style: none; }
  button:focus, menubar:focus,
  .menubar:focus, combobox:focus, switch:focus, slider:focus, spinbutton:focus {
    color: #ffffff;
    background-color: #0c0c0d; }
  button:hover, menubar:hover,
  .menubar:hover, combobox:hover, switch:hover, slider:hover, spinbutton:hover {
    color: #fff;
    background-color: #3790FA; }
  button:active, menubar:active,
  .menubar:active, combobox:active, switch:active, slider:active, spinbutton:active {
    color: #fff;
    background-color: #3790FA; }
  button:checked, menubar:checked,
  .menubar:checked, combobox:checked, switch:checked, slider:checked, spinbutton:checked {
    color: #fff;
    background-color: #3790FA; }
  label:disabled selection.menubar, button:disabled, menubar:disabled,
  .menubar:disabled, combobox:disabled, switch:disabled, slider:disabled, spinbutton:disabled {
    color: rgba(255, 255, 255, 0.3);
    background-color: #1C1C1E; }
  label:backdrop selection.menubar,
  progressbar:backdrop trough.menubar,
  progressbar:backdrop progress.menubar, button:backdrop, menubar:backdrop,
  .menubar:backdrop, combobox:backdrop, switch:backdrop, slider:backdrop, spinbutton:backdrop {
    color: #ffffff;
    background-color: #0c0c0d; }
  label:disabled label:backdrop selection.menubar,
  label:backdrop label:disabled selection.menubar,
  label:disabled selection.menubar:backdrop,
  label:backdrop selection.menubar:disabled,
  progressbar:backdrop trough.menubar:disabled,
  progressbar:backdrop progress.menubar:disabled, button:backdrop:disabled, menubar:backdrop:disabled,
  .menubar:backdrop:disabled, combobox:backdrop:disabled, switch:backdrop:disabled, slider:backdrop:disabled, spinbutton:backdrop:disabled {
    color: rgba(255, 255, 255, 0.3);
    background-image: image(#1C1C1E); }

/**************
 * Buttons *
 **************/
button, combobox button {
  box-shadow: none;
  border-radius: 4px;
  color: #fff;
  background: #393A3E;
  border-style: none; }
  button:focus {
    color: #fff;
    background-image: image(#3790FA); }
  button:hover {
    color: #fff;
    background-image: image(#3790FA); }
  button:active {
    color: #fff;
    background-image: image(#3790FA); }
  button:checked {
    color: #fff;
    background-image: image(#3790FA); }
  button:disabled {
    color: #474747;
    background-image: image(#1C1C1E); }
  button:backdrop {
    color: #ffffff;
    background-image: image(#1C1C1E); }
  button:backdrop:disabled {
    color: #474747;
    background-image: image(#1C1C1E); }

button {
  min-height: 24px; }

/************
 * Menus *
 ************/
menubar,
.menubar {
  background: #3D3E42;
  min-height: 22px; }
  label:disabled selection.menubar, menubar:disabled,
  .menubar:disabled {
    background-image: image(#1C1C1E);
    color: rgba(255, 255, 255, 0.3); }

menubar,
.menubar {
  -GtkWidget-window-dragging: true;
  border-radius: 0px;
  color: #ffffff;
  background-color: #2D2E32; }
  menubar:backdrop,
  .menubar:backdrop,
  label:backdrop selection.menubar,
  progressbar:backdrop trough.menubar,
  progressbar:backdrop progress.menubar {
    background-color: #2D2E32; }
  menubar > menuitem,
  .menubar > menuitem {
    margin-bottom: 3px;
    border-radius: 4px;
    min-height: 16px;
    padding: 4px 8px;
    box-shadow: none;
    color: #ffffff; }
    menubar > menuitem:hover,
    .menubar > menuitem:hover {
      box-shadow: none;
      color: #fff;
      background-color: #3790FA; }
    menubar > menuitem:disabled,
    .menubar > menuitem:disabled {
      color: rgba(255, 255, 255, 0.3);
      box-shadow: none; }
  menubar menu,
  .menubar menu {
    border-radius: 0px; }
    menubar menu > menuitem,
    .menubar menu > menuitem {
      margin-left: 5px;
      margin-right: 5px; }
      menubar menu > menuitem:disabled,
      .menubar menu > menuitem:disabled {
        background-color: #3D3E42; }

menu, .menu, .context-menu {
  margin: 4px;
  padding: 2px 0px;
  background-color: #3D3E42;
  border: 1px solid #0c0c0d; }
  .csd menu,
  .csd .menu,
  .csd .context-menu {
    border: none; }
  menu:backdrop, .menu:backdrop, label:backdrop selection.menu, progressbar:backdrop trough.menu, progressbar:backdrop progress.menu, .context-menu:backdrop, label:backdrop selection.context-menu, progressbar:backdrop trough.context-menu, progressbar:backdrop progress.context-menu {
    background-color: #3D3E42; }
  menu menuitem, .menu menuitem, .context-menu menuitem {
    min-height: 16px;
    min-width: 40px;
    padding: 4px 6px;
    text-shadow: none;
    border-radius: 4px; }
    menu menuitem:hover, .menu menuitem:hover, .context-menu menuitem:hover {
      box-shadow: none;
      color: #fff;
      background-color: #3790FA; }
    menu menuitem:disabled, .menu menuitem:disabled, .context-menu menuitem:disabled {
      color: rgba(255, 255, 255, 0.3); }
      menu menuitem:disabled:backdrop, .menu menuitem:disabled:backdrop, .context-menu menuitem:disabled:backdrop {
        color: rgba(255, 255, 255, 0.3); }
    menu menuitem:backdrop, menu menuitem:backdrop:hover, .menu menuitem:backdrop, .menu menuitem:backdrop:hover, .context-menu menuitem:backdrop, .context-menu menuitem:backdrop:hover {
      color: #ffffff;
      background-color: transparent; }
    menu menuitem arrow, .menu menuitem arrow, .context-menu menuitem arrow {
      min-height: 16px;
      min-width: 16px; }
      menu menuitem arrow:dir(ltr), .menu menuitem arrow:dir(ltr), .context-menu menuitem arrow:dir(ltr) {
        -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
        margin-left: 10px; }
      menu menuitem arrow:dir(rtl), .menu menuitem arrow:dir(rtl), .context-menu menuitem arrow:dir(rtl) {
        -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl");
        margin-right: 10px; }
    menu menuitem label:dir(rtl), menu menuitem label:dir(ltr), .menu menuitem label:dir(rtl), .menu menuitem label:dir(ltr), .context-menu menuitem label:dir(rtl), .context-menu menuitem label:dir(ltr) {
      color: inherit; }
  menu > arrow, .menu > arrow, .context-menu > arrow {
    border-color: transparent;
    background-color: transparent;
    text-shadow: none;
    -gtk-icon-shadow: none;
    min-height: 16px;
    min-width: 16px;
    padding: 4px;
    background-color: #3D3E42; }
    menu > arrow:hover, .menu > arrow:hover, .context-menu > arrow:hover {
      box-shadow: none;
      color: #fff;
      background-color: #3790FA; }
    menu > arrow.top, .menu > arrow.top, .context-menu > arrow.top {
      margin-top: -6px;
      border-bottom: 1px solid #242425;
      -gtk-icon-source: -gtk-icontheme("pan-up-symbolic"); }
    menu > arrow.bottom, .menu > arrow.bottom, .context-menu > arrow.bottom {
      margin-bottom: -6px;
      border-top: 1px solid #242425;
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
    menu > arrow:backdrop, .menu > arrow:backdrop, .context-menu > arrow:backdrop {
      background-color: #0c0c0d; }
    menu > arrow:disabled, .menu > arrow:disabled, .context-menu > arrow:disabled {
      color: rgba(255, 255, 255, 0.3);
      background-color: #1C1C1E;
      border-color: #1C1C1E; }

/******************
 * ComboBoxes*
 ******************/
combobox {
  /* When combobox isn't clicked, it uses button's style. i don't know what the hell. */
  /* When combobox entry isn't clicked, it uses entry's style. I don't want to change it. */ }
  combobox menu menuitem {
    border-radius: 4px;
    min-height: 23px;
    background-color: #1C1C1E; }
    combobox menu menuitem:focus {
      color: #fff;
      background-color: #3790FA; }
    combobox menu menuitem:hover {
      color: #fff;
      background-color: #3790FA; }
  combobox button {
    background-image: image(#2C2C2E); }

/***********************
 * Text Entries *
 **********************/
entry {
  background-color: #242424;
  border-radius: 4px; }

/***************
 * Notebooks*
 ***************/
notebook > header > tabs > tab button {
  border-radius: 10px;
  border-width: 0;
  background-image: image(#2D2E32); }
notebook > header > tabs > tab:checked button {
  border-radius: 10px;
  border-width: 0;
  background-image: image(#0c0c0d); }
notebook > header > tabs > tab:hover button {
  border-radius: 10px;
  border-width: 0;
  background-image: image(#cccccc); }

notebook > header > tabs > tab {
  /* close button in tab */ }

/*********************************
 * Check and Radio items*
 *********************************/
.view.content-view.check:not(list), iconview.content-view.check:not(list),
.content-view .tile .check:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-selectionmode.png"), url("assets/checkbox-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:hover:not(list), iconview.content-view.check:hover:not(list),
.content-view .tile .check:hover:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-hover-selectionmode.png"), url("assets/checkbox-hover-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:active:not(list), iconview.content-view.check:active:not(list),
.content-view .tile .check:active:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-active-selectionmode.png"), url("assets/checkbox-active-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:backdrop:not(list), iconview.content-view.check:backdrop:not(list), label:backdrop selection.view.content-view.check:not(list), progressbar:backdrop trough.view.content-view.check:not(list), progressbar:backdrop progress.view.content-view.check:not(list),
.content-view .tile .check:backdrop:not(list),
.content-view .tile label:backdrop selection.check:not(list),
label:backdrop .content-view .tile selection.check:not(list),
.content-view .tile progressbar:backdrop trough.check:not(list),
progressbar:backdrop .content-view .tile trough.check:not(list),
.content-view .tile progressbar:backdrop progress.check:not(list),
progressbar:backdrop .content-view .tile progress.check:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-backdrop-selectionmode.png"), url("assets/checkbox-backdrop-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:checked:not(list), iconview.content-view.check:checked:not(list),
.content-view .tile .check:checked:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-checked-selectionmode.png"), url("assets/checkbox-checked-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:checked:hover:not(list), iconview.content-view.check:checked:hover:not(list),
.content-view .tile .check:checked:hover:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-checked-hover-selectionmode.png"), url("assets/checkbox-checked-hover-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:checked:active:not(list), iconview.content-view.check:checked:active:not(list),
.content-view .tile .check:checked:active:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-checked-active-selectionmode.png"), url("assets/checkbox-checked-active-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

.view.content-view.check:backdrop:checked:not(list), iconview.content-view.check:backdrop:checked:not(list), label:backdrop selection.view.content-view.check:checked:not(list), progressbar:backdrop trough.view.content-view.check:checked:not(list), progressbar:backdrop progress.view.content-view.check:checked:not(list),
.content-view .tile .check:backdrop:checked:not(list),
.content-view .tile label:backdrop selection.check:checked:not(list),
label:backdrop .content-view .tile selection.check:checked:not(list),
.content-view .tile progressbar:backdrop trough.check:checked:not(list),
progressbar:backdrop .content-view .tile trough.check:checked:not(list),
.content-view .tile progressbar:backdrop progress.check:checked:not(list),
progressbar:backdrop .content-view .tile progress.check:checked:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: -gtk-scaled(url("assets/checkbox-checked-backdrop-selectionmode.png"), url("assets/checkbox-checked-backdrop-selectionmode@2.png"));
  background-color: transparent;
  min-width: 40px;
  min-height: 40px;
  background-image: none;
  transition: none;
  box-shadow: none;
  border-width: 0; }

checkbutton.text-button, radiobutton.text-button {
  padding: 2px 0;
  outline-offset: 0; }
  checkbutton.text-button label:not(:only-child):first-child, radiobutton.text-button label:not(:only-child):first-child {
    margin-left: 4px; }
  checkbutton.text-button label:not(:only-child):last-child, radiobutton.text-button label:not(:only-child):last-child {
    margin-right: 4px; }

menu menuitem radio,
menu menuitem check, check, radio {
  border-radius: 4px;
  color: #ffffff;
  margin: 2px 6px 0 0px;
  min-height: 14px;
  min-width: 14px;
  border: 1px solid;
  box-shadow: none;
  -gtk-icon-shadow: none;
  -gtk-icon-source: none;
  background-image: image(#0c0c0d);
  border-color: #48484D; }
  menu menuitem radio:only-child,
  menu menuitem check:only-child,
  check:only-child,
  radio:only-child {
    margin: 0; }
  popover menu menuitem radio.left:dir(rtl), menu menuitem popover radio.left:dir(rtl),
  popover menu menuitem check.left:dir(rtl),
  menu menuitem popover check.left:dir(rtl),
  popover check.left:dir(rtl),
  popover radio.left:dir(rtl) {
    margin-left: 0;
    margin-right: 12px; }
  popover menu menuitem radio.right:dir(ltr), menu menuitem popover radio.right:dir(ltr),
  popover menu menuitem check.right:dir(ltr),
  menu menuitem popover check.right:dir(ltr),
  popover check.right:dir(ltr),
  popover radio.right:dir(ltr) {
    margin-left: 12px;
    margin-right: 0; }
  menu menuitem radio:focus,
  menu menuitem check:focus, check:focus, radio:focus {
    border-color: #6B8EEB;
    background-image: image(#2D418D);
    color: #2D418D; }
  menu menuitem radio:hover,
  menu menuitem check:hover, check:hover, radio:hover {
    border-color: #6B8EEB;
    background-image: image(#2D418D);
    color: #2D418D; }
  menu menuitem radio:checked,
  menu menuitem check:checked, check:checked, radio:checked {
    background-image: image(#3790FA);
    border-color: #415FC4;
    color: #ffffff; }
    menu menuitem radio:checked:disabled,
    menu menuitem check:checked:disabled, check:checked:disabled, radio:checked:disabled {
      background-image: image(#1C1C1E);
      border-color: #1C1C1E;
      color: #474747; }
  .selection-mode.titlebar:not(headerbar) menu menuitem radio.subtitle:link, menu menuitem .selection-mode.titlebar:not(headerbar) radio.subtitle:link,
  .selection-mode.titlebar:not(headerbar) menu menuitem check.subtitle:link,
  menu menuitem .selection-mode.titlebar:not(headerbar) check.subtitle:link,
  headerbar.selection-mode menu menuitem radio.subtitle:link,
  menu menuitem headerbar.selection-mode radio.subtitle:link,
  headerbar.selection-mode menu menuitem check.subtitle:link,
  menu menuitem headerbar.selection-mode check.subtitle:link, menu menuitem radio:selected,
  menu menuitem check:selected, check:selected, .selection-mode.titlebar:not(headerbar) check.subtitle:link,
  headerbar.selection-mode check.subtitle:link, radio:selected, .selection-mode.titlebar:not(headerbar) radio.subtitle:link,
  headerbar.selection-mode radio.subtitle:link {
    background-image: image(#3790FA);
    border-color: #415FC4;
    color: #ffffff; }
    .selection-mode.titlebar:not(headerbar) menu menuitem radio.subtitle:disabled:link, menu menuitem .selection-mode.titlebar:not(headerbar) radio.subtitle:disabled:link,
    .selection-mode.titlebar:not(headerbar) menu menuitem check.subtitle:disabled:link,
    menu menuitem .selection-mode.titlebar:not(headerbar) check.subtitle:disabled:link,
    headerbar.selection-mode menu menuitem radio.subtitle:disabled:link,
    menu menuitem headerbar.selection-mode radio.subtitle:disabled:link,
    headerbar.selection-mode menu menuitem check.subtitle:disabled:link,
    menu menuitem headerbar.selection-mode check.subtitle:disabled:link, menu menuitem radio:selected:disabled,
    menu menuitem check:selected:disabled, check:selected:disabled, .selection-mode.titlebar:not(headerbar) check.subtitle:disabled:link,
    headerbar.selection-mode check.subtitle:disabled:link, radio:selected:disabled, .selection-mode.titlebar:not(headerbar) radio.subtitle:disabled:link,
    headerbar.selection-mode radio.subtitle:disabled:link {
      background-image: image(#1C1C1E);
      border-color: #1C1C1E;
      color: #474747; }
  menu menuitem radio:disabled,
  menu menuitem check:disabled, check:disabled, radio:disabled {
    background-image: image(#1C1C1E);
    border-color: #1C1C1E;
    color: #474747; }
  menu menuitem radio:backdrop,
  menu menuitem check:backdrop, check:backdrop, radio:backdrop {
    background-image: image(#1C1C1E);
    border-color: #1C1C1E;
    color: #343438; }
    menu menuitem radio:backdrop:disabled,
    menu menuitem check:backdrop:disabled, check:backdrop:disabled, radio:backdrop:disabled {
      background-image: image(#1C1C1E);
      border-color: #1C1C1E;
      color: #474747; }
  row:selected menu menuitem radio, menu menuitem row:selected radio, .selection-mode.titlebar:not(headerbar) row.subtitle:link menu menuitem radio, menu menuitem .selection-mode.titlebar:not(headerbar) row.subtitle:link radio,
  headerbar.selection-mode row.subtitle:link menu menuitem radio,
  menu menuitem headerbar.selection-mode row.subtitle:link radio,
  row:selected menu menuitem check,
  menu menuitem row:selected check,
  .selection-mode.titlebar:not(headerbar) row.subtitle:link menu menuitem check,
  menu menuitem .selection-mode.titlebar:not(headerbar) row.subtitle:link check,
  headerbar.selection-mode row.subtitle:link menu menuitem check,
  menu menuitem headerbar.selection-mode row.subtitle:link check, row:selected check, .selection-mode.titlebar:not(headerbar) row.subtitle:link check,
  headerbar.selection-mode row.subtitle:link check, row:selected radio, .selection-mode.titlebar:not(headerbar) row.subtitle:link radio,
  headerbar.selection-mode row.subtitle:link radio {
    border-color: #3790FA; }
  .osd menu menuitem radio, menu menuitem .osd radio,
  .osd menu menuitem check,
  menu menuitem .osd check, .osd check, .osd radio {
    color: #ffffff;
    border-color: rgba(0, 0, 0, 0.7);
    background-color: transparent;
    background-image: image(rgba(28, 31, 32, 0.7));
    background-clip: padding-box;
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
    text-shadow: 0 1px white;
    -gtk-icon-shadow: 0 1px white;
    outline-color: rgba(255, 255, 255, 0.3); }
    .osd menu menuitem radio:hover, menu menuitem .osd radio:hover,
    .osd menu menuitem check:hover,
    menu menuitem .osd check:hover, .osd check:hover, .osd radio:hover {
      color: #ffffff;
      border-color: rgba(0, 0, 0, 0.7);
      background-color: transparent;
      background-image: image(rgba(28, 31, 32, 0.7));
      background-clip: padding-box;
      box-shadow: inset 0 1px rgba(255, 255, 255, 0.1);
      text-shadow: 0 1px white;
      -gtk-icon-shadow: 0 1px white;
      outline-color: rgba(255, 255, 255, 0.3); }
    .osd menu menuitem radio:active, menu menuitem .osd radio:active,
    .osd menu menuitem check:active,
    menu menuitem .osd check:active, .osd check:active, .osd radio:active {
      color: white;
      border-color: rgba(0, 0, 0, 0.7);
      background-color: transparent;
      background-image: image(rgba(0, 0, 0, 0.7));
      background-clip: padding-box;
      box-shadow: none;
      text-shadow: none;
      -gtk-icon-shadow: none;
      outline-color: rgba(255, 255, 255, 0.3); }
    .osd menu menuitem radio:backdrop, menu menuitem .osd radio:backdrop,
    .osd menu menuitem check:backdrop,
    menu menuitem .osd check:backdrop, .osd check:backdrop, .osd radio:backdrop {
      color: #ffffff;
      border-color: rgba(0, 0, 0, 0.7);
      background-color: transparent;
      background-image: image(rgba(28, 31, 32, 0.7));
      background-clip: padding-box;
      box-shadow: none;
      text-shadow: none;
      -gtk-icon-shadow: none; }
    .osd menu menuitem radio:disabled, menu menuitem .osd radio:disabled,
    .osd menu menuitem check:disabled,
    menu menuitem .osd check:disabled, .osd check:disabled, .osd radio:disabled {
      color: rgba(255, 255, 255, 0.3);
      border-color: rgba(0, 0, 0, 0.7);
      background-color: transparent;
      background-image: image(#1C1C1E);
      background-clip: padding-box;
      box-shadow: none;
      text-shadow: none;
      -gtk-icon-shadow: none; }

menu menuitem radio:checked,
menu menuitem check:checked, treeview.view check:checked,
check:checked {
  -gtk-icon-source: image(-gtk-recolor(url("assets/check-symbolic.svg")), -gtk-recolor(url("assets/check-symbolic.symbolic.png"))); }
menu menuitem radio:indeterminate,
menu menuitem check:indeterminate, treeview.view check:indeterminate,
check:indeterminate {
  -gtk-icon-source: image(-gtk-recolor(url("assets/dash-symbolic.svg")), -gtk-recolor(url("assets/dash-symbolic.symbolic.png"))); }

menu menuitem radio, treeview.view radio,
radio {
  border-radius: 100%; }
  menu menuitem radio:checked, treeview.view radio:checked,
  radio:checked {
    -gtk-icon-source: image(-gtk-recolor(url("assets/bullet-symbolic.svg")), -gtk-recolor(url("assets/bullet-symbolic.symbolic.png"))); }
  menu menuitem radio:indeterminate, treeview.view radio:indeterminate,
  radio:indeterminate {
    -gtk-icon-source: image(-gtk-recolor(url("assets/dash-symbolic.svg")), -gtk-recolor(url("assets/dash-symbolic.symbolic.png"))); }

radio:not(:indeterminate):not(:checked):active:not(:backdrop) {
  -gtk-icon-transform: scale(0); }

check:not(:indeterminate):not(:checked):active:not(:backdrop) {
  -gtk-icon-transform: translate(6px, -3px) rotate(-45deg) scaleY(0.2) rotate(45deg) scaleX(0); }

menu menuitem radio:active,
menu menuitem check:active,
radio:active,
check:active {
  -gtk-icon-transform: scale(0, 1); }
menu menuitem radio:checked:not(:backdrop),
menu menuitem check:checked:not(:backdrop), menu menuitem radio:indeterminate:not(:backdrop),
menu menuitem check:indeterminate:not(:backdrop),
radio:checked:not(:backdrop),
radio:indeterminate:not(:backdrop),
check:checked:not(:backdrop),
check:indeterminate:not(:backdrop) {
  -gtk-icon-transform: unset;
  transition: 400ms; }

treeview.view check {
  background-image: image(#0c0c0d); }
  treeview.view check:checked {
    background-image: image(#3790FA);
    border-color: #415FC4;
    color: #ffffff; }
    treeview.view check:checked:disabled {
      background-image: image(#3790FA);
      border-color: #415FC4;
      color: #ffffff; }
      treeview.view check:checked:disabled:hover {
        background-image: image(#3790FA);
        border-color: #415FC4;
        color: #ffffff; }
  treeview.view check:disabled {
    background-image: image(#0c0c0d);
    border-color: #1C1C1E;
    color: #474747; }
    treeview.view check:disabled:hover {
      border-color: #6B8EEB;
      background-image: image(#2D418D);
      color: #2D418D; }
  treeview.view check:backdrop {
    background-image: image(#1C1C1E);
    border-color: #1C1C1E;
    color: #343438; }
    treeview.view check:backdrop:disabled {
      background-image: image(#1C1C1E);
      border-color: #1C1C1E;
      color: #474747; }

/*********************************
 * Switch & its slider *
 ********************************/
/**************
 * SpinButton *
 **************/
/*************************************
 * trough in scale/progress bar *
 *************************************/
/**************
 * Scale *
 **************/
scale trough {
  /* trough - not filled part */
  background-color: #f8f8f8;
  border-color: #0c0c0d;
  /* trough - filled part (highlight or progress) */ }
  scale trough:disabled {
    background-color: #1C1C1E;
    border-color: #1C1C1E; }
  scale trough highlight {
    background-color: #3790FA;
    border-color: #0c0c0d; }
    scale trough highlight:disabled {
      background-color: #1C1C1E;
      border-color: #1C1C1E; }
scale.horizontal trough {
  min-height: 6px;
  /* slider */ }
  scale.horizontal trough highlight {
    min-height: 6px; }
  scale.horizontal trough slider {
    min-width: 7px;
    min-height: 15px; }
scale.vertical trough {
  min-width: 6px;
  /* slider */ }
  scale.vertical trough highlight {
    min-width: 6px; }
  scale.vertical trough slider {
    min-width: 15px;
    min-height: 7px; }

/**************
 * Progessbar *
 **************/
progressbar trough {
  /* trough - not filled part */
  background-color: #f8f8f8;
  border-color: #0c0c0d;
  /* trough - filled part (highlight or progress) */ }
  progressbar trough:disabled {
    background-color: #1C1C1E;
    border-color: #1C1C1E; }
  progressbar trough progress {
    background-color: #3790FA;
    border-color: #0c0c0d; }
    progressbar trough progress:disabled {
      background-color: #1C1C1E;
      border-color: #1C1C1E; }
progressbar.horizontal trough {
  min-height: 14px; }
  progressbar.horizontal trough progress {
    min-height: 14px; }
progressbar.vertical trough {
  min-width: 14px; }
  progressbar.vertical trough progress {
    min-width: 14px; }

/*********
 * Frame *
 *********/
frame border {
  color: #0c0c0d; }
  frame border:disabled {
    color: rgba(255, 255, 255, 0.3); }
frame label {
  color: #ffffff; }
  frame label:disabled {
    color: rgba(255, 255, 255, 0.3); }

/**************
 * Tree Views *
 **************/
treeview.view {
  border-left-color: #868686;
  border-top-color: #0c0c0d;
  background-color: #18181A; }
  *, .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode button.subtitle:visited, button:link,
  button:visited {
    -GtkTreeView-horizontal-separator: 4;
    -GtkTreeView-grid-line-width: 1;
    -GtkTreeView-grid-line-pattern: '';
    -GtkTreeView-tree-line-width: 1;
    -GtkTreeView-tree-line-pattern: '';
    -GtkTreeView-expander-size: 16; }
  treeview.view:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:focus:link,
  headerbar.selection-mode treeview.view.subtitle:focus:link, treeview.view:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:link,
  headerbar.selection-mode treeview.view.subtitle:link {
    border-radius: 0; }
  treeview.view:selected:backdrop, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:backdrop:link,
  headerbar.selection-mode treeview.view.subtitle:backdrop:link, treeview.view:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:link,
  headerbar.selection-mode treeview.view.subtitle:link {
    border-left-color: #9bc8fd;
    border-top-color: rgba(255, 255, 255, 0.1);
    background-color: #3790FA; }
  treeview.view:disabled {
    color: rgba(255, 255, 255, 0.3); }
    treeview.view:disabled:selected, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:disabled:link,
    headerbar.selection-mode treeview.view.subtitle:disabled:link {
      color: rgba(255, 255, 255, 0.3); }
      treeview.view:disabled:selected:backdrop, .selection-mode.titlebar:not(headerbar) treeview.view.subtitle:disabled:backdrop:link,
      headerbar.selection-mode treeview.view.subtitle:disabled:backdrop:link {
        color: rgba(255, 255, 255, 0.3); }
    treeview.view:disabled:backdrop {
      color: rgba(255, 255, 255, 0.3); }
  treeview.view.separator {
    min-height: 2px;
    color: #0c0c0d; }
    treeview.view.separator:backdrop {
      color: rgba(12, 12, 13, 0.1); }
  treeview.view:backdrop {
    border-left-color: #18181A;
    border-top: #18181A;
    background-color: #18181A;
    color: #ffffff; }
  treeview.view:drop(active) {
    border-style: solid none;
    border-width: 1px;
    border-color: #3790FA; }
    treeview.view:drop(active).after {
      border-top-style: none; }
    treeview.view:drop(active).before {
      border-bottom-style: none; }
  treeview.view.expander {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
    color: #ffffff; }
    treeview.view.expander:dir(rtl) {
      -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl"); }
    treeview.view.expander:hover {
      color: #3790FA; }
    treeview.view.expander:selected, .selection-mode.titlebar:not(headerbar) treeview.view.expander.subtitle:link,
    headerbar.selection-mode treeview.view.expander.subtitle:link {
      color: #fff; }
      treeview.view.expander:selected:hover, .selection-mode.titlebar:not(headerbar) treeview.view.expander.subtitle:hover:link,
      headerbar.selection-mode treeview.view.expander.subtitle:hover:link {
        color: #fff; }
      treeview.view.expander:selected:backdrop, .selection-mode.titlebar:not(headerbar) treeview.view.expander.subtitle:backdrop:link,
      headerbar.selection-mode treeview.view.expander.subtitle:backdrop:link {
        color: #ffffff; }
    treeview.view.expander:checked {
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
    treeview.view.expander:backdrop {
      color: #ffffff; }
  treeview.view.progressbar {
    color: #0c0c0d;
    border: 1px solid #3790FA;
    background-color: #3790FA;
    background-image: linear-gradient(to bottom, #3790FA, #0674f8); }
    treeview.view.progressbar:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.progressbar.subtitle:focus:link,
    headerbar.selection-mode treeview.view.progressbar.subtitle:focus:link, treeview.view.progressbar:selected, .selection-mode.titlebar:not(headerbar) treeview.view.progressbar.subtitle:link,
    headerbar.selection-mode treeview.view.progressbar.subtitle:link {
      border-radius: 0px;
      color: #3790FA;
      box-shadow: none;
      background-image: linear-gradient(to bottom, #0c0c0d, black); }
      treeview.view.progressbar:selected:focus:backdrop, .selection-mode.titlebar:not(headerbar) treeview.view.progressbar.subtitle:focus:backdrop:link,
      headerbar.selection-mode treeview.view.progressbar.subtitle:focus:backdrop:link, treeview.view.progressbar:selected:backdrop, .selection-mode.titlebar:not(headerbar) treeview.view.progressbar.subtitle:backdrop:link,
      headerbar.selection-mode treeview.view.progressbar.subtitle:backdrop:link {
        color: #3790FA;
        border-color: #3790FA;
        background-color: #18181A; }
    treeview.view.progressbar:backdrop {
      color: #0c0c0d;
      background-image: none;
      box-shadow: none; }
  treeview.view.trough {
    background-color: rgba(255, 255, 255, 0.1);
    border-radius: 0px; }
    treeview.view.trough:selected:focus, .selection-mode.titlebar:not(headerbar) treeview.view.trough.subtitle:focus:link,
    headerbar.selection-mode treeview.view.trough.subtitle:focus:link, treeview.view.trough:selected, .selection-mode.titlebar:not(headerbar) treeview.view.trough.subtitle:link,
    headerbar.selection-mode treeview.view.trough.subtitle:link {
      background-color: rgba(255, 255, 255, 0.3);
      border-width: 1px 0;
      border-style: solid;
      border-color: #3790FA; }
  treeview.view header button {
    color: #ffffff;
    background-color: #0c0c0d;
    font-weight: bold;
    text-shadow: none;
    box-shadow: none; }
    treeview.view header button:hover {
      color: white;
      box-shadow: none;
      transition: none; }
    treeview.view header button:active {
      color: #ffffff;
      transition: none; }
  treeview.view button.dnd:active, treeview.view button.dnd:selected, treeview.view .selection-mode.titlebar:not(headerbar) button.dnd.subtitle:link, .selection-mode.titlebar:not(headerbar) treeview.view button.dnd.subtitle:link,
  treeview.view .selection-mode.titlebar:not(headerbar) button.dnd.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) treeview.view button.dnd.subtitle:visited,
  treeview.view headerbar.selection-mode button.dnd.subtitle:link,
  headerbar.selection-mode treeview.view button.dnd.subtitle:link,
  treeview.view headerbar.selection-mode button.dnd.subtitle:visited,
  headerbar.selection-mode treeview.view button.dnd.subtitle:visited, treeview.view button.dnd:hover, treeview.view button.dnd,
  treeview.view header.button.dnd:active,
  treeview.view header.button.dnd:selected,
  treeview.view .selection-mode.titlebar:not(headerbar) header.button.dnd.subtitle:link,
  .selection-mode.titlebar:not(headerbar) treeview.view header.button.dnd.subtitle:link,
  treeview.view headerbar.selection-mode header.button.dnd.subtitle:link,
  headerbar.selection-mode treeview.view header.button.dnd.subtitle:link,
  treeview.view header.button.dnd:hover,
  treeview.view header.button.dnd {
    padding: 0 6px;
    color: #0c0c0d;
    background-image: none;
    background-color: #3790FA;
    border-style: none;
    border-radius: 0;
    text-shadow: none;
    transition: none; }
  treeview.view acceleditor > label {
    background-color: #3790FA; }

treeview.view header button, treeview.view header button:hover, treeview.view header button:active {
  padding: 0 6px;
  background-image: none;
  border-style: none solid solid none;
  border-color: #0c0c0d;
  border-radius: 0;
  text-shadow: none; }
  treeview.view header button:disabled {
    border-color: rgba(255, 255, 255, 0.3);
    background-image: none; }
  treeview.view header button:backdrop {
    color: #868686;
    border-color: #0c0c0d;
    border-style: none solid solid none;
    background-image: none;
    background-color: #0c0c0d; }
    treeview.view header button:backdrop:disabled {
      border-color: #1C1C1E;
      background-image: none; }
  toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.sidebar-button:backdrop, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button:backdrop, toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.titlebutton:backdrop, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.titlebutton:backdrop, toolbar.inline-toolbar treeview.view header .selection-mode toolbutton:last-child > button.titlebutton:backdrop, toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.flat:backdrop, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.flat:backdrop, combobox.linked treeview.view header button:backdrop:nth-child(2):dir(ltr), treeview.view header combobox.linked button:backdrop:nth-child(2):dir(ltr), treeview.view header .linked:not(.vertical) > combobox:last-child > box > button.combo:backdrop, treeview.view header .linked.vertical > combobox:last-child > box > button.combo:backdrop, treeview.view header button:last-child:backdrop, treeview.view header toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton:backdrop, toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.sidebar-button, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.sidebar-button, toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.titlebutton, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.titlebutton, toolbar.inline-toolbar treeview.view header .selection-mode toolbutton:last-child > button.titlebutton, toolbar.inline-toolbar treeview.view header toolbutton:last-child > button.flat, treeview.view header toolbar.inline-toolbar toolbutton:last-child > button.flat, combobox.linked treeview.view header button:nth-child(2):dir(ltr), treeview.view header combobox.linked button:nth-child(2):dir(ltr), treeview.view header .linked:not(.vertical) > combobox:last-child > box > button.combo, treeview.view header .linked.vertical > combobox:last-child > box > button.combo, treeview.view header button:last-child, treeview.view header toolbar.inline-toolbar .selection-mode toolbutton:last-child > button.titlebutton {
    border-right-style: none; }

scrolledwindow {
  border-style: solid; }

/*************************************
 * titlebar - for generic window
 * headerbar - for several applications such as gedit and gnome-calculator
 *****************************************/
.titlebar, headerbar {
  color: #ffffff;
  background-image: image(#0c0c0d); }
  .titlebar:backdrop, label:backdrop selection.titlebar, progressbar:backdrop trough.titlebar, progressbar:backdrop progress.titlebar, headerbar:backdrop {
    color: #ffffff;
    background-image: image(#0c0c0d); }
  .titlebar button.titlebutton, headerbar button.titlebutton {
    border-radius: 6px;
    color: #ffffff;
    background-image: image(#393A3E);
    border-width: 0px;
    /* maximize and minimize close */
    /* close button */ }
    .titlebar button.titlebutton:hover, .titlebar button.titlebutton:backdrop:hover, headerbar button.titlebutton:hover, headerbar button.titlebutton:backdrop:hover {
      color: #fff;
      background-image: image(#545d64); }
    .titlebar button.titlebutton:active, .titlebar button.titlebutton:backdrop:active, headerbar button.titlebutton:active, headerbar button.titlebutton:backdrop:active {
      color: #fff;
      background-image: image(#434d55); }
    .titlebar button.titlebutton:backdrop, headerbar button.titlebutton:backdrop {
      color: #ffffff;
      background-image: image(#0c0c0d); }
    .titlebar button.titlebutton.close:hover, headerbar button.titlebutton.close:hover {
      background-image: image(#d42320); }
    .titlebar button.titlebutton.close:active, headerbar button.titlebutton.close:active {
      background-image: image(#ac4844); }

/****************
 * Toolbars*
 ***************/
toolbar, .inline-toolbar, searchbar,
.location-bar {
  -GtkWidget-window-dragging: true;
  padding: 4px;
  background: #2D2E32; }

toolbar {
  padding: 4px 3px 3px 4px; }
  .osd toolbar {
    background-color: transparent; }
  toolbar.osd {
    padding: 13px;
    border: none;
    border-radius: 5px;
    background-color: rgba(28, 31, 32, 0.7); }
    toolbar.osd.left, toolbar.osd.right, toolbar.osd.top, toolbar.osd.bottom {
      border-radius: 0; }
  toolbar button {
    box-shadow: none;
    border-radius: 4px;
    color: #fff;
    background-image: image(#2D2E32); }
  toolbar.horizontal separator {
    margin: 0 7px 1px 6px; }
  toolbar.vertical separator {
    margin: 6px 1px 7px 0; }
  toolbar:not(.inline-toolbar):not(.osd) switch,
  toolbar:not(.inline-toolbar):not(.osd) scale,
  toolbar:not(.inline-toolbar):not(.osd) entry,
  toolbar:not(.inline-toolbar):not(.osd) spinbutton,
  toolbar:not(.inline-toolbar):not(.osd) button {
    margin-right: 1px; }

.inline-toolbar {
  padding: 3px;
  border-width: 1px;
  border-radius: 0  0 5px 5px; }

searchbar,
.location-bar {
  border-width: 1px;
  padding: 3px;
  border-color: #696969; }

.inline-toolbar, searchbar,
.location-bar {
  border-style: solid;
  border-color: #696969;
  background-color: #2D2E32; }
  label:backdrop selection.inline-toolbar,
  label:backdrop selection.location-bar, progressbar:backdrop trough.inline-toolbar,
  progressbar:backdrop trough.location-bar, progressbar:backdrop progress.inline-toolbar,
  progressbar:backdrop progress.location-bar, .inline-toolbar:backdrop, searchbar:backdrop,
  .location-bar:backdrop {
    color: #ffffff;
    border-color: #696969;
    background-color: #2D2E32;
    box-shadow: none;
    transition: 200ms ease-out; }

/*****************
 * Separator*
 *****************/
separator {
  margin: 3px; }

/*********
 * Links *
 *********/
button:link > label, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label,
headerbar.selection-mode button.subtitle:link > label,
button:visited > label, *:link, .selection-mode.titlebar:not(headerbar) .subtitle:link,
.selection-mode.titlebar:not(headerbar) button.subtitle:visited,
headerbar.selection-mode .subtitle:link,
headerbar.selection-mode button.subtitle:visited, button:link,
button:visited {
  color: #ffffff; }
  button:link > label:visited, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:visited,
  headerbar.selection-mode button.subtitle:link > label:visited,
  button:visited > label:visited, *:link:visited, .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  headerbar.selection-mode .subtitle:visited:link,
  headerbar.selection-mode button.subtitle:visited,
  button:visited {
    color: #ffffff; }
    *:selected button:link > label:visited, .selection-mode.titlebar:not(headerbar) .subtitle:link button:link > label:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:link > label:visited,
    headerbar.selection-mode .subtitle:link button:link > label:visited,
    headerbar.selection-mode button.subtitle:visited button:link > label:visited, button:selected:link button:link > label:visited,
    button:selected:visited button:link > label:visited, .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:link > label:visited, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:visited,
    headerbar.selection-mode button.subtitle:link > label:visited,
    *:selected button:visited > label:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited > label:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited > label:visited,
    headerbar.selection-mode .subtitle:link button:visited > label:visited,
    headerbar.selection-mode button.subtitle:visited button:visited > label:visited,
    button:selected:link button:visited > label:visited,
    button:selected:visited button:visited > label:visited,
    *:selected *:link:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited *:link:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link *:link:visited,
    headerbar.selection-mode .subtitle:link *:link:visited,
    headerbar.selection-mode button.subtitle:visited *:link:visited,
    button:selected:link *:link:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link *:link:visited,
    headerbar.selection-mode button.subtitle:link *:link:visited,
    button:selected:visited *:link:visited,
    *:selected .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) *:selected .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:visited:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:visited:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:visited:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:visited:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:visited:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited .subtitle:visited:link,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:visited,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:visited,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:visited,
    *:selected headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode *:selected .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:visited:link,
    headerbar.selection-mode .subtitle:link .subtitle:visited:link,
    headerbar.selection-mode button.subtitle:visited .subtitle:visited:link,
    button:selected:link headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode button:selected:link .subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:visited:link,
    headerbar.selection-mode button.subtitle:link .subtitle:visited:link,
    button:selected:visited headerbar.selection-mode .subtitle:visited:link,
    headerbar.selection-mode button:selected:visited .subtitle:visited:link,
    *:selected headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode *:selected button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
    headerbar.selection-mode .subtitle:link button.subtitle:visited,
    headerbar.selection-mode button.subtitle:visited button.subtitle:visited,
    button:selected:link headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode button:selected:link button.subtitle:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited,
    headerbar.selection-mode button.subtitle:link button.subtitle:visited,
    button:selected:visited headerbar.selection-mode button.subtitle:visited,
    headerbar.selection-mode button:selected:visited button.subtitle:visited,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:visited:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:visited:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:visited:link,
    *:selected headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode *:selected button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode .subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode button.subtitle:visited button.subtitle:visited:link,
    button:selected:link headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode button:selected:link button.subtitle:visited:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited:link,
    headerbar.selection-mode button.subtitle:link button.subtitle:visited:link,
    button:selected:visited headerbar.selection-mode button.subtitle:visited:link,
    headerbar.selection-mode button:selected:visited button.subtitle:visited:link,
    *:selected button:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:visited,
    headerbar.selection-mode .subtitle:link button:visited,
    headerbar.selection-mode button.subtitle:visited button:visited,
    button:selected:link button:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button:visited,
    headerbar.selection-mode button.subtitle:link button:visited,
    button:selected:visited button:visited {
      color: #fff; }
  button:link > label:hover, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:hover,
  headerbar.selection-mode button.subtitle:link > label:hover,
  button:visited > label:hover, *:link:hover, .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
  headerbar.selection-mode .subtitle:hover:link,
  headerbar.selection-mode button.subtitle:hover:visited, button:hover:link,
  button:hover:visited {
    color: #fff; }
    *:selected button:link > label:hover, .selection-mode.titlebar:not(headerbar) .subtitle:link button:link > label:hover,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:link > label:hover,
    headerbar.selection-mode .subtitle:link button:link > label:hover,
    headerbar.selection-mode button.subtitle:visited button:link > label:hover, button:selected:link button:link > label:hover,
    button:selected:visited button:link > label:hover, .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:link > label:hover, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:hover,
    headerbar.selection-mode button.subtitle:link > label:hover,
    *:selected button:visited > label:hover,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited > label:hover,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited > label:hover,
    headerbar.selection-mode .subtitle:link button:visited > label:hover,
    headerbar.selection-mode button.subtitle:visited button:visited > label:hover,
    button:selected:link button:visited > label:hover,
    button:selected:visited button:visited > label:hover,
    *:selected *:link:hover,
    .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:hover,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited *:link:hover,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:hover,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link *:link:hover,
    headerbar.selection-mode .subtitle:link *:link:hover,
    headerbar.selection-mode button.subtitle:visited *:link:hover,
    button:selected:link *:link:hover,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link *:link:hover,
    headerbar.selection-mode button.subtitle:link *:link:hover,
    button:selected:visited *:link:hover,
    *:selected .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) *:selected .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:hover:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:hover:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:hover:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:hover:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:hover:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited .subtitle:hover:link,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:hover:visited,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:hover:visited,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:hover:visited,
    *:selected headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode *:selected .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:hover:link,
    headerbar.selection-mode .subtitle:link .subtitle:hover:link,
    headerbar.selection-mode button.subtitle:visited .subtitle:hover:link,
    button:selected:link headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode button:selected:link .subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:hover:link,
    headerbar.selection-mode button.subtitle:link .subtitle:hover:link,
    button:selected:visited headerbar.selection-mode .subtitle:hover:link,
    headerbar.selection-mode button:selected:visited .subtitle:hover:link,
    *:selected headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode *:selected button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode .subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode button.subtitle:visited button.subtitle:hover:visited,
    button:selected:link headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode button:selected:link button.subtitle:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:hover:visited,
    headerbar.selection-mode button.subtitle:link button.subtitle:hover:visited,
    button:selected:visited headerbar.selection-mode button.subtitle:hover:visited,
    headerbar.selection-mode button:selected:visited button.subtitle:hover:visited,
    *:selected button:hover:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:hover:link,
    headerbar.selection-mode .subtitle:link button:hover:link,
    headerbar.selection-mode button.subtitle:visited button:hover:link,
    button:selected:link button:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button:hover:link,
    headerbar.selection-mode button.subtitle:link button:hover:link,
    button:selected:visited button:hover:link,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:hover:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:hover:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:hover:link,
    *:selected headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode *:selected button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode .subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode button.subtitle:visited button.subtitle:hover:link,
    button:selected:link headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode button:selected:link button.subtitle:hover:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:hover:link,
    headerbar.selection-mode button.subtitle:link button.subtitle:hover:link,
    button:selected:visited headerbar.selection-mode button.subtitle:hover:link,
    headerbar.selection-mode button:selected:visited button.subtitle:hover:link,
    *:selected button:hover:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:hover:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:hover:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:hover:visited,
    headerbar.selection-mode .subtitle:link button:hover:visited,
    headerbar.selection-mode button.subtitle:visited button:hover:visited,
    button:selected:link button:hover:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button:hover:visited,
    headerbar.selection-mode button.subtitle:link button:hover:visited,
    button:selected:visited button:hover:visited {
      color: #fff; }
  button:link > label:active, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:active,
  headerbar.selection-mode button.subtitle:link > label:active,
  button:visited > label:active, *:link:active, .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
  headerbar.selection-mode .subtitle:active:link,
  headerbar.selection-mode button.subtitle:active:visited, button:active:link,
  button:active:visited {
    color: #fff; }
    *:selected button:link > label:active, .selection-mode.titlebar:not(headerbar) .subtitle:link button:link > label:active,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:link > label:active,
    headerbar.selection-mode .subtitle:link button:link > label:active,
    headerbar.selection-mode button.subtitle:visited button:link > label:active, button:selected:link button:link > label:active,
    button:selected:visited button:link > label:active, .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:link > label:active, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:active,
    headerbar.selection-mode button.subtitle:link > label:active,
    *:selected button:visited > label:active,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited > label:active,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited > label:active,
    headerbar.selection-mode .subtitle:link button:visited > label:active,
    headerbar.selection-mode button.subtitle:visited button:visited > label:active,
    button:selected:link button:visited > label:active,
    button:selected:visited button:visited > label:active,
    *:selected *:link:active,
    .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:active,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited *:link:active,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link *:link:active,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link *:link:active,
    headerbar.selection-mode .subtitle:link *:link:active,
    headerbar.selection-mode button.subtitle:visited *:link:active,
    button:selected:link *:link:active,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link *:link:active,
    headerbar.selection-mode button.subtitle:link *:link:active,
    button:selected:visited *:link:active,
    *:selected .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) *:selected .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:active:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:active:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:active:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:active:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:active:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited .subtitle:active:link,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:active:visited,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:active:visited,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:active:visited,
    *:selected headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode *:selected .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:active:link,
    headerbar.selection-mode .subtitle:link .subtitle:active:link,
    headerbar.selection-mode button.subtitle:visited .subtitle:active:link,
    button:selected:link headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode button:selected:link .subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:active:link,
    headerbar.selection-mode button.subtitle:link .subtitle:active:link,
    button:selected:visited headerbar.selection-mode .subtitle:active:link,
    headerbar.selection-mode button:selected:visited .subtitle:active:link,
    *:selected headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode *:selected button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode .subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode button.subtitle:visited button.subtitle:active:visited,
    button:selected:link headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode button:selected:link button.subtitle:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:active:visited,
    headerbar.selection-mode button.subtitle:link button.subtitle:active:visited,
    button:selected:visited headerbar.selection-mode button.subtitle:active:visited,
    headerbar.selection-mode button:selected:visited button.subtitle:active:visited,
    *:selected button:active:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:active:link,
    headerbar.selection-mode .subtitle:link button:active:link,
    headerbar.selection-mode button.subtitle:visited button:active:link,
    button:selected:link button:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button:active:link,
    headerbar.selection-mode button.subtitle:link button:active:link,
    button:selected:visited button:active:link,
    *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:link,
    headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:link,
    headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:active:link,
    button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:active:link,
    headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:active:link,
    button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:active:link,
    *:selected headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode *:selected button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:active:link,
    headerbar.selection-mode .subtitle:link button.subtitle:active:link,
    headerbar.selection-mode button.subtitle:visited button.subtitle:active:link,
    button:selected:link headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode button:selected:link button.subtitle:active:link,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:active:link,
    headerbar.selection-mode button.subtitle:link button.subtitle:active:link,
    button:selected:visited headerbar.selection-mode button.subtitle:active:link,
    headerbar.selection-mode button:selected:visited button.subtitle:active:link,
    *:selected button:active:visited,
    .selection-mode.titlebar:not(headerbar) .subtitle:link button:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:active:visited,
    headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:active:visited,
    .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:active:visited,
    headerbar.selection-mode .subtitle:link button:active:visited,
    headerbar.selection-mode button.subtitle:visited button:active:visited,
    button:selected:link button:active:visited,
    .selection-mode.titlebar:not(headerbar) button.subtitle:link button:active:visited,
    headerbar.selection-mode button.subtitle:link button:active:visited,
    button:selected:visited button:active:visited {
      color: #fff; }
  button:link > label:backdrop:backdrop:hover, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:backdrop:backdrop:hover,
  headerbar.selection-mode button.subtitle:link > label:backdrop:backdrop:hover,
  button:visited > label:backdrop:backdrop:hover, .selection-mode.titlebar:not(headerbar) button:link > label.subtitle:backdrop:backdrop:hover:link,
  .selection-mode.titlebar:not(headerbar) button:visited > label.subtitle:backdrop:backdrop:hover:link,
  headerbar.selection-mode button:link > label.subtitle:backdrop:backdrop:hover:link,
  headerbar.selection-mode button:visited > label.subtitle:backdrop:backdrop:hover:link, button:link > label:backdrop:backdrop:hover:selected, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:backdrop:backdrop:hover:selected,
  headerbar.selection-mode button.subtitle:link > label:backdrop:backdrop:hover:selected,
  button:visited > label:backdrop:backdrop:hover:selected, button:link > label:backdrop, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:backdrop,
  headerbar.selection-mode button.subtitle:link > label:backdrop,
  button:visited > label:backdrop,
  *:link:backdrop:backdrop:hover,
  label:backdrop selection:link:hover,
  label:backdrop .selection-mode.titlebar:not(headerbar) selection.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) label:backdrop selection.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:backdrop:backdrop:hover:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.subtitle:hover:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.subtitle:hover:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.subtitle:hover:link,
  headerbar.selection-mode .subtitle:backdrop:backdrop:hover:link,
  progressbar:backdrop trough:link:hover,
  progressbar:backdrop progress:link:hover,
  button:backdrop:backdrop:hover:link,
  button:backdrop:backdrop:hover:visited,
  *:link:backdrop:backdrop:hover:selected,
  label:backdrop selection:link:hover:selected,
  label:backdrop .selection-mode.titlebar:not(headerbar) selection.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) label:backdrop selection.subtitle:hover:link,
  label:backdrop headerbar.selection-mode selection.subtitle:hover:link,
  headerbar.selection-mode label:backdrop selection.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:backdrop:backdrop:hover:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.subtitle:hover:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.subtitle:hover:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.subtitle:hover:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:backdrop:backdrop:hover:visited,
  headerbar.selection-mode .subtitle:backdrop:backdrop:hover:link,
  headerbar.selection-mode progressbar:backdrop trough.subtitle:hover:link,
  progressbar:backdrop headerbar.selection-mode trough.subtitle:hover:link,
  headerbar.selection-mode progressbar:backdrop progress.subtitle:hover:link,
  progressbar:backdrop headerbar.selection-mode progress.subtitle:hover:link,
  headerbar.selection-mode button.subtitle:backdrop:backdrop:hover:visited,
  progressbar:backdrop trough:link:hover:selected,
  progressbar:backdrop progress:link:hover:selected,
  button:backdrop:backdrop:hover:selected:link,
  button:backdrop:backdrop:hover:selected:visited,
  *:link:backdrop,
  label:backdrop selection:link,
  label:backdrop .selection-mode.titlebar:not(headerbar) selection.subtitle:link,
  .selection-mode.titlebar:not(headerbar) label:backdrop selection.subtitle:link,
  label:backdrop headerbar.selection-mode selection.subtitle:link,
  headerbar.selection-mode label:backdrop selection.subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:backdrop:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop trough.subtitle:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) trough.subtitle:link,
  .selection-mode.titlebar:not(headerbar) progressbar:backdrop progress.subtitle:link,
  progressbar:backdrop .selection-mode.titlebar:not(headerbar) progress.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:backdrop:visited,
  headerbar.selection-mode .subtitle:backdrop:link,
  headerbar.selection-mode progressbar:backdrop trough.subtitle:link,
  progressbar:backdrop headerbar.selection-mode trough.subtitle:link,
  headerbar.selection-mode progressbar:backdrop progress.subtitle:link,
  progressbar:backdrop headerbar.selection-mode progress.subtitle:link,
  headerbar.selection-mode button.subtitle:backdrop:visited,
  progressbar:backdrop trough:link,
  progressbar:backdrop progress:link,
  button:backdrop:link,
  button:backdrop:visited {
    color: #3790FA; }
  infobar.info :link, infobar.info .selection-mode.titlebar:not(headerbar) .subtitle:link, .selection-mode.titlebar:not(headerbar) infobar.info .subtitle:link,
  infobar.info headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode infobar.info .subtitle:link,
  infobar.info button:visited, infobar.question :link, infobar.question .selection-mode.titlebar:not(headerbar) .subtitle:link, .selection-mode.titlebar:not(headerbar) infobar.question .subtitle:link,
  infobar.question headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode infobar.question .subtitle:link,
  infobar.question button:visited, infobar.warning :link, infobar.warning .selection-mode.titlebar:not(headerbar) .subtitle:link, .selection-mode.titlebar:not(headerbar) infobar.warning .subtitle:link,
  infobar.warning headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode infobar.warning .subtitle:link,
  infobar.warning button:visited, infobar.error :link, infobar.error .selection-mode.titlebar:not(headerbar) .subtitle:link, .selection-mode.titlebar:not(headerbar) infobar.error .subtitle:link,
  infobar.error headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode infobar.error .subtitle:link,
  infobar.error button:visited, .selection-mode.titlebar:not(headerbar) button:link > label.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button:visited > label.subtitle:link,
  headerbar.selection-mode button:link > label.subtitle:link,
  headerbar.selection-mode button:visited > label.subtitle:link, button:link > label:selected, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label:selected,
  headerbar.selection-mode button.subtitle:link > label:selected,
  button:visited > label:selected, *:selected button:link > label, .selection-mode.titlebar:not(headerbar) .subtitle:link button:link > label,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:link > label,
  headerbar.selection-mode .subtitle:link button:link > label,
  headerbar.selection-mode button.subtitle:visited button:link > label, button:selected:link button:link > label,
  button:selected:visited button:link > label, .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:link > label, .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:link > label, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label,
  .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:link > label,
  headerbar.selection-mode *:selected button.subtitle:link > label,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link > label,
  headerbar.selection-mode button.subtitle:link > label,
  *:selected button:visited > label,
  .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited > label,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited > label,
  headerbar.selection-mode .subtitle:link button:visited > label,
  headerbar.selection-mode button.subtitle:visited button:visited > label,
  button:selected:link button:visited > label,
  button:selected:visited button:visited > label,
  *:link:selected,
  .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode button.subtitle:visited,
  button:selected:link,
  button:selected:visited,
  *:selected *:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link *:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited *:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link *:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link *:link,
  headerbar.selection-mode .subtitle:link *:link,
  headerbar.selection-mode button.subtitle:visited *:link,
  button:selected:link *:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link *:link,
  headerbar.selection-mode button.subtitle:link *:link,
  button:selected:visited *:link,
  *:selected .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) *:selected .subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:link,
  button:selected:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button:selected:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:link,
  button:selected:visited .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button:selected:visited .subtitle:link,
  *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
  headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
  headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:visited,
  button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited,
  headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:visited,
  button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:visited,
  *:selected headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) *:selected .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:link,
  button:selected:link headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button:selected:link .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:link,
  button:selected:visited headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button:selected:visited .subtitle:link,
  *:selected .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode *:selected .subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited .subtitle:link,
  button:selected:link .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button:selected:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link .subtitle:link,
  button:selected:visited .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button:selected:visited .subtitle:link,
  *:selected headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode *:selected .subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link .subtitle:link,
  headerbar.selection-mode .subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:visited .subtitle:link,
  button:selected:link headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode button:selected:link .subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link .subtitle:link,
  headerbar.selection-mode button.subtitle:link .subtitle:link,
  button:selected:visited headerbar.selection-mode .subtitle:link,
  headerbar.selection-mode button:selected:visited .subtitle:link,
  *:selected headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode *:selected button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:visited,
  headerbar.selection-mode .subtitle:link button.subtitle:visited,
  headerbar.selection-mode button.subtitle:visited button.subtitle:visited,
  button:selected:link headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode button:selected:link button.subtitle:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:visited,
  headerbar.selection-mode button.subtitle:link button.subtitle:visited,
  button:selected:visited headerbar.selection-mode button.subtitle:visited,
  headerbar.selection-mode button:selected:visited button.subtitle:visited,
  *:selected button:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link button:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:link,
  headerbar.selection-mode .subtitle:link button:link,
  headerbar.selection-mode button.subtitle:visited button:link,
  button:selected:link button:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link button:link,
  headerbar.selection-mode button.subtitle:link button:link,
  button:selected:visited button:link,
  *:selected .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) *:selected button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:link,
  headerbar.selection-mode .subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:link,
  headerbar.selection-mode button.subtitle:visited .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:visited button.subtitle:link,
  button:selected:link .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button:selected:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:link,
  headerbar.selection-mode button.subtitle:link .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode button.subtitle:link button.subtitle:link,
  button:selected:visited .selection-mode.titlebar:not(headerbar) button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button:selected:visited button.subtitle:link,
  *:selected headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode *:selected button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) .subtitle:link headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:visited button.subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button.subtitle:link,
  headerbar.selection-mode .subtitle:link button.subtitle:link,
  headerbar.selection-mode button.subtitle:visited button.subtitle:link,
  button:selected:link headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode button:selected:link button.subtitle:link,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) button.subtitle:link button.subtitle:link,
  headerbar.selection-mode button.subtitle:link button.subtitle:link,
  button:selected:visited headerbar.selection-mode button.subtitle:link,
  headerbar.selection-mode button:selected:visited button.subtitle:link,
  *:selected button:visited,
  .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:visited button:visited,
  headerbar.selection-mode .selection-mode.titlebar:not(headerbar) .subtitle:link button:visited,
  .selection-mode.titlebar:not(headerbar) headerbar.selection-mode .subtitle:link button:visited,
  headerbar.selection-mode .subtitle:link button:visited,
  headerbar.selection-mode button.subtitle:visited button:visited,
  button:selected:link button:visited,
  .selection-mode.titlebar:not(headerbar) button.subtitle:link button:visited,
  headerbar.selection-mode button.subtitle:link button:visited,
  button:selected:visited button:visited {
    color: #d7e9fe; }

button:link, .selection-mode.titlebar:not(headerbar) button.subtitle:link,
headerbar.selection-mode button.subtitle:link,
button:visited,
button:visited {
  text-shadow: none; }
  button:link:hover, .selection-mode.titlebar:not(headerbar) button.subtitle:hover:link,
  headerbar.selection-mode button.subtitle:hover:link,
  button:hover:visited, button:link:active, .selection-mode.titlebar:not(headerbar) button.subtitle:active:link,
  headerbar.selection-mode button.subtitle:active:link,
  button:active:visited, button:link:checked, .selection-mode.titlebar:not(headerbar) button.subtitle:checked:link,
  headerbar.selection-mode button.subtitle:checked:link,
  button:checked:visited,
  button:visited:hover,
  button:visited:active,
  button:visited:checked {
    text-shadow: none; }
  button:link > label, .selection-mode.titlebar:not(headerbar) button.subtitle:link > label,
  headerbar.selection-mode button.subtitle:link > label,
  button:visited > label,
  button:visited > label {
    text-decoration-line: underline; }

/*********
 * Emoji *
 ********/
popover.emoji-picker {
  padding-left: 0;
  padding-right: 0; }

button.emoji-section {
  border-color: transparent;
  border-width: 3px;
  border-style: none none solid;
  border-radius: 0;
  margin: 2px 4px 2px 4px;
  padding: 3px 0 0;
  min-width: 32px;
  min-height: 28px;
  background: none;
  box-shadow: none;
  text-shadow: none;
  outline-offset: -5px; }
  button.emoji-section:backdrop:not(:checked) {
    border-color: transparent; }
  button.emoji-section:hover {
    border-color: #0c0c0d; }
  button.emoji-section:checked {
    border-color: #3790FA; }
  button.emoji-section label {
    padding: 0;
    opacity: 0.55; }
  button.emoji-section:hover label {
    opacity: 0.775; }
  button.emoji-section:checked label {
    opacity: 1; }

.emoji {
  font-size: x-large;
  padding: 6px;
  border-radius: 6px; }
  .emoji :hover {
    background: #3790FA; }

/*# sourceMappingURL=gtk.css.map */
